/* routine called at end of iteration when advection is turned on */
void DynaEndIter(void);

/* DynaStartZone called at end of iteration when advection is turned on */
void DynaStartZone(void);

/* DynaIonize, called from ionize to evaluate advective terms for current conditions */
void DynaIonize(void);

/* DynaChangeDensity, called from PressureChange to evaluate new density needed for
 * current conditions and wind solution, returns ratio of new to old density,
 * called when wind velocity is negative for D-critical solutions  */
double DynaChangeDensity(void);

/* all of these are initialized in zero */
EXTERN struct t_advec 
{

	/* is advection turned on ?, set to FALSE in zero */
	int lgAdvection;

	/* rate of advection H neutral ionization */
	double AdvecHIon;

	/* velocity that is assumed, constant for now, obtained from advection command */
	double Velocity;

	/* timestep, as derived in DynaEndIter */
	double timestep;

	/* advected work */
	double work;

	/* advective cooling minus heating (erg cm^-3 s^-1) */
	double CoolHeat, dCoolHeatdT;

	/* largest fraction of cooling and heating */
	double CoolMax, HeatMax;

	/* upstream HI density */
	double HAtom;

	/* the upstream grams per unit vol*/
	double pdenstr;

	/* the recombination term due to advection */
	double HRecom;

	/* the advection photo rate (s^-1) */
	double Photo;

	/* the interpolated upstream densities of all ionization stages */
	double UpstreamIon[LIMELM+2][LIMELM];

	/* the advective recombination rate (s^-1)*/
	double Recomb[LIMELM+2][LIMELM];

}	dynamics;
