/* HeRecom - do recomb coef for He, called by HeLike */
/*static void HeRecom( */
/* evaluate collisional rates 
static void HeCollid( long ipZ);*/
/* evaluate photoionization rates  
static void HePhoto( long ipZ);*/
/* evaluate photoionization rates  
static void HeLevel( long ipZ);*/
/* create he-like series, called by ContCreatePointers
void HeCreate(void) */
/* HeCSInterp interpolate on He1 collision strengths 
static float HeCSInterp( long ipZ , long ipHi , long ipLo );*/

/* 
the following block of defines occurs in cddefines.c, and echoed in cddefines.h 
const int ipHe1s1S = 0;
const int ipHe2s3S = 1;
const int ipHe2s1S = 2;
const int ipHe2p3P = 3;
const int ipHe2p1P = 4;
*/

/* temperatures at which CS for He1 are stored */
#	define HE1CSARRAY 10
static double CSTemp[HE1CSARRAY];

#include <float.h> 
#include "cddefines.h" 
#include "physconst.h" 
#include "nhe1lvl.h"
#include "dynamics.h"
#include "he1rb.h"
#include "herec.h"
#include "phe1lv.h"
#include "getgf.h"
#include "refindex.h"
#include "otsmin.h"
#include "taulines.h"
#include "linpack.h"
#include "receff.h"
#include "abscf.h"
#include "path.h"
#include "ph1com.h"
#include "veclib.h"
#include "hydrogenic.h"
#include "rfield.h"
#include "negcon.h"
#include "hphoto.h"
#include "typmatrx.h"
#include "supprs.h"
#include "trace.h"
#include "destcrt.h"
#include "heavy.h"
#include "drfe.h"
#include "cfit.h"
#include "collionrate.h"
#include "ionfracs.h"
#include "gionrc.h"
#include "opacity.h"
#include "gammas.h"
#include "yield.h"
#include "secondaries.h"
#include "chargtran.h"
#include "heat.h"
#include "phycon.h"
#include "rrfit.h"
#include "rcota.h"
#include "hmi.h"
#include "photrate.h"
#include "recomrate.h"
#include "colion.h"
#include "abundances.h"
#include "ionrange.h"
#include "helike.h" 
/* Function he_assign and modified he_assign assigns, in order of increasing energy, 	*/
/* quantum numbers n,l, and s to each element of vector he_levels.	*/ 
static void org_assign(void);
/* Passed to function loop is the index of a lower level.	*/
/* Since the levels are indexed in increasing energy order,	*/
/* the upper level of a transition must have a higher index.	*/
static void loop(long f);

/* compute quantum defect */
static double defect(
		Elevels L_ptr,
		long Z1
		);
 
static double scqdri( 
              double nstar,
              long int l, 
              double npstar, 
              long int lp, 
              double iz 
              ); 

static double AngerJ( 
              double v, 
              double z 
              ); 
static double Jint( 
            double theta 
            ); 

static double he_energy(
		double nistar, 
		double nfstar,
		long Z1,
		long s,
		long li,
		long ipHi , 
		long ipLo 
		);

		double forbidden(
		Elevels Li,
		long Z1
		);
 /* compute energy diffference in wn and Aul for given line
  * return is 0 for success, 1 for failure */
static void he_1trans(  
			  /* charge on the C scale, 1 is helium */
			  long ipZ , 
			  /* structure giving n, l, s of upper level */
			  Elevels he_upper , 
			  /* structure giving n, l, s of lower level */
			  Elevels he_lower , 
			  /* energy difference in wavenumber */
			  double *Enerwn , 
			  /* transition probability */
			  double *Aul ,
			  long ipHi , long ipLo );

#define       hbar          6.5821220e-16       /* eV sec */

/* Bohr Radius in meters */
/* 0.529177249e-10 meters */
#define BORHRADIUS_METERS      0.529177249e-10

/* Bohr Radius in atomic units*/
/* 1 au */
/* 1 atomic unit */
#define BORHRADIUS_AU      1.0

/* 2/Pi */
#define TWOBYPI     0.63661977237
 
/* 
	AngerJ.c 
	Here we impliment the Anger J_v(x) functions 
 
				1 
J_v(z) = ---- Integral[ cos[vx - z sin[x]] , {x,0,PI} ] 
			   Pi 
 */
 
#define AA(I_,J_)       (*(A+(J_)*(NDIM)+(I_))) 
#define y(I_)           (*(yVector+(I_))) 
#define SS(I_,J_)       (*(S+(J_)*(SDIM)+(I_))) 
#define ipivector(I_)   (*(ipiv+(I_))) 
 
/******************** Program Modifying Defines ***********************/ 
#ifndef  REPLACE_GROUND_STATE 
#define  REPLACE_GROUND_STATE 0 
#endif /* REPLACE_GROUND_STATE */ 
/**********************************************************************/ 

static double vv; 
static double zz; 
static long Z1,maxn,nlevels;
static void HeRecom( long ipZ );
/* evaluate collisional rates */
static void HeCollid( long ipZ);
/* evaluate photoionization rates  */
static void HePhoto( long ipZ);
/* evaluate photoionization rates  */
static void HeLevel( long ipZ);
/* HeCSInterp interpolate on He1 collision strengths */
static float HeCSInterp( long ipZ , long ipHi , long ipLo );

static FILE *ofp;  


/* experimental energies, in wavenumbers, for atomic helium */
#define NHE1LEVELS 111
double He1Energies[NHE1LEVELS] =
{0, 159850.318, 166271.7, 169081.495, 171129.148, 183231.08, 184859.06, 
185559.2, 186095.9, 186099.22, 186208.62, 190292.46, 190934.5, 191211.48, 
191438.83, 191440.71, 191446.61, 191447.24, 191486.96, 193341.33, 193657.78, 
193796.07, 193911.48, 193912.54, 193914.31, 193915.79, 193936.76, 194930.46, 
195109.17, 195187.81, 195254.37, 195255.02, 195256.7, 195256.82, 195269.17, 
195862.63, 195973.1, 196021.7, 196064, 196064.3, 196065.4, 196065.51, 
196073.41, 196455.79, 196529.03, 196561.08, 196589.42, 196589.73, 196590.3, 
196590.42, 196595.56, 196856.37, 196907.13, 196929.68, 196949.49, 196949.63, 
196950.3, 196950.36, 196953.95, 197139.76, 197176.36, 197192.63, 197207.08, 
197207.3, 197208, 197210.41, 197347.05, 197386.98, 197397.62, 197397.75, 
197398.6, 197400.18, 197503.69, 197524.26, 197534.44, 197542.54, 197542.67, 
197544.56, 197624.98, 197649.07, 197649.78, 197655.19, 197655.47, 197656.95, 
197721.13, 197739.9, 197744.918, 197744.94, 197746.15, 197796.63, 197813.11, 
197817.05, 197818.12, 197872.95, 197876.41, 197877.04, 197922.51, 197925.33, 
197925.87, 197964.02, 197966.75, 197966.8, 197999.12, 198001.43, 198001.44, 
198029.07, 198031.02, 198031.41, 198054.83, 198056.5, 198077.15};

#define NHE1AS 7
long ipHe1AsLo[NHE1AS]={0,2,1,3,3,1,3},
	ipHe1AsHi[NHE1AS] = {4,4,3,8,14,7,5};
double He1AsLam[NHE1AS] = {584.,20600.,10830.,5876.,4471.,3889.,7065.};
double He1AsAul[NHE1AS] = {1.80E+09,1.97E+06,1.02E+07,7.07E+07,2.46E+07,
	9.47E+06,2.78E+07};

/* the largest helium like ion in the sequence */
static long int Limit_Helium;
/* this block of variables will be obtained and freed here */
static long int *ipiv ; /* malloc out to [nHeLevels[ipZ]+1] */
static double **SaveZ/*[nHeLevels[ipZ]+2][nHLevels+2]*/, 
  *bvec/*[nHeLevels[ipZ]+2]*/, 
  *error/*[nHeLevels[ipZ]+2]*/, 
  *work/*[nHeLevels[ipZ]+2]*/, 
  **z/*[nHeLevels[ipZ]+2][nHLevels+2]*/;

/*He-like main routine to call HeLevel and determine model he-like species atom level balance,
 * called by ionize */
void HeLike(void)
{
	int lowsav , ihisav;
	long int i,
		ipLo ,
		ipHi , 
		ipZ;
	static int lgFIRST=TRUE;

#	ifdef DEBUG_FUN
	fputs( "<+>HeLike()\n", debug_fp );
#	endif

	/* on very first call, find how large the largest atom is a allocate the space */
	if( lgFIRST )
	{
		Limit_Helium = 0;
		for( ipZ=1; ipZ < LIMELM; ipZ++ )
		{
			/* do not consider elements that have been turned off */
			if( abundances.lgElmtOn[ipZ] )
			{
				/* this will be the very largest possible atom */
				Limit_Helium = MAX2( Limit_Helium , nHeLevels[ipZ] );
			}
		}

		/* now malloc some scratch space */
		ipiv = (long int *)malloc(sizeof(long int)*(unsigned)(Limit_Helium+1) ) ;
		bvec = (double *)malloc(sizeof(double)*(unsigned)(Limit_Helium+2) ) ;
		error = (double *)malloc(sizeof(double)*(unsigned)(Limit_Helium+2) ) ;
		work = (double *)malloc(sizeof(double)*(unsigned)(Limit_Helium+2) ) ;
		if( ipiv==NULL || bvec==NULL || error==NULL || work==NULL )
		{
			fprintf( ioQQQ, " could not malloc 1D arrays\n" );
			puts( "[Stop in HeLike]" );
			cdEXIT(1);
		}

		/* now do the 2D arrays */
		if( (SaveZ = (double **)malloc(sizeof(double *)*(unsigned)(Limit_Helium+2) ) ) ==NULL )
		{
			fprintf( ioQQQ, " could not malloc SaveZ array in 1D\n" );
			puts( "[Stop in HeLike]" );
			cdEXIT(1);
		}

		if(  (z = (double **)malloc(sizeof(double *)*(unsigned)(Limit_Helium+2) ))==NULL  )
		{
			fprintf( ioQQQ, " could not malloc z arrays in 1D\n" );
			puts( "[Stop in HeLike]" );
			cdEXIT(1);
		}

		/* now do the second dimension */
		for( i=0; i<(Limit_Helium+2); ++i )
		{
			SaveZ[i] = (double *)malloc(sizeof(double)*(unsigned)(Limit_Helium+2) ) ;
			if( SaveZ[i]==NULL )
			{
				fprintf( ioQQQ, " could not malloc 2D SaveZ\n" );
				puts( "[Stop in HeLike]" );
				cdEXIT(1);
			}

			z[i] = (double *)malloc(sizeof(double)*(unsigned)(Limit_Helium+2) ) ;
			if( z[i]==NULL )
			{
				fprintf( ioQQQ, " could not malloc 2D z\n" );
				puts( "[Stop in HeLike]" );
				cdEXIT(1);
			}
		}


		lgFIRST=  FALSE;
	}

	/* save state of he-like low and high ionziation, since must not change here,
	 * since there is a parallel helium atom that must decrement he */
	lowsav = IonRange.IonLow[1];
	ihisav = IonRange.IonHigh[1];
	for( ipZ=1; ipZ < LIMELM; ipZ++ )
	{
		/* do not consider elements that have been turned off */
		if( abundances.lgElmtOn[ipZ] )
		{
			/* note that ipZ scale is totally on c not physical scale, so 0 is h */
			/* evaluate helium-like balance if ionization reaches this high */
			if( (IonRange.IonHigh[ipZ] >= ipZ)  )
			{

				/* evaluate recombination rates */
				HeRecom(ipZ);

				/* evaluate collisional rates */
				HeCollid(ipZ);

				/* evaluate photoionization rates  */
				HePhoto(ipZ);

				/* solve for the level populations */
				HeLevel(ipZ);
			}
			else
			{
				helike.HeIonFrac[ipZ] = 0.;
				helike.HeIonSimple[ipZ] = 0.;

				/* zero it out since no population*/
				for( ipHi=1; ipHi < nHeLevels[ipZ]; ipHi++ )
				{
					for( ipLo=0; ipLo < ipHi; ipLo++ )
					{
						/* population of lower level rel to ion */
						HeLines[ipZ][ipHi][ipLo].PopLo = 0.;

						/* population of upper level rel to ion */
						HeLines[ipZ][ipHi][ipLo].PopHi =  0.;

						/* population of lower level rel to ion, corrected for stim em */
						HeLines[ipZ][ipHi][ipLo].PopOpc =  0.;
					}
				}
			}
		}
	}
	IonRange.IonLow[1] = lowsav;
	IonRange.IonHigh[1] = ihisav;

	return;
}
void testhelike(void)
{
	long f;

	/* Ask user for maximum value of n to be considered.	
	printf("Gimme maximum value of n: \n");
	scanf("%ld", &maxn); */
	maxn = 5;

	/* Upper level must have n >= 2.	*/
	assert(maxn > 1L);

	/* Open output file and print header.	*/
	ofp = fopen("output.txt","w");
	fprintf(ofp," Z nflf   nl    s/t  A \n");

	/* nlevels is the total number of levels	*/
	/* up to and including maximum value of n.	*/
	/* Autoionization levels are also included.	*/
	nlevels = maxn * (maxn + 1L) + 3L;

	/* Allocate space so that vector he_levels contains	*/
	/* nlevels number of elements.			*/
	if ( (he_levels= calloc((size_t)nlevels,sizeof(Elevels)) ) == NULL )
	{
		fprintf(ofp," Could not calloc he_levels[] \n");
		exit(1);
	}

	/* Assign values to elements of vector he_levels.	*/
	org_assign(); 

	/* Start with Hydrogen and increment the nuclear charge	*/
	/* after completing all transitions.			*/
	/* Transitions for a given charge are handled by first 	*/
	/* picking the lower level of the transition (moving up */
	/* the spectrum from ground to continuum) and then 	*/
	/* stepping through all possible upper levels within	*/
	/* function loop.					*/
	for ( Z1 = 1; Z1 <= LIMELM; ++Z1 )
	{
		for ( f = 0; f < nlevels - 1L; ++f )
		{
			loop(f);
		}  
	}
	fclose(ofp);
	return;
}

static double AngerJ(
			 double v, 
			 double z1 
			 ) 
{ 
	double yy; 

	vv = v; 
	zz = z1; 

	yy = qg32( (double)0.0, (double) PI, Jint ); 

	return yy; 
} 

static double Jint(
		   double theta 
		   ) 
{ 
	double 
	  d0, 
	  d1, 
	  d2, 
	  d3, 
	  d4, 
	  d5; 


	d0 = (1.0 / (double) PI); 
	d1 = vv * theta; 
	d2 = zz * sin(theta); 
	d3 = d1 - d2; 
	d4 = cos(d3); 
	d5 = d0 * d4; 

	return d5; 
} 

double correct(
			  Elevels Li,
			  Elevels Lf,
			  long ZZ1)
{
	/* 	Correction factors for singlets		*/

	double cor1[3][4] =     /* Z > 8, lf = 0 	*/ 
	{{1.48,1.2,0.94,0.77},		/* nf = 1	*/
		{1.0,0.98,0.85,0.73},		/* nf = 2	*/
		{1.0,1.0,1.02,0.92}};		/* nf = 3	*/
	double cor2[2][3] =         /* li = 0 		*/  
	{{0.77,0.62,0.52},		/* nf = 2	*/
		{1.0,0.90,0.81}};		/* nf = 3 	*/
	double cor3[2][3] =     /* lf = 1, li = 2 	*/  
	{{0.90,0.86,0.77},		/* nf = 2	*/
		{1.0,0.95,0.89}};		/* nf = 3 	*/
	double cor4[3] =        /* lf = 2, li = 1 	*/  
	{1.0,0.88,0.75};		/* nf = 3	*/
	double cor[3][4] =      /* nf = 1, lf = 0 	*/ 
	{{0.76,0.62,0.50,0.40},		/* z = 1 	*/
		{2.0,1.65,1.28,1.02},		/* z = 2,3 	*/
		{1.65,1.35,1.07,0.86}};		/* z = 4...8 	*/

	/*	Correction factors for triplets		*/

	double cor6[2][4] =     /* lf = 0 		*/ 
	{{1.0,1.05,0.88,0.75},		/* nf = 2	*/
		{1.0,1.0,1.01,0.92}};		/* nf = 3	*/
	double cor7[2][3] =         /* li = 0 		*/  
	{{0.74,0.60,0.49},		/* nf = 2	*/
		{1.0,0.86,0.78}};		/* nf = 3 	*/
	double cor8[2][3] =     /* lf = 1, li = 2 	*/  
	{{0.92,0.89,0.79},		/* nf = 2	*/
		{1.0,0.96,0.91}};		/* nf = 3 	*/
	double cor9[3] =        /* lf = 2, li = 1 	*/  
	{1.0,0.73,0.58};		/* nf = 3	*/

	long ni = Li.n;
	long nf = Lf.n;
	long li = Li.l;
	long lf = Lf.l;
	long s = Li.s;
	long n1,n2,l1,l2;
	double pz,pn,c,a,b,a2,b2,d,e;  

	/* For high initial n and low changes in n, the Coulomb	*/
	/* approximation works well and no correction is needed.*/
	if ( (nf >= 7) && ((ni - nf) <= (0.5*nf)) )
		return 1.0;

	/* Only correction parameters up to ni = 5 and nf=3 	*/
	/* are stored in the above arrays. So for transitions	*/
	/* with higher initial and/or final n, the highest one	*/
	/* stored is read, and the value is changed by a	*/
	/* function of the actual initial and/or final n.	*/
	/* 	(See last if/else clause in routine.)		*/
	n1 = (ni > 5L) ? 5L : ni;
	n2 = (nf > 3L) ? 3L : nf;

	/*	Same thing applies for l as for n with one 	*/
	/*	exception:	correction factors involving 	*/
	/*	orbitals higher than d are assumed to be the	*/
	/*	same as for d itself.				*/
	if ( (li > 2L) || (lf > 2L) )
	{
		l1 = (li > lf) ? 2L : 1L;
		l2 = (li > lf) ? 1L : 2L;
	}
	else
	{
		l1 = li;
		l2 = lf;
	}

	assert( n1 <= 5L);
	assert( n2 <= 3L);
	assert( l1 <= 2L);
	assert( l2 <= 2L);

	/* Corrections for singlet transitions	*/
	if ( s == 0L )
	{ 
		/* Errors in transitions from high n to ground	*/
		/* are easily fit to the function below.	*/
		if ( (nf == 1L) && (ni > 5L) )
		{
			pz = 1.37*pow((double)ZZ1,-0.105);
			pn = 1.83*exp(-0.2*(double)(ni-1L));
			return(pz*pn);
		}

		/* Transitions to ground are highly inaccurate,	*/
		/* especially for low nuclear charge.		*/
		/* This routine reads the correction factor.	*/
		if ( (ZZ1 <= 8L) && (nf == 1L) )
		{
			if ( ZZ1 > 3L )
				return cor[2][n1-2];
			else if ( ZZ1 > 1L )
				return cor[1][n1-2];
			else
				return cor[ZZ1-1][n1-2];
		}

		/* Corrections for np - n's transitions.	*/
		if ( (lf == 0L) && (li == 1L) )
		{
			c = cor1[n2-1L][n1-2L];
		}

		/* Corrections for ns - n'p transitions.	*/
		else if ( (li == 0L) && (lf == 1L) )
		{
			c = cor2[n2-2L][n1-3L];
		}

		/* Corrections for nd - n'p transitions.	*/
		else if ( (l2 == 1L) && (l1 == 2L) )
		{
			c = cor3[n2-2L][n1-3L];
		}

		/* Corrections for np - n'd transitons.		*/
		else if ( (l1 == 1L) && (l2 == 2L) )
		{
			c = cor4[n1-3L];
		}
		else
		{
			c = 1.0;
		}
	}

	/* Correction factors for triplet tranistions.		*/
	else
	{
		/*	np - n's transitions.	*/
		if ( (lf == 0L) && (li == 1L) )
		{
			c = cor6[n2-2L][n1-2L];
		}

		/*	ns - n'p transitions.	*/
		else if ( (li == 0L) && (lf == 1L) )
		{
			c = cor7[n2-2L][n1-3L];
		}

		/*	nd - n'p transitions.	*/
		else if ( (l2 == 1L) && (l1 == 2L) )
		{
			c = cor8[n2-2L][n1-3L];
		}

		/* 	np - n'd transitions.	*/
		else if ( (l2 == 2L) && (l1 == 1L) )
		{
			c = cor9[n1-3L];
		}
		else
		{
			c = 1.0;
		}
	}    

	/* The following lines manipulate the stored correction	*/
	/* factor according to the actual inital and final n.	*/

	/* For a decrease in angular momentum.	*/
	if ( li > lf )
	{
		a = -0.0084*ni+1.0837;
		b = 0.3335*ni-0.797;
		a2 = 0.073*ni+0.8565;
		d = a/(1+exp(-(nf-b)/c));
		e = a/(1+exp(-(n2-b)/c));   
		return(c+(ni-n1)*(0.01*n2-0.1)+d-e);
	}

	/* For an increase in angular momentum.	*/
	else
	{
		a = -0.2424*n2+3.3095;
		b = 0.6243*log((double)n2)-1.4469;
		a2 = -0.0026*ni*ni+0.0488*ni-0.2519;
		b2 = 0.0124*ni*ni-0.2488*ni+1.5622;
		d = a2*nf*nf+b2*nf;
		e = a2*n2*n2+b2*n2;
		return(c+a*(pow((double)ni,b)-pow((double)n1,b))+d-e);
	}
}

/* This routine calculates the quantum defect	*/ 
/* for a given level and nuclear charge.	*/

static double defect(Elevels L_ptr, long ZZ1)  
{  
	double qd;	/* The quantum defect	*/

	/* Parameters for fits to quantum defects for	*/
	/* P triplet and S orbitals. The dimensions are	*/
	/*	first: l				*/
	/*	second: n				*/
	/* 	third: parameters a,b,and c.		*/
	double param[3][4][3]=  
	{ 
		{{0.6451941,0.3119437,-1.2722842}, 
			{0.7664874,0.3455675,-1.3976462}, 
			{0.8247101,0.3603131,-1.4520500}, 
			{0.8878402,0.3714450,-1.4995732}}, 
		{{1.4203514,0.5311096,-2.6728087}, 
			{1.5733513,0.5997339,-2.9253834}, 
			{1.4531025,0.5924751,-2.8662756}, 
			{1.6038999,0.6342552,-3.0298071}}, 
		{{-2.2323488,0.0890840,-0.5166053}, 
			{-2.0463691,0.1222081,-0.6672983}, 
			{-1.9904104,0.1328918,-0.7150879}, 
			{-1.9500974,0.1452111,-0.7649031}} 
	};   

	/* Because they cannot be fit to a funtion of	*/
	/* the same form as the other orbitals, the 	*/
	/* P singlets are fit to a different function.	*/
	/* These are the parameters, with dimensions	*/
	/*	first: n				*/
	/*	second: parameters a and b.		*/
	double P1[4][2]=
	{
		{-0.020195,8.365235},
		{-0.022612,6.320906},
		{-0.019151,6.670690},
		{-0.022884,5.790262}
	};

	/* The P singlets of He do not conform to the 	*/
	/* above, so the defects themselves are listed,	*/
	/* for n = 2,3,4, and 5.			*/
	double P1He[4]=
	{-0.009492,-0.011351,-0.011904,-0.01218};

	/* Ionization potentials (in wavenumber) for 	*/
	/* each ion in the iso-seq up to Z=30.		*/
	double Eion[29] =       
	{198310.76,610078.5,1241242,2091976,3162408,4452722,
		5963078,7693810,9645400,11816996,14209908,16824910,
		19660800,22700000,26001513,29507951.12,33237172.57,
		37188596.44,41366547,45770332.07,50401565.71,
		55259441.05,60343958.1,65658343.06,71200000,
		76978329.86,82984738.24,89224079.45,95800000};

	double a,b,c;   
	long n = L_ptr.n;
	long lqn = L_ptr.l;
	long s = L_ptr.s;
	long nold = n;  

	assert(n >= 1L);
	assert((ZZ1 >= 1L) && (ZZ1 <= LIMELM));       

	/* Quantum defects for Hydrogen are identically zero.*/
	if ( ZZ1 == 1L )
	{
		return(0.0);
	}
	else if ( n == 1L )
	{
		/* Quantum defects for ground state are found	*/
		/* from the rydberg equation, and the		*/
		/* ionization potential of the ion.		*/
		qd = 1.0 - (ZZ1-1L)*sqrt(109737.315/Eion[ZZ1-2]);
		return qd;  
	}
	else
	{
		/* For levels with n > 5, the quantum defect	*/
		/* is approximately the same if n equaled 5.	*/
		if ( n > 5L )
		{
			n = 5L;
		}
		if ( lqn==1L && s==0L )
		{
			if ( ZZ1 == 2L )
				qd = P1He[n-2];
			else
				qd = (P1[n-2][0])*exp(-1*ZZ1/(P1[n-2][1]));  
			return qd;  
		}
		/* Defects for orbitals with l>2 	*/
		/* are approximately equal to zero.	*/
		else if ( lqn < 2L )
		{
			a = param[2*(lqn+1)-s-1][n-2][0];  
			b = param[2*(lqn+1)-s-1][n-2][1];  
			c = param[2*(lqn+1)-s-1][n-2][2];  
			qd = exp((a+c*ZZ1)/(1.0+b*ZZ1));  
			return qd;  
		}
		else if ( lqn < nold )
		{
			return 0.0; 
		}
		else
		{
			exit(9);  
		}  
	}   
}

static double he_energy(double Eff_nupper,double Eff_nlower,
		/* ZZ1 is nuclear charge on physical scale */
		long ZZ1,long s,long li,
		long ipHi , long ipLo )
{
	double Ei,Ef,energy_wn,a,b;
	long Z0;

	/* Calculate energy for autoionization levels.	*/
	if ( li < 0L )
	{
		assert( li > -3L);
		switch ( li )
		{
		case -1:
			a = (s == 0L) ? 75827. : 72116.;
			b = (s == 0L) ? 1.8956 : 1.9084;
			break;
		case -2:
			a = (s == 0L) ? 74853. : 74010.;
			b = (s == 0L) ? 1.9 : 1.9021;
			break;
		default:
			a = -1.0;
			b = 0.0;
			exit(4);    
			break;
		}

		energy_wn = a*pow((double)ZZ1,b);
	}
	/* for atomic helium use experimental energies */
	else if( ZZ1 == 2 && ipHi < NHE1LEVELS )
	{
		energy_wn = He1Energies[ipHi] - He1Energies[ipLo];
	}
	else
	/* Calculate energy difference for all other trans.	*/
	{
		Z0 = (ZZ1 == 1L) ? ZZ1 : (ZZ1 - 1L);

		Ei = (Z0/Eff_nupper)*(Z0/Eff_nupper);
		Ef = (Z0/Eff_nlower)*(Z0/Eff_nlower);  

		/* Energy difference (in wavenumbers) is calculated 	*/
		/* using the Rydberg equation.				*/
		energy_wn = 109737.315*(Ef - Ei);
	}

	return energy_wn;
}

double forbidden(
				Elevels Li,
				long ZZ1
				)
{
	double A,b,c;

	if ( Li.s == 0 )
	{
		if ( ZZ1 == 2L )
		{
			return 51;
		}
		b = 3.93;
		c = 6.3856;
	}
	else if ( Li.l == 0 )
	{
		if ( ZZ1 == 2L )
		{
			return 0.000127;
		}
		b = 4e-07;
		c = 10.39;
	}
	else
	{
		if ( ZZ1 == 2L )
		{
			return 176;
		}
		b = 0.9317;
		c = 9.7306;
	}       

	A = b * pow((double)ZZ1,c);
	assert( A > 0.);
	return A;   
}

/*  Function orbit is only necessary for output */

void orbit(long index, int x)
{
	char *c = "     "; 

	/*  Negative angular momentum values are used as flags	*/
	/*  to denote autoionization levels.			*/
	/*  A string containing the orbit name is output.	*/

	switch ( (int)he_levels[index].l )
	{
	case -1:
		c = (he_levels[index].s == 0) ? "s2p1P" : "s2p3P";
		break;
	case -2:
		c = (he_levels[index].s == 0) ? "p2,1D" : "p2,3P";
		break;
	case 0:
		c = "s";
		break;
	case 1:
		c = "p";
		break;
	case 2:
		c = "d";
		break;
	default:
		fprintf(ofp,"%-*c ",x,99+(int)he_levels[index].l);
		return;
	}
	fprintf(ofp,"%-*.*s ",x,x,c);
}

/* Function unit is only necessary for output	*/
char *unit(long i)
{
	char *d = "s-1 "; 

	/* Default is Einstein A is output */

	if ( he_levels[i].l < 0 )
		d = "cm-1";

	/* For autoionization levels, wavenumbers	*/
	/* relative to ground are output. 		*/

	return d;
}

/* Passed to function loop is the index of a lower level.	*/
/* Since the levels are indexed in increasing energy order,	*/
/* the upper level of a transition must have a higher index.	*/
static void loop(long f)
{
	/* Variables used are:				*/
	/*	i = index of upper level		*/
	/*	s = ascii code for spin designation,	*/
	/*		s -> singlet			*/
	/*		t -> triplet			*/
	/*		a -> autoionization		*/
	/*		i -> intercombined		*/
	/*		f -> forbidden			*/
	/*	Eff_nupper,Eff_nlower = initial and final	*/
	/*		principal quantum numbers.	*/
	/*	A = Einstein A				*/
	/*	RI2 = square of radial integral.	*/
	/*	k = level wavenumber (or delta)		*/
	/*	Aork = stores value of output to be 	*/
	/* 		printed, either A or k.		*/

	long i;
	int s;
	double Eff_nupper,Eff_nlower,A,RI2,k,Aork;

	for ( i = f + 1; i <= nlevels - 1L; ++i )
	{
		if ( ((he_levels[i].s == he_levels[f].s) && (abs((int)(he_levels[i].l - he_levels[f].l)) == 1)) || 
			 ((he_levels[i].n == 2L) && (he_levels[f].n == 1L)) )
		{

			/* Skip same-n transitions for Hydrogen and n >3 */
			if ( ((Z1==1L)||(he_levels[i].n>3L)||(he_levels[i].l<0)) && (he_levels[i].n==he_levels[f].n) )
				continue;

			/* Hydrogren has no triplicate levels, and	*/
			/* transition down to autoionization levels 	*/
			/* are not considered. 				*/
			if ( ((Z1 == 1L) && (he_levels[i].s == 1L)) || (he_levels[f].l < 0) )
				continue;

			/* Eff_nupper & Eff_nlower calculated by subtracting 	*/
			/* quantum defect from principal quantum number */
			Eff_nupper = (double)he_levels[i].n - defect(he_levels[i],Z1);
			Eff_nlower = (double)he_levels[f].n - defect(he_levels[f],Z1);

			/* energy calculates wavenumber gap between	*/
			/* upper and lower levels.			*/
			k = he_energy(Eff_nupper,Eff_nlower,Z1,he_levels[i].s,he_levels[i].l,
				f , i );
			if ( he_levels[i].l < 0 )
			{
				Aork = k;
				s = 97;		/* ascii code for 'a'	*/
			}
			else if ( (he_levels[i].l == he_levels[f].l) || (he_levels[i].s != he_levels[f].s) )
			{
				Aork = forbidden(he_levels[i],Z1);
				s = (he_levels[i].s == 0L) ? 105:102;
				/* above: ascii code for 'i' or 'f'*/
			}
			else
			{
				RI2 = scqdri(Eff_nupper,he_levels[i].l,Eff_nlower,he_levels[f].l,(double)(Z1-1L));
				A = ritoa(he_levels[i].l,he_levels[f].l,Z1,k,RI2);
				A *= correct(he_levels[i],he_levels[f],Z1);
				Aork = A;
				s = 115+(int)he_levels[i].s; /* ascii code	*/
				/* for 's', 't' */
			}

			/* Results are printed to output file.	*/
			fprintf(ofp,"%2ld  %2ld",Z1,he_levels[f].n);
			orbit(f,1);
			fprintf(ofp,"-%2ld",he_levels[i].n);
			orbit(i,5);
			fprintf(ofp," %c  %.3e %.4s  %f\n",s,Aork,unit(i),k);
		}
		else continue;
	}
}


/* Passed to function loop is the index of a lower level.	*/
/* Since the levels are indexed in increasing energy order,	*/
/* the upper level of a transition must have a higher index.	*/
static void he_1trans( long ipZ , Elevels he_upper , Elevels he_lower , 
			   double *Enerwn , 
			   double *Aul ,
			   /* incides for the upper and lower levels */
			   long ipHi , long ipLo )
{
	/* this will be A returned for totally forbidden transition */
#	define SMALLA 1e-20
	/* Variables used are:				*/
	/* ipZ is charge on the c scale */
	/*	i = index of upper level		*/
	/*	s = ascii code for spin designation,	*/
	/*		s -> singlet			*/
	/*		t -> triplet			*/
	/*		a -> autoionization		*/
	/*		i -> intercombined		*/
	/*		f -> forbidden			*/
	/*	Eff_nupper,Eff_nlower = initial and final	*/
	/*		principal quantum numbers.	*/
	/*	*Aul = Einstein A				*/
	/*	RI2 = square of radial integral.	*/
	/*	*Enerwn = level wavenumber (or delta)		*/
	/*	Aork = stores value of output to be 	*/
	/* 		printed, either A or k.		*/

	double Eff_nupper,Eff_nlower,RI2;

	/* Z1 is on the physical scale */
	Z1 = ipZ+1;

	/* if we can do nothing, return 1e-20 */
	*Aul = SMALLA;

	/* Eff_nupper & Eff_nlower calculated by subtracting 	*/
	/* quantum defect from principal quantum number */
	Eff_nupper = (double)he_upper.n - defect(he_upper,Z1);
	Eff_nlower = (double)he_lower.n - defect(he_lower,Z1);

	/* energy calculates wavenumber gap between	*/
	/* upper and lower levels.			*/
	*Enerwn = he_energy(Eff_nupper,Eff_nlower,Z1,he_upper.s,he_upper.l,
		ipHi , ipLo );
	if( *Enerwn == 0. )
	{
		*Enerwn = 1.;
	}

	/* check if this is an allowed transition, or the 2s-1s transition */
	if ( ((he_upper.s == he_lower.s) && (abs((int)(he_upper.l - he_lower.l)) == 1)) || 
		 ((he_upper.n == 2L) && (he_lower.n == 1L)) )
	{

		/* Skip same-n transitions for Hydrogen and n >3 */
		if ( ((Z1==1L)||(he_upper.n>3L)||(he_upper.l<0)) && (he_upper.n==he_lower.n) )
		{
			return;
		}

		/* Hydrogren has no triplet levels, and	*/
		/* transition down to autoionization levels 	*/
		/* are not considered. 				*/
		if ( ((Z1 == 1L) && (he_upper.s == 1L)) || (he_lower.l < 0) )
		{
			return;
		}

		if ( he_upper.l < 0 )
		{
			/* flag for autoionizing levels? */
			*Aul = 1e-10;
		}
		else if ( (he_upper.l == he_lower.l) || (he_upper.s != he_lower.s) )
		{
			*Aul = forbidden(he_upper,Z1);
			assert( *Aul > 0.);
		}
		else
		{
			RI2 = scqdri(Eff_nupper,he_upper.l,Eff_nlower,he_lower.l,(double)(Z1-1L));
			*Aul = ritoa(he_upper.l,he_lower.l,Z1,*Enerwn,RI2);
			*Aul *= correct(he_upper,he_lower,Z1);
			assert( *Aul > 0.);

		}

		/* Results are printed to output file.	
		fprintf(ofp,"%2ld  %2ld",Z1,he_lower.n);
		orbit(f,1);
		fprintf(ofp,"-%2ld",he_upper.n);
		orbit(i,5);
		fprintf(ofp," %c  %.3e %.4s  %f\n",s,Aork,unit(i),*Enerwn);*/
		return ;
	}

	/* negative energy for a transition with substantial transition probability
	 * would be major logical error - but is ok for same-n l transitions */
	if( *Enerwn < 0. && *Aul > SMALLA )
	{
		fprintf( ioQQQ," he_1trans hit negative energy, ipZ=%li, val was %f \n", ipZ ,*Enerwn );
	}

	return ;
}

/* Function org_assign assigns, in order of increasing energy, 	*/
/* quantum numbers n,l, and s to each element of vector he_levels.	*/ 
static void org_assign(void)
{
	/*	in, il, and is are dummies for n, l, and s.	*/
	/*	i is the index in the vector he_levels.			*/
	long in, il, is, i = 0;
	for ( in = 1L; in <= maxn ; ++in )
	{
		for ( il = 0L; il < in; ++il )
		{
			for ( is = 1L; is >= 0 ; --is )
			{
				/* All levels except singlet P follow	*/
				/* the ordering scheme:			*/
				/*	lower l's have lower energy	*/
				/* 	triplets have lower energy	*/
				if ( (il == 1L) && (is == 0L) )
					continue;
				/* Ground state is not triplicate.	*/
				if ( (in == 1L) && (is == 1L) )
					continue;
				he_levels[i].n = in;
				he_levels[i].s = is;
				he_levels[i].l = il;
				++i;
			}
		}
		/*	Insert singlet P at the end of	*/
		/*	every sequence for a given n.	*/
		if ( in > 1L )
		{
			he_levels[i].n = in;
			he_levels[i].s = 0L;
			he_levels[i].l = 1L;
			++i;
		}
		/*	Insert autoionization levels at end of n = 2.	*/
		/*	Values of l are:	-1 ->	2s2p		*/
		/*				-2 ->	2p^2		*/
		if ( in == 2L )
		{
			he_levels[i].n = 2L;
			he_levels[i].s = 0L;
			he_levels[i].l = -1L;
			he_levels[i+1].n = 2L;
			he_levels[i+1].s = 1L;
			he_levels[i+1].l = -1L;
			he_levels[i+2].n = 2L;
			he_levels[i+2].s = 0L;
			he_levels[i+2].l = -2L;
			he_levels[i+3].n = 2L;
			he_levels[i+3].s = 1L;
			he_levels[i+3].l = -2L;
			i += 4;
		}
	}
}

/* Function he_assign assigns, in order of increasing energy, 	*/
/* quantum numbers n,l, and s to each element of vector he_levels.	
 * nmax is the upper quantum number, there will be many l levels within it.
 * ndim is dimension of vector, used to confirm that size not exceeded */ 
void he_assign( long nmax , long ndim )
{
	/*	in, il, and is are dummies for n, l, and s.	*/
	/*	i is the index in the vector he_levels.			*/
	long in, il, is, i = 0;
	for ( in = 1L; in <= nmax ; ++in )
	{
		for ( il = 0L; il < in; ++il )
		{
			for ( is = 1L; is >= 0 ; --is )
			{
				/* All levels except singlet P follow	*/
				/* the ordering scheme:			*/
				/*	lower l's have lower energy	*/
				/* 	triplets have lower energy	*/
				if ( (il == 1L) && (is == 0L) )
					continue;
				/* Ground state is not triplicate.	*/
				if ( (in == 1L) && (is == 1L) )
					continue;
				he_levels[i].n = in;
				he_levels[i].s = is;
				he_levels[i].l = il;
				++i;
			}
		}
		/*	Insert singlet P at the end of	*/
		/*	every sequence for a given n.	*/
		if ( in > 1L )
		{
			he_levels[i].n = in;
			he_levels[i].s = 0L;
			he_levels[i].l = 1L;
			++i;
		}
		/*	Insert autoionization levels at end of n = 2.	*/
		/*	Values of l are:	-1 ->	2s2p		*/
		/*				-2 ->	2p^2		*/
#		if 0
		/* >>chng 00 nov 20, killed this logic, which added
		 * these 4 levels right in the middle of the stack of quantum numbers */
		if ( in == 2L )
		{
			he_levels[i].n = 2L;
			he_levels[i].s = 0L;
			he_levels[i].l = -1L;
			he_levels[i+1].n = 2L;
			he_levels[i+1].s = 1L;
			he_levels[i+1].l = -1L;
			he_levels[i+2].n = 2L;
			he_levels[i+2].s = 0L;
			he_levels[i+2].l = -2L;
			he_levels[i+3].n = 2L;
			he_levels[i+3].s = 1L;
			he_levels[i+3].l = -2L;
			i += 4;
		}
#		endif
	}

	/* confirm that we did not overrun  the array */
	assert( i < ndim );

	return;
}

double ritoa(long li, long lf, long ZZ1, double k, double RI2)
{  
	/*	Variables are as follows:		*/
	/*	lg = larger of li and lf		*/
	/*	fmean = mean oscillator strength	*/
	/*		for a given level.		*/
	/*	mu = reduced mass of optical electron.	*/
	/*	EinsteinA = Einstein emission coef.	*/
	/*	w = angular frequency of transition.	*/
	/*	RI2_cm = square of rad. int. in cm^2.	*/
	/*	A = mass of ion in atomic mass units.	*/
	long lg;
	double fmean,mu,EinsteinA,w,RI2_cm;
	double A[30] =
	{1.00,4.0026,6.941,9.01218,10.81,
		12.011,14.0067,15.9994,18.998403,20.179,
		22.98977,24.305,26.98154,28.0855,30.97376,
		32.06,35.453,39.948,39.0983,40.08,
		44.9559,47.9,50.9415,51.996,54.938,
		55.847,58.9332,58.7,63.546,65.38};


	mu = ELECTRON_MASS/(1+ELECTRON_MASS/(A[ZZ1-1]*ATOMIC_MASS_UNIT));

	/* 8065.541 converts cm-1 to eV 	*/
	/* hbar then converts to rad/sec 	*/

	w = k /(8065.541*hbar);

	RI2_cm = RI2 * (2.8002856e-17);

	lg = (lf > li) ? lf : li;

	fmean = 2.0*mu*w*lg*RI2_cm/((3.0*H_BAR) * (2.0*li+1.0));

	/* fmean = -(2/3)*(mu/hbar)*w*(lg/(2*li+1))*RI		*/

	EinsteinA = (.66702)*k*k*fmean;

	return EinsteinA;

}

#undef DEBUG 

/* Semi-Classical Quantum Defect Radial Integral */ 

/* See for example 
   Atomic, Molecular & Optical Physics Handbook 
   Gordon W. F. Drake; Editor 
   AIP Press 
   Woddbury, New York. 
   1996 
*/ 
static double scqdri(
			 double nstar,
			 long int l, 
			 double npstar,
			 long int lp, 
			 double iz 
			 ) 
{ 
	const double a_o = (double)BORHRADIUS_AU; 
	const double TwobyPi = (double)TWOBYPI; 

	double n_c; 
	double D_n; 
	double D_l; 
	double lg; 
	double x; 
	double e; 
	double d1,d2,d3,d4,d5,d6,d7,d8,d9; 
	double d6_1; 
	double v,z1; 

	assert( l >= 0L ); 
	assert( lp >= 0L ); 

	/*  assert( (l == lp + 1) || ( l == lp - 1) );  */

	if ( iz == 0.0 )
		iz += 1.0;

	/* 
	   The reference gives this as 
	  n_c = ( 2.0 * nstar * npstar ) / ( nstar + npstar ) ; 
	 */

	n_c = sqrt( (double)( nstar * npstar ) ); 

	assert(n_c != 0.0 ); 

	D_n = npstar - nstar; 

	if ( D_n == 0.0 )
	{
		d9 = 0.0;
		return d9;
	}

	/* I not sure why the following isn't 
	   D_l = (l' - l) 
	   as per the reference but this 
	   gives the "correct" values of R(n,l;n',l') 
	*/
	D_l = (double)(l - lp); 

	if ( lp > l )
	{
		lg = (double)lp; 
	}
	else
	{
		lg = (double)l; 
	} 

	assert( 1.0 - (lg/n_c)*(lg/n_c) > 0 ); 
	e = sqrt( 1.0 - (lg/n_c)*(lg/n_c) ); 

	x = e * D_n; 

	d1 = ( a_o / iz ); 

	d2 = (n_c * n_c)/(2.0 * D_n); 

	d3 = (1.0 - ((D_l * lg)/n_c)); 

	v =  (D_n - 1.0); 
	z1 =  (-1.0 * x); 

	d4 = AngerJ( v, z1 ); 

	d5 = (1.0 + ((D_l * lg)/n_c)); 

	v =  (D_n + 1.0); 
	z1 =  (-1.0 * x); 

	d6 = AngerJ( v , z1 ); 

	d6_1 = (double)PI * (double)D_n; 

	d7 = TwobyPi * sin( d6_1 ) * (1.0-e); 

	d8 = d1 * d2 * ( (d3 * d4) - (d5 * d6) + d7 ); 

	d9 = d8 * d8; 

	return d9; 

} 
/* create he-like series, called by ContCreatePointers */
void HeCreate(void)
{
	long int nmax, nl , nhedim;
	long int ipLo ,
		ipHi,
		ipZ,
		need;
	char chLine[132] , 
		chFilename[134] ;
	FILE *ioDATA;
	long int i, i1, i2, i3 ;
	int lgEOL,
		lgHIT;
	long int j;

	char chSpin[2]={'1','3'};
	char chL[6]={'S','P','D','F','G','H'};

#	ifdef DEBUG_FUN
	fputs( "<+>HeCreate()\n", debug_fp );
#	endif

	/* now malloc the space for the helium-like lines, 
	 * but this must be done exactly one time per coreload */
	if( lgHeMalloc )
	{
#		ifdef DEBUG_FUN
		fputs( " <->HeCreate()\n", debug_fp );
#		endif
		return;
	}
	
	/* we will never malloc space again, in this coreload,
	 * following says never change number of levels in he atom again,
	 * future he-like levels commands will be ignored*/
	lgHeMalloc = TRUE;	

	/*float ***helike herec[LIMELM][LMHLVL+1][3]*/
	if( (helike.herec = (float ***)malloc(sizeof(float **)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.herec 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/*double ***helike helbolt[LIMELM][lev][lev]*/
	if( (helike.helbolt = (double ***)malloc(sizeof(double **)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.helbolt 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/*double **HeLTEPop HeLTEPop[ipZ][ipLo]*/
	if( (helike.HeLTEPop = (double **)malloc(sizeof(double *)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.double 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/*double **HeDepCoef HeDepCoef[ipZ][ipLo]*/
	if( (helike.HeDepCoef = (double **)malloc(sizeof(double *)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.HeDepCoef 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/* flouat **HeOpacRatio[ipZ][ipLo]  */
	if( (helike.HeOpacRatio = (float **)malloc(sizeof(float *)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.HeOpacRatio 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/*double **HeLevNIonRyd HeLevNIonRyd[ipZ][ipLo]*/
	if( (helike.HeLevNIonRyd = (double **)malloc(sizeof(double *)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.HeLevNIonRyd 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/*long int **HeIonPnt HeLevNIonRyd[ipZ][ipLo]*/
	if( (helike.ipHeLevNIonCon = (long int **)malloc(sizeof(long int *)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.ipHeLevNIonCon 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/*double **hecolnc hecolnc[LIMELM][LMHLVL+1]*/
	if( (helike.hecolnc = (double **)malloc(sizeof(double *)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.hecolnc 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/*float **hecolnc hecolnc[LIMELM][LMHLVL+1]*/
	if( (helike.hegamnc = (double **)malloc(sizeof(double *)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.hegamnc 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/*char ***helike chLevel[lev][lev][str]*/
	if( (helike.chLevel = (char **)malloc(sizeof(char *)*(unsigned)(nHeLevels[ipHELIUM]) ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.chLevel 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}
	for( ipLo=0; ipLo < nHeLevels[ipHELIUM] ;++ipLo )
	{
		if( (helike.chLevel[ipLo] = 
			(char*)malloc(sizeof(char)*(unsigned)10 ))==NULL )
		{
			fprintf( ioQQQ, 
				" HeCreate could not 2 malloc helike.chLevel 1\n" );
			puts( "[Stop in HeCreate]" );
			cdEXIT(1);
		}
	}


	for( ipZ=1; ipZ < LIMELM; ++ipZ )
	{
		if( ipZ < 2 || abundances.lgElmtOn[ipZ] )
		{

			if( (helike.hecolnc[ipZ] = 
				(double*)malloc(sizeof(double)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.hecolnc 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			if( (helike.hegamnc[ipZ] = 
				(double*)malloc(sizeof(double)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.hegamnc 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			if( (helike.herec[ipZ] = 
				(float**)malloc(sizeof(float*)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.herec 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			if( (helike.helbolt[ipZ] = 
				(double**)malloc(sizeof(double*)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.helbolt 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			if( (helike.HeLTEPop[ipZ] = 
				(double*)malloc(sizeof(double)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.HeLTEPop 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			if( (helike.HeDepCoef[ipZ] = 
				(double*)malloc(sizeof(double)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.HeDepCoef 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			if( (helike.HeLevNIonRyd[ipZ] = 
				(double*)malloc(sizeof(double)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.HeLevNIonRyd 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			if( (helike.HeOpacRatio[ipZ] = 
				(float*)malloc(sizeof(float)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.HeOpacRatio 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			if( (helike.ipHeLevNIonCon[ipZ] = 
				(long int*)malloc(sizeof(long int)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.ipHeLevNIonCon 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}

			for( ipLo=0; ipLo < nHeLevels[ipZ] ;++ipLo )
			{
				if( (helike.herec[ipZ][ipLo] = 
					(float*)malloc(sizeof(float)*(unsigned)3 ))==NULL )
				{
					fprintf( ioQQQ, 
						" HeCreate could not third malloc helike.herec 1\n" );
					puts( "[Stop in HeCreate]" );
					cdEXIT(1);
				}
				if( (helike.helbolt[ipZ][ipLo] = 
					(double*)malloc(sizeof(double)*(unsigned)nHeLevels[ipZ] ))==NULL )
				{
					fprintf( ioQQQ, 
						" HeCreate could not third malloc helike.herec 1\n" );
					puts( "[Stop in HeCreate]" );
					cdEXIT(1);
				}
			}
		}
	}

	/* first find the largest number of l states in any he-like ion */
	/* main helium-like arrays, fill with sane values */
	need = 0;
	for( ipZ=1; ipZ < LIMELM; ipZ++ )
	{
		/* only check elements that are turned on */
		if( ipZ < 2 || abundances.lgElmtOn[ipZ] )
		{
			need = MAX2( need, nHeLevels[ipZ] );
		}
	}

	/* we know the number of l levels, now find number of n to pass
	 * to routine */
	nl = 1;
	/* the max number of n levels that will be needed */
	nmax = 1;
	/* count number of levels, remembering that there are three autoionizing levels */
	while( nl <= need )
	{
		++nmax;
		nl += 2*nmax;
	}
	nhedim = 2* nl;

	/* Allocate space so that vector he_levels contains	*/
	/* nlevels number of elements. */
	if ( (he_levels= malloc((unsigned)nhedim*sizeof(Elevels)) ) == NULL )
	{
		fprintf( ioQQQ, 
			" HydroCreate could not malloc he_levels\n" );
		puts( "[Stop in HydroCreate]" );
		cdEXIT(1);
	}

	/* Function he_assign assigns, in order of increasing energy, 	*/
	/* quantum numbers n,l, and s to each element of vector he_levels.
	 * first arg is number of n levels, sec is dim of structure he_levels (with type Elevels)
	 * in malloc above */ 
	he_assign( nmax , nhedim );
	{
		/* option to print particulars of some line when called
		 * a prettier print statement is near where chSpin is defined below*/
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG )
		{
			/* first print internal representation */
			for( i=0; i<need; ++i )
			{
				fprintf(ioQQQ,"i %li n %li s %li l %li\n", 
					i, 
					he_levels[i].n,
					he_levels[i].s,
					he_levels[i].l );
			}

			exit(0);
		}
	}

	/* main helium-like arrays, fill with sane values */
	for( ipZ=1; ipZ < LIMELM; ipZ++ )
	{
		/* must always do helium even if turned off */
		if( ipZ < 2 || abundances.lgElmtOn[ipZ] )
		{
			for( ipLo=ipHe1s1S; ipLo < (nHeLevels[ipZ] - 1); ipLo++ )
			{
				/*float ghe1[5]={1., 3., 1., 9.,3.};*/
				for( ipHi=ipLo + 1; ipHi <nHeLevels[ipZ]; ipHi++ )
				{
					double Enerwn , Aul;
					float gup , glo;

					/* pointers to transitions, ipLo, ipHi
					 * actual quantum numbers, nLo nHi
					 * atomic number or charge and stage: */
					HeLines[ipZ][ipHi][ipLo].nelem = (int)(ipZ+1);
					HeLines[ipZ][ipHi][ipLo].IonStg = (int)(ipZ);

					/* stat. weights, s is 0 (singlets) or 1 (triplets)   */
					gup = (2.f*he_levels[ipHi].l + 1.f)* (2.f*he_levels[ipHi].s + 1.f) ;
					glo = (2.f*he_levels[ipLo].l + 1.f)* (2.f*he_levels[ipLo].s + 1.f) ;
					HeLines[ipZ][ipHi][ipLo].gLo = glo;
					HeLines[ipZ][ipHi][ipLo].gHi = gup;
					assert( HeLines[ipZ][ipHi][ipLo].gHi> 0.);
					assert( HeLines[ipZ][ipHi][ipLo].gLo> 0.);

					/* Passed to function loop is the index of a lower level.	*/
					/* Since the levels are indexed in increasing energy order,	*/
					/* the upper level of a transition must have a higher index.	*/
					/* ret is 0 if line ok, 1 if not*/
					he_1trans( ipZ , he_levels[ipHi] , he_levels[ipLo] , 
						&Enerwn , &Aul , ipHi , ipLo );

					/* transition prob */
					HeLines[ipZ][ipHi][ipLo].Aul = 
						(float)(Aul);
					assert(HeLines[ipZ][ipHi][ipLo].Aul > 0.);

					/* the energy of the transition in wavenumbers, this can be negative
					 * for highly forbidden upper levels where energy switches */
					HeLines[ipZ][ipHi][ipLo].EnergyWN = (float)Enerwn;

					HeLines[ipZ][ipHi][ipLo].EnergyRyd = 
						HeLines[ipZ][ipHi][ipLo].EnergyWN* (float)WAVNRYD;

					HeLines[ipZ][ipHi][ipLo].EnergyErg = 
						(float)(HeLines[ipZ][ipHi][ipLo].EnergyRyd*
					  EN1RYD);

					HeLines[ipZ][ipHi][ipLo].EnergyK = 
						(float)(HeLines[ipZ][ipHi][ipLo].EnergyRyd*
					  TE1RYD );

					/* very forbidden transitions may have negative energies */
					if( HeLines[ipZ][ipHi][ipLo].Aul < 1e-20f ||
						HeLines[ipZ][ipHi][ipLo].EnergyWN <= 0.)
					{

						/* this branch - bogus line */
						/* transition energy in various units:*/

						/* take abs value since some have order switched */
						/* make following an air wavelength */
						HeLines[ipZ][ipHi][ipLo].WLAng = 1e6f;

						HeLines[ipZ][ipHi][ipLo].gf = 1e-20f;

						/* derive the abs coef, call to function is gf, wl (A), g_low */
						HeLines[ipZ][ipHi][ipLo].opacity = 0.;
					}
					else
					{

						/* transition energy in various units:*/

						/* take abs value since some have order switched */
						/* make following an air wavelength */
						HeLines[ipZ][ipHi][ipLo].WLAng = 
							(float)fabs(1.0e8/
						  HeLines[ipZ][ipHi][ipLo].EnergyWN/
						  RefIndex(&HeLines[ipZ][ipHi][ipLo]));
						HeLines[ipZ][ipHi][ipLo].gf = 
							(float)(GetGF(HeLines[ipZ][ipHi][ipLo].Aul,
						  HeLines[ipZ][ipHi][ipLo].EnergyWN,
						  HeLines[ipZ][ipHi][ipLo].gHi));
						assert(HeLines[ipZ][ipHi][ipLo].gf > 0.);

						/* derive the abs coef, call to function is gf, wl (A), g_low */
						HeLines[ipZ][ipHi][ipLo].opacity = 
							(float)(abscf(HeLines[ipZ][ipHi][ipLo].gf,
						  HeLines[ipZ][ipHi][ipLo].EnergyWN,
						  HeLines[ipZ][ipHi][ipLo].gLo));
						HeLines[ipZ][ipHi][ipLo].opacity = MAX2(0.f, HeLines[ipZ][ipHi][ipLo].opacity );
					}

					/* redistribution function = complete */
					if( ipLo == 0 )
					{
						HeLines[ipZ][ipHi][ipLo].iRedisFun = 1;
					}
					else
					{
						/* >>chng 01 feb 27, had been -1, crd with core only,
						 * change to crd with wings as per discussion with Ivan Hubeny */
						/*HeLines[ipZ][ipHi][ipLo].iRedisFun = -1;*/
						HeLines[ipZ][ipHi][ipLo].iRedisFun = 2;
					}

					HeLines[ipZ][ipHi][ipLo].Pesc = 1.0;

					/* destruction probability*/
					HeLines[ipZ][ipHi][ipLo].Pdest = 0.0;

					/* upper and lower level populations */
					HeLines[ipZ][ipHi][ipLo].PopHi = 0.0;
					HeLines[ipZ][ipHi][ipLo].PopLo = 0.0;

					HeLines[ipZ][ipHi][ipLo].xIntensity = 0.0;

					/* basic collision Info:
					 * these are not Mewe lines, they are high quality: */
					HeLines[ipZ][ipHi][ipLo].cs1 = 0.0;

					/* following three will eventually be set by model atoms, but
					 * must have reasonable value at start */
					HeLines[ipZ][ipHi][ipLo].pump = 0.;
					HeLines[ipZ][ipHi][ipLo].AovTot = 0.;
					HeLines[ipZ][ipHi][ipLo].cool = 0.;
					HeLines[ipZ][ipHi][ipLo].heat = 0.;
					HeLines[ipZ][ipHi][ipLo].ColOvTot = 0.;
					HeLines[ipZ][ipHi][ipLo].PopOpc = 0.;
					HeLines[ipZ][ipHi][ipLo].damprel = 1e-30f;
					HeLines[ipZ][ipHi][ipLo].damp = 1e-30f;
				}
			}

			/* this is the two photon transition, designated by wl of 0 */
			HeLines[ipZ][ipHe2s1S][ipHe1s1S].WLAng = 0.;

			/* opacity in two-photon continuum is not correct since treated as line,
			 * fix now */
			HeLines[ipZ][ipHe2s1S][ipHe1s1S].opacity /= 1e4f;

			/* Lya has special redistribution function */
			HeLines[ipZ][ipHe2p1P][ipHe1s1S].iRedisFun = 1;

			HeLines[ipZ][ipHe2p1P][ipHe1s1S].damprel = 
				(float)(
				/* this is sum of A's to n=2 for H by itself */
				HeLines[ipZ][ipHe2p1P][ipHe1s1S].Aul/
			  PI4/HeLines[ipZ][ipHe2p1P][ipHe1s1S].EnergyWN);
			/* needs the velocity to become the damping constant itself */
			HeLines[ipZ][ipHe2p1P][ipHe1s1S].damp = HeLines[ipZ][ipHe2p1P][ipHe1s1S].damprel/1e6f;
		}
	}

	/* above used screened hydrogenic, not real good for atomic helium.  
	 * Fix up lowe levels of atomic helium with good As */
	for( ipLo=0; ipLo<NHE1AS; ++ipLo )
	{
		 float save;
		assert( fabs( He1AsLam[ipLo] - 
			HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].WLAng)/He1AsLam[ipLo] < 0.01 );

		HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].Aul = (float)He1AsAul[ipLo];

		save = HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].gf;
		HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].gf = 
		  (float)(GetGF(HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].Aul,
		  HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].EnergyWN,
		  HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].gHi));

		assert(HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].gf > 0.);
		assert(fabs(HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].gf-save)/save < 0.4);

		/* derive the abs coef, call to function is gf, wl (A), g_low */
		save = HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].opacity;
		HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].opacity = 
		  (float)(abscf(HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].gf,
		  HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].EnergyWN,
		  HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].gLo));

		HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].opacity = 
			MAX2(0.f, HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].opacity );
		assert( fabs(save-HeLines[ipHELIUM][ipHe1AsHi[ipLo]][ipHe1AsLo[ipLo]].opacity)/save < 0.4);
	}

	/* create spectroscopic designation of labels */
	for( ipLo=0; ipLo < nHeLevels[ipHELIUM]; ++ipLo )
	{
		sprintf( helike.chLevel[ipLo] , "%li %c%c", 
				he_levels[ipLo].n,
				chSpin[he_levels[ipLo].s],
				chL[MIN2(5,he_levels[ipLo].l)] );
	}

	/* option to create trace printout */
	if( trace.lgTrace && trace.lgHeBug )
	{

		/* only do helium itself for now */
		ipZ = 1;
		/* next print designation of levels */
		fprintf(ioQQQ,"index\tdesig\tE(wn,He)\tE(wn,O)\tE(wn,Fe)\n");
		for( ipHi=1; ipHi<need; ++ipHi )
		{
			float enHe , enO, enFe;
			if( ipHi < nHeLevels[ipHELIUM] )
			{
				enHe = HeLines[ipHELIUM][ipHi][0].EnergyWN;
			}
			else
			{
				enHe = 0.;
			}
			if( ipHi < nHeLevels[ipOXYGEN] )
			{
				enO = HeLines[ipOXYGEN][ipHi][0].EnergyWN;
			}
			else
			{
				enO = 0.;
			}
			if( ipHi < nHeLevels[ipIRON] )
			{
				enFe = HeLines[ipIRON][ipHi][0].EnergyWN;
			}
			else
			{
				enFe = 0.;
			}
			fprintf(ioQQQ,"%li\t%li %c%c\t%.5e\t%.5e\t%.5e\n", 
				ipHi, 
				he_levels[ipHi].n,
				chSpin[he_levels[ipHi].s],
				chL[MIN2(5,he_levels[ipHi].l)] , 
				enHe ,
				enO ,
				enFe 
				);
		}
		for( ipLo=0; ipLo<nHeLevels[ipHELIUM]-1 ; ++ipLo )
		{
			for( ipHi=ipLo+1; ipHi<nHeLevels[ipHELIUM] ; ++ipHi )
			{
				fprintf(ioQQQ," %li\t%li\t%li %c%c\t%li %c%c\n",
					ipLo , ipHi ,
					he_levels[ipLo].n,
					chSpin[he_levels[ipLo].s],
					chL[MIN2(5,he_levels[ipLo].l)] , 
					he_levels[ipHi].n,
					chSpin[he_levels[ipHi].s],
					chL[MIN2(5,he_levels[ipHi].l)]
					);
			}
		}
	}

	/* some wavelengths of HeI are expected to be exact */
	HeLines[ipHELIUM][ipHe3d3D][ipHe2p3P].WLAng = 5876.;
	HeLines[ipHELIUM][ipHe4d3D][ipHe2p3P].WLAng = 4471.;
	HeLines[ipHELIUM][ipHe3d1D][ipHe2p1P].WLAng = 6678.;
	HeLines[ipHELIUM][ipHe2p3P][ipHe2s3S].WLAng = 10830.;
	HeLines[ipHELIUM][ipHe2p1P][ipHe2s1S].WLAng = 20600.;
	HeLines[ipHELIUM][ipHe3s3S][ipHe2p3P].WLAng = 7065.;
	HeLines[ipHELIUM][ipHe3p3P][ipHe2s3S].WLAng = 3889.;

	/* free the memeory allocated for the he level structure */
	free(he_levels );

	/* define excitation and ionization temperatures for he-like iso sequence */
	for( ipZ=1; ipZ < LIMELM; ipZ++ )
	{
		/* define these for H and He always */
		if( ipZ < 2 || abundances.lgElmtOn[ipZ] )
		{
			/* PH1 args are shell, num elec, ipZ
			 * energy of ground level in Ryd */
			helike.HeLevNIonRyd[ipZ][0] = PH1COM.PH1[0][1][ipZ][0]/EVRYD* 0.9998787;

			/* define ionization energies for all excited levels */
			for( ipLo=1; ipLo<nHeLevels[ipZ] ; ++ipLo )
			{
				helike.HeLevNIonRyd[ipZ][ipLo] = 
					helike.HeLevNIonRyd[ipZ][0] - HeLines[ipZ][ipLo][0].EnergyRyd ;
				assert( helike.HeLevNIonRyd[ipZ][ipLo]> 0. );
			}

			/* geneate pointers that will blow if used */
			for( ipLo=ipHe1s1S; ipLo < (nHeLevels[ipZ] - 1); ipLo++ )
			{
				for( ipHi=ipLo + 1; ipHi <nHeLevels[ipZ]; ipHi++ )
				{
					HeLines[ipZ][ipH2p][ipH1s].ipCont = INT_MIN;
				}
			}
			for( ipLo=0; ipLo <nHeLevels[ipZ]; ipLo++ )
			{
				helike.herec[ipZ][ipLo][ipRecNetEsc] = 1.;
				helike.herec[ipZ][ipLo][ipRecEsc] = 1.;
				helike.HeOpacRatio[ipZ][ipLo] = 0.;
			}
		}
	}

	/* get the collision strength data for the He 1 lines */

	/* check on path is file not here and path set */
	/* path was parsed in getset */
	if( lgDataPathSet == TRUE )
	{
		/*path set, so look only there */
		strcpy( chFilename , chDataPath );
		strcat( chFilename , "he1_cs.dat" );
	}
	else
	{
		/* path not set, check local space only */
		strcpy( chFilename , "he1_cs.dat" );
	}

	if( trace.lgTrace )
		fprintf( ioQQQ," HeCreate opening he1_cs.dat:");

	if( ( ioDATA = fopen( chFilename , "r" ) ) == NULL )
	{
		fprintf( ioQQQ, " HeCreate could not open he1_cs.dat\n" );
		if( lgDataPathSet == TRUE )
			fprintf( ioQQQ, " even tried path\n" );

		if( lgDataPathSet == TRUE )
		{
			fprintf( ioQQQ, " HeCreate could not open he1_cs.dat\n");
			fprintf( ioQQQ, " path is ==%s==\n",chDataPath );
			fprintf( ioQQQ, " final path is ==%s==\n",chFilename );
		}

		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/* check that magic number is ok */
	if( fgets( chLine , (int)sizeof(chLine) , ioDATA ) == NULL )
	{
		fprintf( ioQQQ, " HeCreate could not read first line of he1_cs.dat.\n");
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}
	i = 1;
	i1 = (long)FFmtRead(chLine,&i,LINELENGTH,&lgEOL);
	i2 = (long)FFmtRead(chLine,&i,LINELENGTH,&lgEOL);
	i3 = (long)FFmtRead(chLine,&i,LINELENGTH,&lgEOL);

	/* the following is the set of numbers that appear at the start of level1.dat 00 10 26 */
	if( ( i1 != 01 ) || ( i2 != 02 ) || ( i3 != 20 ) )
	{
		fprintf( ioQQQ, 
			" HeCreate: the version of he1_cs.dat is not the current version.\n" );
		fprintf( ioQQQ, 
			" I expected to find the number 01 02 20 and got %li %li %li instead.\n" ,
			i1 , i2 , i3 );
		fprintf( ioQQQ, "Here is the line image:\n==%s==\n", chLine );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/* get the array of temperatures */
	lgHIT = FALSE;
	while( fgets( chLine , (int)sizeof(chLine) , ioDATA ) != NULL )
	{
		/* only look at lines without '#' in first col */
		if( chLine[0] != '#')
		{
			lgHIT = TRUE;
			helike.nCS = 0;
			lgEOL = FALSE;
			i = 1;
			while( !lgEOL && helike.nCS < HE1CSARRAY)
			{
				CSTemp[helike.nCS] = FFmtRead(chLine,&i,LINELENGTH,&lgEOL);
				++helike.nCS;
			}
			break;
		}
	}
	--helike.nCS;
	if( !lgHIT )
	{
		fprintf( ioQQQ, " HeCreate could not find line in CS temperatures in he1_cs.dat.\n");
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}

	/* create space for array of CS values, if not already done */
	if( (helike.HeCS = (float ****)malloc(sizeof(float ***)*(unsigned)LIMELM ) )==NULL )
	{
		fprintf( ioQQQ, 
			" HeCreate could not malloc helike.HeCS 1\n" );
		puts( "[Stop in HeCreate]" );
		cdEXIT(1);
	}
	for( ipZ=1; ipZ<LIMELM; ++ipZ )
	{
		/* only grab core for elements that are turned on */
		/* for now only do he itself, since we have no other data */
		if( ipZ < 2 /*|| abundances.lgElmtOn[ipZ]*/ )
		{
			if( (helike.HeCS[ipZ] = 
				(float***)malloc(sizeof(float**)*(unsigned)(nHeLevels[ipZ]) ))==NULL )
			{
				fprintf( ioQQQ, 
					" HeCreate could not second malloc helike.HeCS 1\n" );
				puts( "[Stop in HeCreate]" );
				cdEXIT(1);
			}
			/* avoid allocating 0 bytes, some OS return NULL pointer, PvH */
			helike.HeCS[ipZ][0] = NULL;
			for( ipHi=1; ipHi < nHeLevels[ipZ] ;++ipHi )
			{
				if( (helike.HeCS[ipZ][ipHi] = 
					(float**)malloc(sizeof(float*)*(unsigned)ipHi ))==NULL )
				{
					fprintf( ioQQQ, 
						" HeCreate could not third malloc helike.HeCS 1\n" );
					puts( "[Stop in HeCreate]" );
					cdEXIT(1);
				}

				for( ipLo=0; ipLo<ipHi; ++ipLo )
				{
					if( (helike.HeCS[ipZ][ipHi][ipLo] = 
						(float*)malloc(sizeof(float)*(unsigned)helike.nCS ))==NULL )
					{
						fprintf( ioQQQ, 
							" HeCreate could not 4th malloc helike.HeCS 1\n" );
						puts( "[Stop in HeCreate]" );
						cdEXIT(1);
					}
					/* fill them with the default value, in case no better comes along */
					for( j=0; j< helike.nCS; ++j )
					{
						helike.HeCS[ipZ][ipHi][ipLo][j] = 0.1f;
					}
				}
			}
		}
	}

	/* now read in the CS data */
	lgHIT = FALSE;
	ipZ = 1;
	while( fgets( chLine , (int)sizeof(chLine) , ioDATA ) != NULL )
	{
		char *chTemp;
		/* only look at lines without '#' in first col */
		if( (chLine[0] == ' ') || (chLine[0]=='\n') )
			break;
		if( chLine[0] != '#')
		{
			lgHIT = TRUE;

			/* get lower and upper level index */
			j = 1;
			ipLo = (long)FFmtRead(chLine,&j,LINELENGTH,&lgEOL);
			ipHi = (long)FFmtRead(chLine,&j,LINELENGTH,&lgEOL);
			assert( ipLo < ipHi );
			assert( ipHi < nHeLevels[ipZ] );

			chTemp = chLine;
			/* skip over 4 tabs to start of cs data */
			for( i=0; i<3; ++i )
			{
				if( (chTemp = strchr( chTemp, '\t' )) == NULL )
				{
					fprintf( ioQQQ, " HeCreate no init cs\n" );
					puts( "[Stop in HeCreate]" );
					cdEXIT(1);
				}
				++chTemp;
			}

			for( i=0; i<helike.nCS; ++i )
			{
				float a;
				if( (chTemp = strchr( chTemp, '\t' )) == NULL )
				{
					fprintf( ioQQQ, " HeCreate not scan cs\n" );
					puts( "[Stop in HeCreate]" );
					cdEXIT(1);
				}
				++chTemp;
				sscanf( chTemp , "%e" , &a );
				helike.HeCS[ipZ][ipHi][ipLo][i] = a;
			}
		}
	}

	/* close the data file */
	fclose( ioDATA );

	/* option to print cs data for AGN */
	/* create spectroscopic designation of labels */
	{
		/* option to print particulars of some line when called
		 * a prettier print statement is near where chSpin is defined below*/
		/*@-redef@*/
		enum {AGN=FALSE};
		/*@+redef@*/
		if( AGN )
		{
#			define NTEMP 6
			double te[NTEMP]={6000.,8000.,10000.,15000.,20000.,25000. };
			double telog[NTEMP] ,
				cs ,
				ratecoef;
			ipZ = 1;
			fprintf( ioQQQ,"trans");
			for(i=0; i<NTEMP; ++i)
			{
				telog[i] = log10( te[i] );
				fprintf( ioQQQ,"\t%.3e",te[i]);
			}
			for(i=0; i<NTEMP; ++i)
			{
				fprintf( ioQQQ,"\t%.3e",te[i]);
			}
			fprintf(ioQQQ,"\n");
			for( ipLo=0; ipLo < nHeLevels[ipHELIUM]-1; ++ipLo )
			{
				for( ipHi=ipLo+1; ipHi< nHeLevels[ipHELIUM]; ++ipHi )
				{
					/* print the designations of the lower and upper levels */
					fprintf( ioQQQ,"%s - %s",
						helike.chLevel[ipLo] , helike.chLevel[ipHi] );

					/* print the interpolated collision strengths */
					for(i=0; i<NTEMP; ++i)
					{
						phycon.alogte = (float)telog[i];
						/* print cs */
						cs = HeCSInterp( ipZ , ipHi , ipLo );
						fprintf(ioQQQ,"\t%.2e", cs );
					}

					/* print the rate coefficients */
					for(i=0; i<NTEMP; ++i)
					{
						phycon.alogte = (float)telog[i];
						phycon.te = (float)pow(10.,telog[i] );
						cs = HeCSInterp( ipZ , ipHi , ipLo );
						/* collisional deexcitation rate */
						ratecoef = cs/sqrt(phycon.te) *8.629e-6/HeLines[ipZ][ipHi][ipLo].gLo *
							sexp( HeLines[ipZ][ipHi][ipLo].EnergyK / phycon.te );
						fprintf(ioQQQ,"\t%.2e", ratecoef );
					}
					fprintf(ioQQQ,"\n");
				}
			}
			exit(1);
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->HeCreate()\n", debug_fp );
#	endif
	return;
}

/*===================================================================================*/
/* HeRecom - do recomb coef for He, called by HeLike */
static void HeRecom( 
			/* ipZ on the c scale, He is 1 */
			long ipZ )
{
	static double ditcrt[LIMELM]= {0.,0.,1.20E+04,1.10E+04,4.40E+05,7.00E+05,1.00E+06,1.50E+06,
	  1.50E+06,3.80E+06,3.40E+04,4.00E+06,5.50E+06,6.30E+06,5.00E+06,9.00E+06,1.40E+07,
	  1.40E+07,2.00E+05,2.00E+05,2.00E+05,1.00E+05,7.00E+04,4.00E+04,6.00E+06,1.00E+11,
	  1.00E+20,1.00E+20,1.00E+20,1.00E+20};

	static double dicoef0[LIMELM]= {0.,0.,6.15E-03,1.62E-03,4.78E-02,3.22E-02,4.61E-02,6.23E-02,
	  2.52E-01,1.44E-01,1.80E-01,1.44E-01,3.40E+00,1.89E-01,4.02E-01,2.41E-01,1.10E+01,2.97E-01,
	  5.49E-01,3.55E-01,2.30E-01,1.40E-01,1.10E-01,4.10E-02,7.47E-01,5.19E-01,5.75E-01,5.75E-01,
	  5.75E-01,5.75E-01};

	static double dite0[LIMELM]= {0.,0.,1.41E+05,8.19E+04,3.44E+06,4.06E+06,5.44E+06,7.01E+06,1.40E+07,
	  1.50E+07,1.30E+07,1.50E+07,1.70E+07,1.99E+07,2.41E+07,2.54E+07,2.80E+07,3.13E+07,3.65E+07,
	  3.78E+07,1.09E+06,9.62E+05,7.23E+05,4.23E+05,5.84E+07,6.01E+07,6.81E+07,6.81E+07,6.81E+07,
	  6.81E+07};

	static double dicoef1[LIMELM]= {0.,0.,5.88E-02,3.43E-01,3.62E-01,3.15E-01,3.09E-01,3.04E-01,
	  3.04E-01,2.96E-01,0.00E+00,2.91E-01,0.00E+00,2.86E-01,2.98E-01,2.81E-01,0.00E+00,2.77E-01,
	  2.92E-01,2.75E-01,3.60E+00,4.90E+00,1.60E+00,4.20E+00,2.84E-01,2.79E-01,0.00E+00,2.86E-01,
	  2.86E-01,2.86E-01};

	static double dite1[LIMELM]= {0.,0.,1.41E+05,1.59E+05,5.87E+05,8.31E+05,1.14E+06,1.47E+06,
	  1.80E+06,2.24E+06,0.00E+00,3.09E+06,0.00E+00,4.14E+06,4.67E+06,5.30E+06,0.00E+00,6.54E+06,
	  7.25E+06,7.68E+06,1.10E+07,8.34E+06,1.01E+07,1.07E+07,1.17E+07,9.97E+06,0.00E+00,9.08E+06,
	  9.08E+06,9.08E+06};


	/* set of rad rec coef for increasing levels above ground, at 10^4K for He0, from Table 1 of
	 >>refer	helium	recombination	Benjamin, R.A., Skillman, E.D., & Smits, D.P., 1999, ApJ, 514, 307 
	 */
	static double ScaleRadRec[29] ={0.,1.49E+00,5.55E-01,5.61E+00,1.26E+00,3.72E-01,1.62E-01,1.95E+00,
	1.33E+00,4.31E-01,5.01E-01,1.50E-01,7.00E-02,9.04E-01,8.29E-01,2.67E-01,4.15E-01,1.38E-01,
	2.43E-01,7.51E-02,3.66E-02,4.84E-01,4.99E-01,1.60E-01,3.65E-01,1.22E-01,1.34E-01,4.46E-02,
	1.35E-01};


	long ipLo,
		n;
	float TotRadRec = 0.;
	double rectot , SumRec,
		checksum;

#	ifdef DEBUG_FUN
	fputs( "<+>HeRecom()\n", debug_fp );
#	endif

	fixit(); /* self ionization correction included, but not ots */
	/* evaluate recombination escape probability for all levels */

	if( ipZ==1 )
	{
		/* this is helium itself, which we want to do well because of its abundance
		 * re1rtt is total rec rate, including ground, 
		 * we want to exclude ground in this sum */
		TotRadRec = he1rb.he1rtt + herecCom.rectri - phe1lv.he1rec[0][0]*phe1lv.he1rec[ipRecNetEsc][0];
		assert( TotRadRec > 0. );
		helike.herec[ipZ][0][ipRecRad] = 1.6e-11f/phycon.sqrte;	
	}
	else
	{
		/* grab total radiative recombination rates from rrfit, ipZ is atomic number on 
		 * physical scale, 2 is number of bound electrons in recombined species */
		rrfit(ipZ+1,2,phycon.te,&TotRadRec);
		helike.herec[ipZ][0][ipRecRad] = 0.;	
	}

	for( n=0; n<nHeLevels[ipZ]; ++n )
	{
		/* this is escape probability */
		helike.herec[ipZ][n][ipRecEsc] = (float)receff(helike.ipHeLevNIonCon[ipZ][n]);
		/* otsmin set to zero in zerologic, set to 1 with 
		 * the NO ON THE SPOT command */
		helike.herec[ipZ][n][ipRecEsc] = 
			(float)MAX2(helike.herec[ipZ][n][ipRecEsc], otsminCom.otsmin);

		/* net escape prob includes dest by background opacity */
		helike.herec[ipZ][n][ipRecNetEsc] = (float)MIN2(1.,helike.herec[ipZ][n][ipRecEsc]+
		  (1.-helike.herec[ipZ][n][ipRecEsc])*helike.HeOpacRatio[ipZ][n]);
		assert( helike.herec[ipZ][n][ipRecNetEsc] >= 0. );
	}

	/* keep track of total recombination */
	rectot = phycon.eden*(TotRadRec + rcota.CotaRate[ipZ-1]);

	/* recombinatinon on grain surface */
	rectot += gionrc.GrnIonRec;

	/* save the rate */
	RecomRate.RecombinRate[ipZ-1][ipZ] = (float)rectot;
	DestCrt.Recombine[ipZ-1][ipZ] = (float)rectot;

	/* for case of iron, add on Dima's fits to diel part, above was zero */
	if( ipZ==25 )
	{
		float DieRate;
		drfe(25,phycon.te,&DieRate);
		/*fprintf(ioQQQ,"%li %g %g\n", IonStage_C_Scale, RecomRate.RecombinRate[IonStage_C_Scale][NELEM-1], DieRate*phycon.eden );*/
		rectot += (float)(DieRate*phycon.eden);
	}
	/* Burgess dielectronic recombination */
	else if( phycon.te > (float)(ditcrt[ipZ]*0.1) )
	{
		double diburg;
		/* the suppression factor is on a charge scale, so rec to neutral
		 * is always 0, hence the ipZ-1 */
		diburg = supprs.DielSupprs[0][ipZ-1]/phycon.te32*
		  dicoef0[ipZ]*exp(-dite0[ipZ]/phycon.te)*
		  (1. + dicoef1[ipZ]*
		  sexp(dite1[ipZ]/phycon.te));
		rectot += (float)(diburg* phycon.eden);
	}

	{
		/* following should be set true to print recombination rates */
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG /*&& (iteration==2)*/ )
		{
			if( ipZ==1 )
				fprintf(ioQQQ,"\n");
			fprintf(ioQQQ,"%li rec\t%.4e", ipZ,
				RecomRate.RecombinRate[ipZ-1][ipZ] );

			fprintf(ioQQQ,"\t%.4e\t%.4e\n", rectot ,
				(RecomRate.RecombinRate[ipZ-1][ipZ]-rectot)/RecomRate.RecombinRate[ipZ-1][ipZ] );
			/*fprintf( ioQQQ," %li\t%.2e \n", ipZ, RecomRate.RecombinRate[ipZ-1][ipZ] );*/
		}
	}

	/* first sum up rec coef from above */
	SumRec = 0.;
	for( ipLo=1; ipLo < MIN2( 29, nHeLevels[ipZ] ); ipLo++ )
	{
		/* find total of these recs */
		SumRec += ScaleRadRec[ipLo];
	}
	/* detault number of levels is 28 */
	if( nHeLevels[ipZ] > 29 )
	{
		/* assume some small recom contribution to high levels, if atom extends beyond 29 levels */
		SumRec += (nHeLevels[ipZ]-28.)*0.1 ;
	}

	/* this is scale factor for each level */
	checksum = 0.;
	for( ipLo=1; ipLo < MIN2( 29, nHeLevels[ipZ] ); ipLo++ )
	{
		/* rectot is total recom rate per unit vol, with eden included, 
		 * we want to save rec per ion */
		helike.herec[ipZ][ipLo][ipRecRad] = (float)(rectot/phycon.eden*ScaleRadRec[ipLo]/SumRec);

		checksum += helike.herec[ipZ][ipLo][ipRecRad]*helike.herec[ipZ][ipLo][ipRecNetEsc]*(float)phycon.eden;
		/*fprintf(ioQQQ,"%li %.2e\n", ipLo, 
		helike.herec[ipZ][ipLo][ipRecRad]*helike.herec[ipZ][ipLo][ipRecNetEsc]*(float)phycon.eden);*/
	}

	/* this is scale factor for each level */
	for( ipLo=29; ipLo < nHeLevels[ipZ]; ipLo++ )
	{
		/* herec is per electron */
		helike.herec[ipZ][ipLo][ipRecRad] = (float)(rectot/phycon.eden*0.1/SumRec);
	}

	return;

#	ifdef DEBUG_FUN
	fputs( " <->HeRecom()\n", debug_fp );
#	endif
}

/*===================================================================================*/
/* evaluate collisional rates */
static void HeCollid( long ipZ)
{

	long int ipLo , ipHi ;
	float DimaRate, crate;
	static float TeUsed[LIMELM]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
		0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};

#	ifdef DEBUG_FUN
	fputs( "<+>HeCollid()\n", debug_fp );
#	endif

	if( TeUsed[ipZ] != phycon.te )
	{
		TeUsed[ipZ] = phycon.te;

		/* this branch of helium, a special case since extensive quantal calculations exist */
		if( ipZ == 1 )
		{
			for( ipLo=0; ipLo < (nHeLevels[ipZ] - 1); ipLo++ )
			{
				for( ipHi=ipLo + 1; ipHi <nHeLevels[ipZ]; ipHi++ )
				{
					/* get collision strength */
					HeLines[ipZ][ipHi][ipLo].cs = HeCSInterp( ipZ , ipHi , ipLo );

					/* collisional deexcitation rate, lgHeClON is normally TRUE (1),
					 * and set FALSE (0) with atom he-like collisions off command */
					HeLines[ipZ][ipHi][ipLo].ColUL = 
						(float)(HeLines[ipZ][ipHi][ipLo].cs/
					  phycon.sqrte*8.629e-6/HeLines[ipZ][ipHi][ipLo].gHi*helike.lgHeClON );
					/*This is downward collision rate */

					helike.helbolt[ipZ][ipLo][ipHi] = 
						sexp( HeLines[ipZ][ipHi][ipLo].EnergyK / phycon.te );

					/* I don't think that this is used for anything, and should be converted
					 * into a triangular matrix */
					helike.helbolt[ipZ][ipHi][ipLo] = -FLT_MAX;
				}
				helike.hecolnc[ipZ][ipLo] = 0.;
			}
			/* above loop left off highest level */
			helike.hecolnc[ipZ][nHeLevels[ipZ] - 1] = 0.;
		}
		/* this branch for everything else - mostly no data at all */
		else
		{
			for( ipLo=0; ipLo < (nHeLevels[ipZ] - 1); ipLo++ )
			{
				for( ipHi=ipLo + 1; ipHi <nHeLevels[ipZ]; ipHi++ )
				{
					HeLines[ipZ][ipHi][ipLo].cs = 0.1f;

					/* collisional deexcitation rate */
					HeLines[ipZ][ipHi][ipLo].ColUL = 
						(float)(HeLines[ipZ][ipHi][ipLo].cs/
					  phycon.sqrte*8.629e-6/HeLines[ipZ][ipHi][ipLo].gHi*helike.lgHeClON );
					/*This is downward collision rate */

					helike.helbolt[ipZ][ipLo][ipHi] = 
						sexp( HeLines[ipZ][ipHi][ipLo].EnergyK / phycon.te );

					helike.helbolt[ipZ][ipHi][ipLo] = -FLT_MAX;
				}
				helike.hecolnc[ipZ][ipLo] = 0.;
			}
			/* above loop left off highest level */
			helike.hecolnc[ipZ][nHeLevels[ipZ] - 1] = 0.;
		}
	}

	/* cfit( Atom num (H=1), Num elec, >=0 */
	cfit( ipZ+1, 2 , phycon.te , &DimaRate );

	crate = DimaRate*(float)phycon.eden;
	/* NB hecolnc does not include secondaries since not total, but CollidRate does,
	 * since is total */
	helike.hecolnc[ipZ][0] = DimaRate;

	/* total collisional ionization rate 
	 * with both thermal and suprathermal electrons */
	/* there is no attempt to use scaled csupra here, since this is mostly 
	 * ionic and csupra will be zero.  For He itself, other routine is used.
	 * when this routine becomes fundamental for He a scale factor should
	 * use used.  TH85 Tab 10 suggests should be 0.5, but this is for
	 * their background rate - the rate derived here is self-consistent with unity */
	CollIonRate.CollidRate[0][ipZ-1][ipZ] = crate + Secondaries.csupra;

	/* cooling due to collisional ionization, which only includes thermal */
	CollIonRate.CollidRate[1][ipZ-1][ipZ] = (float)(crate*
		rfield.anu[Heavy.ipHeavy[ipZ-1][ipZ]-1]* EN1RYD);

	/* collisional ionizaiton of atomic helium */
	if( ipZ == 1 )
	{
#		include "he3c.h"
		helike.hecolnc[ipZ][ipHe2s3S] = he3c.c2sion/ phycon.eden;
		helike.hecolnc[ipZ][ipHe2p3P] = he3c.c2pion/ phycon.eden;
		helike.hecolnc[ipZ][ipHe2s1S] = 0.;
		helike.hecolnc[ipZ][ipHe2p1P] = 0.;
	}

#	ifdef DEBUG_FUN
	fputs( " <->HeCollid()\n", debug_fp );
#	endif
}

/*===================================================================================*/
/* evaluate photoionization rates  */
static void HePhoto( long ipZ)
{
	long int ipLo,
		ns,
		iplow , iphi , ipop;
	double funduc , cind;

#	ifdef DEBUG_FUN
	fputs( "<+>HePhoto()\n", debug_fp );
#	endif

	/* ground state will be only one for now */
	/* ipZ-1 is stage of ionization */
	ns = 0;
	iplow = opac.ipElement[0][ns][ipZ-1][ipZ];
	iphi = opac.ipElement[1][ns][ipZ-1][ipZ];
	ipop = opac.ipElement[2][ns][ipZ-1][ipZ];

	helike.hegamnc[ipZ][ipHe1s1S] = 
		GammaBn(iplow,iphi,
	  ipop,helike.HeLevNIonRyd[ipZ][0],&funduc , &cind)*PhotRate.PhotScaleOn;

	/* the heating rate */
	PhotRate.PhotoRate[0][ns][ipZ-1][ipZ] = helike.hegamnc[ipZ][ipH1s];
	PhotRate.PhotoRate[1][ns][ipZ-1][ipZ] = heat.HeatLowEnr*PhotRate.PhotScaleOn;
	PhotRate.PhotoRate[2][ns][ipZ-1][ipZ] = heat.HeatHiEnr*PhotRate.PhotScaleOn;

#	if 0
	if( ipZ==1 )
		fprintf(ioQQQ,"\n");
	fprintf(ioQQQ,"%li\t%.4e", ipZ,
		PhotRate.PhotoRate[0][0][ipZ-1][ipZ] );

	fprintf(ioQQQ,"\t%.4e\t%.4e\n", helike.hegamnc[ipZ][ipHe1s1S] ,
		(PhotRate.PhotoRate[0][0][ipZ-1][ipZ]-helike.hegamnc[ipZ][ipHe1s1S])/PhotRate.PhotoRate[0][0][ipZ-1][ipZ] );
#	endif

	/* rest zero */
	for( ipLo=ipHe2s3S; ipLo < nHeLevels[ipZ]; ipLo++ )
	{
		/* HColOn is option to turn off collisions, "hydrogen collis off" comnd */
		helike.hegamnc[ipZ][ipLo] = 0.;
	}

	/* except for HeI itself, use he routines */
	if( ipZ==1 )
	{
#		include "nhe3lvl.h"
#		include "he3gams.h"
		helike.hegamnc[ipZ][ipHe2s3S] = he3gams.he3gam[0];
		helike.hegamnc[ipZ][ipHe2p3P] = he3gams.he3gam[1];
		helike.hegamnc[ipZ][ipHe2s1S] = phe1lv.he1gam[1];
		helike.hegamnc[ipZ][ipHe2p1P] = phe1lv.he1gam[1];
	}

#	ifdef DEBUG_FUN
	fputs( " <->HePhoto()\n", debug_fp );
#	endif
}

/*===================================================================================*/
/* solve for level populations  */
static void HeLevel( long ipZ)
{
	long int n,
		ipHi,
		ipLo,
		j,
		job,
		level,
		nerror1,
		nerror2;
	double create,
		destroy;
	int lgNegPop;
	static double *pops;
	static int lgFIRST=TRUE;

#	ifdef DEBUG_FUN
	fputs( "<+>HeLevel()\n", debug_fp );
#	endif

	assert(ipZ < LIMELM );

	if( lgFIRST )
	{
		lgFIRST = FALSE;
		if( (pops = (double *)malloc(sizeof(double )*(unsigned)Limit_Helium ) )==NULL )
		{
			fprintf( ioQQQ, 
				" HeLevel could not malloc pops 1\n" );
			puts( "[Stop in HeLevel]" );
			cdEXIT(1);
		}
	}

	/* sum rec coef */
	create = 0.;
	for( n=0; n < nHeLevels[ipZ]; n++ )
	{
		float fac = helike.herec[ipZ][n][ipRecRad]*helike.herec[ipZ][n][ipRecNetEsc]*(float)phycon.eden;
		assert( fac >= 0. );
		/* note that create will sum up to zero if helium cannot be ionized, since cont esc prob 0 */
		create += fac;
		z[nHeLevels[ipZ]][n] = 
			      fac;
		/*fprintf(ioQQQ,"%li %.2e\n", n, 
		helike.herec[ipZ][n][ipRecRad]*helike.herec[ipZ][n][ipRecNetEsc]*(float)phycon.eden);*/
	}

	/* add on charge transfer recombination*/
	/* add it to ground for now, can't really do any better */
	z[nHeLevels[ipZ]][0] += 
		ChargTran.HeCharExcRec[ipZ][ipZ-1]*xIonFracs[1][ipHELIUM] + 
		ChargTran.HCharExcRec[ipZ][ipZ-1]*xIonFracs[1][ipHYDROGEN] ;

	{
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG /*&& (iteration==2)*/ )
		{
			if( ipZ==1 )
				fprintf(ioQQQ,"\n");
			fprintf(ioQQQ,"%li rec\tstore=%.4e", ipZ,
				RecomRate.RecombinRate[ipZ-1][ipZ] );

			fprintf(ioQQQ,"\there=%.4e\tratio=%.4e\n", create ,
				(RecomRate.RecombinRate[ipZ-1][ipZ]-create)/create );
			/*fprintf( ioQQQ," %li\t%.2e \n", ipZ, RecomRate.RecombinRate[ipZ-1][ipZ] );*/
		}
	}

	/* add on charge transfer recombination*/
	create += ChargTran.HeCharExcRec[ipZ][ipZ-1]*xIonFracs[1][ipHELIUM] + 
		ChargTran.HCharExcRec[ipZ][ipZ-1]*xIonFracs[1][ipHYDROGEN] ;

	/* total destruction rate, photo and collis ioniz first, CollidRate already
	 *includes suprathermals */
	destroy = helike.hegamnc[ipZ][0] + CollIonRate.CollidRate[0][ipZ-1][ipZ];

	/* here is charge transfer ionization */
	destroy += ChargTran.HeCharExcIon[ipZ][ipZ-1]*xIonFracs[2][ipHELIUM] + 
	  ChargTran.HCharExcIon[ipZ][ipZ-1]*xIonFracs[2][ipHYDROGEN];
	DestCrt.xIonize[ipZ-1][ipZ] = destroy;

	/* ratio of ion to atom */
	if( create > 0. )
	{
		helike.HeIonSimple[ipZ] = (float)(destroy / create);
	}
	else
	{
		helike.HeIonSimple[ipZ] = 0.;
	}
#	define TOOSMALL 1e-10
	if( helike.HeIonSimple[ipZ] < TOOSMALL )
	{
		for( n=0; n < nHeLevels[ipZ]; n++ )
		{
			/*hydro.hbn[ipZ][i] = 1.;*/
			pops[n] =  0.;
		}
		lgNegPop = FALSE;
		helike.HeIonFrac[ipZ] = 0.;
	}
	else
	{
		/* master balance equation, use when significant population */
		for( level=0; level < nHeLevels[ipZ]; level++ )
		{
			/* 
			 * all process depopulating level
			 */
			/* there is no attempt to use scaled csupra here, since this is mostly 
			 * ionic and csupra will be zero.  For He itself, other routine is used.
			 * when this routine becomes fundamental for He a scale factor should
			 * use used.  TH85 Tab 10 suggests should be 0.5, but this is for
			 * their background rate - the rate derived here is self-consistent with unity */
			/* NB collisional ionization rate already includes suprathermals */
			z[level][level] = helike.hegamnc[ipZ][level] + 
			  helike.hecolnc[ipZ][level]* phycon.eden ;
			/* in final version move this up a line, add to all levels 
			 * for now only add to groud to keep parallel to bidiag */
			if( level==0 ) z[level][level] +=Secondaries.csupra;

			/* all processes populating level from below */
			for( ipLo=0; ipLo < level; ipLo++ )
			{
				z[ipLo][level] = 
					-HeLines[ipZ][level][ipLo].ColUL* phycon.EdenHCorr * 
				  HeLines[ipZ][level][ipLo].gHi/HeLines[ipZ][level][ipLo].gLo*
				  helike.helbolt[ipZ][ipLo][level]  -
				  HeLines[ipZ][level][ipLo].pump ;

				/* pumping out of here to lower level */
				z[level][level] += HeLines[ipZ][level][ipLo].pump *
				  HeLines[ipZ][level][ipLo].gLo/HeLines[ipZ][level][ipLo].gHi;

				/* collisions out of here to lower level */
				z[level][level] += 
					HeLines[ipZ][level][ipLo].ColUL * phycon.EdenHCorr;

				/* radiative decays out of here to lower level */
				z[level][level] += 
					HeLines[ipZ][level][ipLo].Aul*
				  (HeLines[ipZ][level][ipLo].Pesc + 
				  HeLines[ipZ][level][ipLo].Pdest);
			}

			/* all processes populating level from above */
			for( ipHi=level + 1; ipHi < nHeLevels[ipZ]; ipHi++ )
			{
				double RadDecay;
				RadDecay =
					HeLines[ipZ][ipHi][level].Aul*
				  (HeLines[ipZ][ipHi][level].Pesc + 
				  HeLines[ipZ][ipHi][level].Pdest);

				z[ipHi][level] = 
					-(RadDecay + 
				  HeLines[ipZ][ipHi][level].pump*
				  HeLines[ipZ][ipHi][level].gLo/HeLines[ipZ][ipHi][level].gHi + 
				  HeLines[ipZ][ipHi][level].ColUL *phycon.EdenHCorr);

				/* pumping out of here to upper level */
				z[level][level] += HeLines[ipZ][ipHi][level].pump ;

				/* collisions out of here to upper level */
				z[level][level] += HeLines[ipZ][ipHi][level].ColUL *
					HeLines[ipZ][ipHi][level].gHi / HeLines[ipZ][ipHi][level].gLo *
					helike.helbolt[ipZ][level][ipHi]*phycon.EdenHCorr;
			}
		}

		/* here is charge transfer ionization */
		z[0][0] += ChargTran.HeCharExcIon[ipZ][ipZ-1]*xIonFracs[2][ipHELIUM] + 
		  ChargTran.HCharExcIon[ipZ][ipZ-1]*xIonFracs[2][ipHYDROGEN];

		/* add on advection terms, normally zero */
		z[nHeLevels[ipZ]][ipH1s] += dynamics.Recomb[ipZ-1][ipZ];
		z[ipH1s][ipH1s] += dynamics.Photo;

		fixit();/* neglect secondary ionization for now - csupra occurs in above look [level][level] */
#		if 0
		if( Secondaries.Hx12[0][1] > 0. )
		{
			/* now add on supra thermal excitation */
			for( level=1; level < nHeLevels[ipZ]; level++ )
			{
				double RateUp , RateDown;

				RateUp = Secondaries.Hx12[MIN2(ipZ,1)][level];
				RateDown = RateUp * (double)HeLines[ipZ][level][ipH1s].gLo /
					(double)HeLines[ipZ][level][ipH1s].gHi;

				/* stuff in min after Hx12 evaluates to 0 for atom, or 1 for ion */
				/* total rate out of lower level */
				broken();/* these are all wrong */
				z[ipH1s][ipH1s] += RateUp;

				/* rate from the upper level to ground */
				z[level][ipH1s] -= RateDown ;

				/* rate from ground to upper level */
				z[ipH1s][level] -= RateUp ;

				z[level][level] += RateDown;  
			}
		}
#		endif

		if( trace.lgTrace && trace.lgHeBugFull && (ipZ == trace.ipZTrace) )
		{
			fprintf( ioQQQ, "  pop level     others => (HeliumLevelPop)\n" );
			for( n=0; n < nHeLevels[ipZ]; n++ )
			{
				fprintf( ioQQQ, "       HII%2ld", n );
				/* limit is <= since last has rec coef */
				for( j=0; j <= nHeLevels[ipZ]; j++ )
				{
					fprintf( ioQQQ," %.9e\t", z[j][n] );
				}
				fprintf( ioQQQ, "\n" );
			}
		}

		/* save matrix */
		for( ipLo=0; ipLo < nHeLevels[ipZ]; ipLo++ )
		{
			for( n=0; n < (nHeLevels[ipZ] + 1); n++ )
			{
				SaveZ[n][ipLo] = z[n][ipLo];
			}
		}

		/* which matrix solver? */
		if( strcmp(TypMatrx.chMatrix,"matin1 ") == 0 )
		{
			/* this is the usual matrix inversion method, slightly faster */
			/*matin1();*/
			nerror1 = 0;
			if( nerror1 != 0 )
			{
				fprintf( ioQQQ, " helike matrix error, stop.\n" );
				puts( "[Stop in helike]" );
				cdEXIT(1);
			}
		}

		/* this is the default */
		else if( strcmp(TypMatrx.chMatrix,"linpack") == 0 )
		{
			double *amat;
	#		ifdef AMAT
	#			undef AMAT
	#		endif
	#		define AMAT(I_,J_)	(*(amat+(I_)*(nHeLevels[ipZ])+(J_)))
			/* malloc space for the  1-d array */
			if( (amat=(double*)malloc( (sizeof(double)*(unsigned)((nHeLevels[ipZ])*(nHeLevels[ipZ])) ))) == NULL )
			{
				fprintf( ioQQQ, " helike malloc amat error\n" );
				puts( "[Stop in helike]" );
				cdEXIT(1);
			}

			/* this one may be more robust */
			for( j=0; j < nHeLevels[ipZ]; j++ )
			{
				for( n=0; n < nHeLevels[ipZ]; n++ )
				{
					/*amat[n][j] = z[n][j];*/
					AMAT(n,j) = z[n][j];
				}
				bvec[j] = z[nHeLevels[ipZ]][j];
			}
			DGETRF(nHeLevels[ipZ],nHeLevels[ipZ],
			  amat,(nHeLevels[ipZ]),ipiv,&nerror1);

			DGETRS('N',nHeLevels[ipZ],1,amat,(nHeLevels[ipZ]),ipiv,
			  bvec,(nHeLevels[ipZ]),&nerror2);

			if( nerror1 != 0 || nerror2 != 0 )
			{
				fprintf( ioQQQ, " helike dgetrs error\n" );
				puts( "[Stop in helike]" );
				cdEXIT(1);
			}
	#		if 0
	#		endif
			/* now put results back into z so rest of code treates only
			 * one case - as if matin1 had been used */
			for( n=0; n < nHeLevels[ipZ]; n++ )
			{
				z[nHeLevels[ipZ]+1][n] = bvec[n];
			}
			free( amat);
		}
		else if( strcmp(TypMatrx.chMatrix,"veclib ") == 0 )
		{
			double /*amat[LMHLVL+2][LMHLVL+2], */ rcond;
			/* Jason found this one on the Exemplar, distributed source just stops */
			fprintf( ioQQQ, " this has not been checked since H atom conv\n" );
			for( j=ipH1s; j <= nHLevels[ipZ]; j++ )
			{
				/*for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
				{
					amat[n][j] = z[n][j];
				}*/
				bvec[j] = z[nHLevels[ipZ]+1][j];
			}
			job = 0;
			rcond = 0.;
			/*dgeco((double*)amat,nHLevels[ipZ]+2,nHLevels+1,ipiv,rcond, work);
			dgesl((double*)amat,nHLevels[ipZ]+2,nHLevels+1,ipiv,bvec, job);*/
			dgeco((double*)z,nHeLevels[ipZ]+1,nHeLevels[ipZ],ipiv,rcond, work);
			dgesl((double*)z,nHeLevels[ipZ]+1,nHeLevels[ipZ],ipiv,bvec, job);
			/* now put results back into z so rest of code treates only
			 * one case - as if matin1 had been used */
			for( n=0; n < nHeLevels[ipZ]; n++ )
			{
				z[nHeLevels[ipZ]+1][n] = bvec[n];
			}
			puts( "[Stop in helike]" );
			cdEXIT(1);
		}
		else
		{
			fprintf( ioQQQ, " chMatrix type insane in helike, was%7.7s\n", 
			  TypMatrx.chMatrix );
			puts( "[Stop in helike]" );
			cdEXIT(1);
		}
		/* end of branches for which matrix solution, now check if valid */

		/* >>chng 00 dec 12, changed from n <= nHeLevels[ipZ] to n < nHeLevels[ipZ], PvH */
		for( n=0; n < nHeLevels[ipZ]; n++ )
		{
			/*hydro.hbn[ipZ][i] = 1.;*/
			pops[n] =  bvec[n];
		}

		/* this will eventually become the ratio of ionized to neutral hydrogenic 
		 * species, create sum of level pops per ion first */
		helike.HeIonFrac[ipZ] = 0.;
		lgNegPop = FALSE;
		for( level=0; level < nHeLevels[ipZ]; level++ )
		{
			/* check for non-positive level populations */
			if( pops[level] <= 0. )
				lgNegPop = TRUE;

			/* pops are actually populations relative to ion */
			helike.HeIonFrac[ipZ] += (float)pops[level];
		}
	}

	if( helike.HeIonFrac[ipZ] > SMALLFLOAT )
	{
		helike.HeIonFrac[ipZ] = 1.f/helike.HeIonFrac[ipZ];
	}
	else
	{
		helike.HeIonFrac[ipZ] = 0.;
	}

	/* check on the sum of the populations */
	if( helike.HeIonFrac[ipZ] < 0. && lgNegPop )
	{
		int i;
		helike.HeIonFrac[ipZ] = 0.;
		fprintf( ioQQQ, 
			" helevel finds negative He-like ion fraction for ipZ=%ld using routine HeLevel, IonFrac= %10.3e simple=%10.3e TE=%10.3e ZONE=%4ld\n", 
		  ipZ, helike.HeIonFrac[ipZ], 
		  helike.HeIonSimple[ipZ], 
		  phycon.te, nzone );
		fprintf( ioQQQ, " level pop are:\n" );
		for( i=0; i < (int)nHeLevels[ipZ]; i++ )
		{
			fprintf( ioQQQ," %8.1e;", pops[i] );
			if( (i!=0) && !(i%10) ) fprintf( ioQQQ,"\n" );
		}
		fprintf( ioQQQ, "\n" );
		negcon();
		ShowMe();
		puts( "[Stop in helike]" );
		cdEXIT(1);
	}

	/* get level populations, two cases, 
	 * first, may be zero since all cases drop down to here, 
	 * this branch of if is for trivial abundance, so zero out species */
	else if( helike.HeIonFrac[ipZ] >= 0. && helike.HeIonFrac[ipZ] < TOOSMALL )
	{
		helike.HeIonFrac[ipZ] = 0.;
		/* decrement highest stage of ionization, must not
		 * go up to He-like being ionized again */
		IonRange.IonHigh[ipZ] = ipZ-1;

		/* now zero this out */
		for( ipLo=0; ipLo < nHeLevels[ipZ]; ipLo++ )
		{
			for( ipHi=ipLo + 1; ipHi < nHeLevels[ipZ]; ipHi++ )
			{
				/* population of lower level rel to ion */
				HeLines[ipZ][ipHi][ipLo].PopLo = 0.;

				/* population of upper level rel to ion */
				HeLines[ipZ][ipHi][ipLo].PopHi =  0.;

				/* population of lower level rel to ion, corrected for stim em */
				HeLines[ipZ][ipHi][ipLo].PopOpc =  0.;

				/* local ots destruction, cm^-3 s^-1 */
				HeLines[ipZ][ipHi][ipLo].ots =  0.;
			}
		}
		if( trace.lgTrace && trace.lgHeBug )
		{
			fprintf(ioQQQ,"  HeLike for Z=%li finds small population set to zerozero.\n",ipZ);
		}
	}
	/* this is the main branch that applies when we have non-trivial abundance */
	else
	{
		/* level populations 
		 * this brance we have significant population as sum of levels per ion,
		 * store inverse, ion per atom, here.  For non-H species this is ratio
		 * of highest to next highest stage of ionization */

		/* now find emission in each line */
		for( ipHi=1; ipHi<nHeLevels[ipZ]; ++ipHi )
		{
			for( ipLo=0; ipLo<ipHi; ++ipLo )
			{
				HeLines[ipZ][ipHi][ipLo].PopLo = (float)pops[ipLo];
				HeLines[ipZ][ipHi][ipLo].PopHi = (float)pops[ipHi];
				HeLines[ipZ][ipHi][ipLo].PopOpc = (float)(pops[ipLo] - pops[ipHi]*
					HeLines[ipZ][ipHi][ipLo].gLo/HeLines[ipZ][ipHi][ipLo].gHi);
				/* some lines had strong masers, quench for now */
				fixit();
				HeLines[ipZ][ipHi][ipLo].PopOpc = MAX2(SMALLFLOAT,HeLines[ipZ][ipHi][ipLo].PopOpc );
			}
		}

		fixit();/* for now just do ground state, like bidiag will do */
		helike.HeIonFrac[ipZ] = 1.f/(float)pops[0];

	}

	{
		/* following should be set true to print information */
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG /*&& (iteration==2)*/ )
		{
			if( ipZ==1 )
				fprintf(ioQQQ,"\n");
			fprintf(ioQQQ,"%li\t%.4e", ipZ,
				helike.HeIonFrac[ipZ] );
			if( xIonFracs[ipZ+1][ipZ]>SMALLFLOAT )
			{
				float ratio = xIonFracs[ipZ+1][ipZ]/ xIonFracs[ipZ][ipZ];

			fprintf(ioQQQ,"\t%.4e\t%.4e", ratio ,
				(ratio-helike.HeIonFrac[ipZ])/ratio );
			}
			fprintf(ioQQQ,"\n");
		}

	}
	{
		/* following should be set true to print information */
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG /*&& (iteration==2)*/ )
		{
			if( xIonFracs[ipZ][ipZ] > SMALLFLOAT )
			{
				float ratio = xIonFracs[ipZ+1][ipZ]/xIonFracs[ipZ][ipZ] ;
				if( ratio > SMALLFLOAT )
				{
					fprintf(ioQQQ,"%li\t%.2e\t%.2e\t%.2e\n", 
						ipZ, helike.HeIonFrac[ipZ] ,
					(xIonFracs[ipZ+1][ipZ]/xIonFracs[ipZ][ipZ]) , helike.HeIonFrac[ipZ]/ratio );
					/*fprintf(ioQQQ,"helike %li\t%.2e\t%.2e\n", 
						ipZ, helike.hegamnc[ipZ][0] , create );*/
				}
			}
		}
	}

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "     He-like Z=%2ld H2OVH1=",ipZ);
		fprintf( ioQQQ,"%10.3e", helike.HeIonFrac[ipZ]);
		fprintf( ioQQQ, " simple=");
		fprintf( ioQQQ,"%10.3e", helike.HeIonSimple[ipZ]);
		fprintf( ioQQQ," dest, creat rates %.2e %.2e", destroy , create);
		fprintf( ioQQQ, "\n"); 
	}

#	ifdef DEBUG_FUN
	fputs( " <->HeLevel()\n", debug_fp );
#	endif
}

/* HeCSInterp interpolate on He1 collision strengths */
static float HeCSInterp( long ipZ , long ipHi , long ipLo )
{
	long ipArray;
	float cs,
		flow;

#	ifdef DEBUG_FUN
	fputs( "<+>HeLevel()\n", debug_fp );
#	endif

	assert( ipZ > 0 );
	assert( ipZ < LIMELM );

	if( phycon.alogte <= CSTemp[0] )
	{
		cs = helike.HeCS[ipZ][ipHi][ipLo][0];
	}
	else if( phycon.alogte >= CSTemp[helike.nCS-1] )
	{
		cs = helike.HeCS[ipZ][ipHi][ipLo][helike.nCS-1];
	}
	else
	{
		/* the array element within the cs vs temp array */
		ipArray = (long)((phycon.alogte - CSTemp[0])/(CSTemp[1]-CSTemp[0]));
		assert( ipArray < helike.nCS );
		assert( ipArray >= 0 );

		flow = (float)( (phycon.alogte - CSTemp[ipArray])/
			(CSTemp[ipArray+1]-CSTemp[ipArray])) ;
		assert( (flow >= 0.) && (flow <= 1.) );

		cs = helike.HeCS[ipZ][ipHi][ipLo][ipArray] * flow +
			helike.HeCS[ipZ][ipHi][ipLo][ipArray+1] * (1.f-flow);
	}

	return(cs);

#	ifdef DEBUG_FUN
	fputs( " <->HeCSInterp()\n", debug_fp );
#	endif
}

#undef  AA   
#undef  y 
#undef  SS 
#undef  ipivector 
