/*HydroCollid evaluate collision rate for model hydrogen atom */
#include <float.h>
#include "cddefines.h"
#include "physconst.h"
#include "taulines.h"
#include "hydrogenic.h"
#include "casbhs.h"
#include "trace.h"
#include "collionrate.h"
#include "phycon.h"
#include "rfield.h"
#include "heavy.h"
#include "hcolrt.h"
#include "secondaries.h"
#include "hdeltat.h"
#include "hlteok.h"
#include "h.h"
#include "chargtran.h"
#include "converge.h"
#include "cfit.h"
#include "smeets.h"
#include "atomcwgt.h"

static float HCSAR_interp( int ipLo , int ipHi );

/* >>refer	H1	cs	Anderson, H., Ballance, C.P., Badnell, N.R., & Summers, H.P., 
 * >>refercon	2000, J Phys B, 33, 1255 */
#define NHCSTE 8
#define NHCS 6
static float HCSTE[NHCSTE] = {5802.,11604.,34812.,58020.,116040.,174060.,232080.,290100.};

static float HCS[NHCSTE*NHCS*(NHCS-1)] =
{0., 0., 0., 0., 0., 0., 0., 0., /* 1s - 1s */
2.600E-01f, 2.960E-01f, 3.250E-01f, 3.370E-01f, 3.560E-01f, 3.680E-01f, 3.750E-01f, 3.800E-01f,/* 1s-2s*/
4.270E-01f, 5.360E-01f, 8.570E-01f, 1.150E+00f, 1.750E+00f, 2.130E+00f, 2.350E+00f, 2.460E+00f,/* 1s-2p*/
2.372E-01f, 2.605E-01f, 3.413E-01f, 4.104E-01f, 5.063E-01f, 5.351E-01f, 5.335E-01f, 5.236E-01f,/* 1s-3*/
9.990E-02f, 1.144E-01f, 1.564E-01f, 1.857E-01f, 2.263E-01f, 2.422E-01f, 2.445E-01f, 2.428E-01f,/* 1s-4*/
7.325E-02f, 8.263E-02f, 9.641E-02f, 1.064E-01f, 1.202E-01f, 1.233E-01f, 1.219E-01f, 1.191E-01f,/* 1s-5*/
0., 0., 0., 0., 0., 0., 0., 0., /* 2s - 1s */
0., 0., 0., 0., 0., 0., 0., 0., /* 2s - 2s */
0., 0., 0., 0., 0., 0., 0., 0., /* 2s - 2p */
5.860E+00f, 7.640E+00f, 1.419E+01f, 2.015E+01f, 3.156E+01f, 3.969E+01f, 4.598E+01f, 5.114E+01f,/* 2s - 3 */
2.276E+00f, 2.747E+00f, 4.365E+00f, 5.609E+00f, 7.803E+00f, 9.267E+00f, 1.038E+01f, 1.126E+01f,/* 2s - 4 */
1.610E+00f, 1.897E+00f, 2.421E+00f, 2.799E+00f, 3.512E+00f, 4.023E+00f, 4.450E+00f, 4.790E+00f,/* 2s - 5 */
0., 0., 0., 0., 0., 0., 0., 0., /* 2p - 1s */
0., 0., 0., 0., 0., 0., 0., 0., /* 2p - 2s */
0., 0., 0., 0., 0., 0., 0., 0., /* 2p - 2p */
2.228E+01f, 2.818E+01f, 5.026E+01f, 7.193E+01f, 1.178E+02f, 1.542E+02f, 1.828E+02f, 2.078E+02f,/* 2p - 3 */
8.772E+00f, 1.042E+01f, 1.600E+01f, 2.058E+01f, 2.898E+01f, 3.482E+01f, 3.923E+01f, 4.280E+01f,/* 2p - 4 */
6.161E+00f, 7.209E+00f, 9.091E+00f, 1.049E+01f, 1.308E+01f, 1.488E+01f, 1.626E+01f, 1.736E+01f,/* 2p - 5 */
0., 0., 0., 0., 0., 0., 0., 0., /* 3 - 1s */
0., 0., 0., 0., 0., 0., 0., 0., /* 3 - 2s */
0., 0., 0., 0., 0., 0., 0., 0., /* 3 - 2p */
0., 0., 0., 0., 0., 0., 0., 0., /* 3 - 3 */
8.648E+01f, 1.366E+02f, 3.252E+02f, 4.834E+02f, 7.715E+02f, 9.692E+02f, 1.123E+03f, 1.247E+03f,/* 3 - 4 */
6.229E+01f, 8.263E+01f, 1.255E+02f, 1.527E+02f, 1.951E+02f, 2.211E+02f, 2.400E+02f, 2.544E+02f,/* 3 - 5 */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 1s */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 2s */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 2p */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 3 */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 4 */
4.625E+02f, 8.070E+02f, 1.707E+03f, 2.269E+03f, 3.122E+03f, 3.675E+03f, 4.083E+03f, 4.414E+03f /* 4 - 5 */
};

void HydroCollid(long int ipZ)
{
	long int i, 
	  ipHi, 
	  ipLo;
	double CStemp, 
	  factor ,
	  HCol2s2pp, 
	  ConvLTEPOP;
	static long nzUsed = -1;
	/* too few initializers for aggregate, but it defaults to all zeros */
	static float told[LIMELM] = {0.};

#	ifdef DEBUG_FUN
	fputs( "<+>HydroCollid()\n", debug_fp );
#	endif

	/* check that we were called with valid charge */
	assert( ipZ >= 0);
	assert( ipZ < LIMELM );

	/* evaluate collision rate for model hydrogen atom */
	/* ipZ is atomic number minus 1, so H is 0*/

	{
		/*@-redef@*/
		enum {AGN=FALSE};
		/*@+redef@*/
		if( AGN  )
		{
			float tear[3]={10000.f,15000.f,20000.f};
			for( i=0;i<3;++i)
			{
				phycon.te = tear[i];
				for( ipHi=1;ipHi<4;++ipHi )
				{
					for( ipLo=0; ipLo<ipHi; ++ipLo )
					{
						if( !(ipLo==1 && ipHi==2 ) )
						{
							fprintf(ioQQQ,"%.3f\t", HCSAR_interp(ipLo,ipHi) ) ;
						}
					}
				}
				fprintf(ioQQQ," \n");
			}
			exit(0);
		}
	}

	/***********************************************************************
	 *                                                                     *
	 * get secondary excitation by suprathermal electrons                  *
	 *                                                                     *
	 ***********************************************************************/

	if( ipZ == 0 )
	{
		/* following are for hydrogen ATOMS, ions are in next else */
		/* secondary excitation of Lyman lines */
		Secondaries.Hx12[0][ipH2p] = (float)(Secondaries.x12tot*2./3./1.261);
		Secondaries.Hx12[0][ipH2s] = (float)(Secondaries.x12tot*1./3./1.261);
		Secondaries.Hx12[0][3] = (float)(Secondaries.x12tot*0.165/1.261);
		Secondaries.Hx12[0][4] = (float)(Secondaries.x12tot*0.059/1.261);

		/* rest are just a wild guess */
		for( i=5; i <= nHLevels[ipZ]; i++ )
		{
			Secondaries.Hx12[0][i] = (float)(Secondaries.Hx12[0][i-1]/2.);
		}
	}
	else if( ipZ==1 )
	{
		/* following for ions, guess from Janev et al. 
		 * only evaluated for He, and there will be a problem if
		 * hydrogenic helium not present, but hydrogenic species
		 * of heavier elements are present.  I can see how this
		 * would ever happen */
		Secondaries.Hx12[1][ipH2p] = (float)(Secondaries.x12tot/3.*2./3.);
		Secondaries.Hx12[1][ipH2s] = (float)(Secondaries.x12tot/3./3.);
		/* this is just a wild guess */
		for( i=3; i <= nHLevels[ipZ]; i++ )
		{
			Secondaries.Hx12[1][i] = (float)(Secondaries.Hx12[1][i-1]/2.);
		}
	}

	/* skip most of this sub if temperature has not changed by much,
	 * the check on conv.nTotalIoniz is to make sure that we redo this
	 * on the very first call in a grid calc - it is 0 on the first call */
	/* loc1 bug 99 apr 08, tried commenting out first block so always reeval, no joy */
	if( fabs(phycon.te-told[ipZ])/phycon.te < 0.00002 && 
		HydroLines[ipZ][ipH2p][ipH1s].ColUL > 0. && conv.nTotalIoniz )
	{
		if( trace.lgTrace )
		{
			fprintf( ioQQQ, 
				"       HydroCollid called ipZ %li - no reval Boltz fac, LTE dens\n",
				ipZ );
		}
		/* (but still eval col below, search for 
		 * following code executed even when temperature did not change" ) */
	}
	else
	{
		if( trace.lgTrace )
		{
			fprintf( ioQQQ, 
				"       HydroCollid called ipZ %li - will reeval Boltz fac, LTE dens\n",
				ipZ );
		}
		told[ipZ] = phycon.te;

		/***********************************************************************
		 *                                                                     *
		 * Boltzmann factors for all levels, ionization and                    *
		 * exciation between levels                                            *
		 *                                                                     *
		 ***********************************************************************/

		/* define boltz facs between adjacent levels, will go from ipHi to one below */
		for( ipHi=ipH2s; ipHi <= nHLevels[ipZ]; ipHi++ )
		{
			/* HdeltaTe is array of line temps, upper to lower
			 * their boltzmann factors for this temp,
			 * the array addressing for hlbolt is correct, and backwards from
			 * other similar variables - [up][lo] */
			hydro.hlbolt[ipZ][ipHi-1][ipHi] = 
				exp(-(double)HydroLines[ipZ][ipHi][ipHi-1].EnergyK/(double)phycon.te);
		}

		/* the highest level, up to the continuum */
		hydro.hcbolt[ipZ][nHLevels[ipZ]] =
			exp(-(double)HdeltaT.HCionTe[ipZ][nHLevels[ipZ]]/(double)phycon.te);

		/* define remaining line Boltzmann factors in terms of existing factors between levels */
		for( ipLo=ipH1s; ipLo <= (nHLevels[ipZ] - 2); ipLo++ )
		{
			for( ipHi=ipLo + 2; ipHi <= nHLevels[ipZ]; ipHi++ )
			{
				hydro.hlbolt[ipZ][ipLo][ipHi] = 
					hydro.hlbolt[ipZ][ipLo][ipHi-1]*
				  hydro.hlbolt[ipZ][ipHi-1][ipHi];
			}
		}

		for( i=ipH1s; i <= (nHLevels[ipZ] - 1); i++ )
		{
			hydro.hcbolt[ipZ][i] = 
				hydro.hlbolt[ipZ][i][nHLevels[ipZ]]*
			  hydro.hcbolt[ipZ][nHLevels[ipZ]];
			/* this is to prevent close-to-overflow problems in model atom */
			if( hydro.hcbolt[ipZ][i] < SMALLDOUBLE )
			{
				hydro.hcbolt[ipZ][i] = 0.;
			}
		}

		/***********************************************************************
		 *                                                                     *
		 * LTE abundances for all levels, ionization and                       *
		 * exciation between levels                                            *
		 *                                                                     *
		 ***********************************************************************/

		/* following factor is actually 4.1412957e-16 (old=4.14158E-16), 
		 * but e- stat weight is in */
		/* ConvLTEPOP = 2.0708e-16/phycon.te32;*/
		/* >>chng 99 jun 02, use codata and infinite mass nuc */
		factor = HION_LTE_POP*AtomcWgt.AtomicWeight[ipZ]/
			(AtomcWgt.AtomicWeight[ipZ]+ELECTRON_MASS/ATOMIC_MASS_UNIT) ;

		ConvLTEPOP = pow(factor,1.5)/2./phycon.te32;

		for( i=ipH1s; i < nHLevels[ipZ]; i++ )
		{
			/* >>>chng 99 mar 16, had gone to gt 0 previous week, but problems
			 * with very very small hcbolt, change back to 1e-100 */
			if( hydro.hcbolt[ipZ][i] > 1e-100 )
			{
				hcolrt.hlte[ipZ][i] = 
					HydroLines[ipZ][nHLevels[ipZ]][i].gLo / hydro.hcbolt[ipZ][i]* ConvLTEPOP;
			}
			else
			{
				hcolrt.hlte[ipZ][i] = 0.;
			}
		}
		if( hydro.hcbolt[ipZ][nHLevels[ipZ]]> 1e-100 )
		{
			hcolrt.hlte[ipZ][nHLevels[ipZ]] = 
				HydroLines[ipZ][nHLevels[ipZ]][ipH1s].gHi / hydro.hcbolt[ipZ][nHLevels[ipZ]]* ConvLTEPOP;
		}
		else
		{
			hcolrt.hlte[ipZ][nHLevels[ipZ]] = 0.;
		}

		/* now check for any zeros - if present then matrix cannot be used */
		HlteOk.lgHlteOk[ipZ] = TRUE;
		for( i=ipH1s; i <= nHLevels[ipZ]; i++ )
		{
			if( hcolrt.hlte[ipZ][i] <= 0. )
			{
				HlteOk.lgHlteOk[ipZ] = FALSE;
				break;
			}
		}

		/***********************************************************************
		 *                                                                     *
		 * collisional ionization, and three body recombination                *
		 * Vriens and Smeets collisional ionization data set                   *
		 * use for hydrogen itselt                                             *
		 *                                                                     *
		 ***********************************************************************/
		if( ipZ == 0 )
		{
			/* atomic hydrogen
			 * this is total to 2s and 2p, will be split below
			 * HColOn is set to 0 with hydrogen collisions off command, usually 1 */
			hydro.hcolnc[ipZ][ipH2p] = (float)(smeets(2)*hydro.HColIonOn);
			/* now do rest of levels, up to second highest */
			for( i=3; i < nHLevels[ipZ]; i++ )
			{
				/* HColOn is option to turn off collisions, "hydrogen collis off" comnd */
				hydro.hcolnc[ipZ][i] = (float)(smeets(i)*hydro.HColIonOn);
			}
			/* always leave highest level coupled to continuum - HColIonOn not there*/
			hydro.hcolnc[ipZ][nHLevels[ipZ]] = (float)(smeets(nHLevels[ipZ]));
		}
		else
		{
			/* hydrogenic ions */
			/* this is total to 2s and 2p */
			hydro.hcolnc[ipZ][ipH2p] = (float)(HydColIon(2,ipZ)*hydro.HColIonOn);
			for( i=3; i < nHLevels[ipZ]; i++ )
			{
				/* HColOn is option to turn off collisions, "hydrogen collis off" comnd */
				hydro.hcolnc[ipZ][i] = (float)(HydColIon(i,ipZ)*hydro.HColIonOn);
			}
			/* always leave highest level coupled to continuum - HColIonOn not there*/
			hydro.hcolnc[ipZ][nHLevels[ipZ]] = (float)(HydColIon(nHLevels[ipZ],ipZ));
		}

		/* total 2s and 2p was set in previous branch, now split into 2s and 2p */
		hydro.hcolnc[ipZ][ipH2p] *= 0.75;
		hydro.hcolnc[ipZ][ipH2s] = (float)(hydro.hcolnc[ipZ][ipH2p]/3.);

		/* collisional ionization from ground */
		cfit(ipZ+1,1,phycon.te,&hydro.hcolnc[ipZ][ipH1s]);

		/* option to kill collisional ionization with Hydro collis off */
		hydro.hcolnc[ipZ][ipH1s] *= hydro.HColIonOn;
		hydro.hcolnc[ipZ][ipH2s] *= hydro.HColIonOn;
		hydro.hcolnc[ipZ][ipH2p] *= hydro.HColIonOn;

		/* save into main array */
		CollIonRate.CollidRate[0][ipZ][ipZ] = 
			hydro.hcolnc[ipZ][ipH1s]*(float)phycon.eden + Secondaries.csupra;

		/* cooling due to collisional ionization, which only includes thermal */
		CollIonRate.CollidRate[1][ipZ][ipZ] = (float)(hydro.hcolnc[ipZ][ipH1s]*(float)phycon.eden*
			rfield.anu[Heavy.ipHeavy[ipZ][ipZ]-1]* EN1RYD);

		/* hummer and storey option for case b kills coll ioniz from 1 and 2 */
		if( casbhs.lgCasBhs )
		{
			hydro.hcolnc[ipZ][ipH1s] = 0.;
			hydro.hcolnc[ipZ][ipH2p] = 0.;
			hydro.hcolnc[ipZ][ipH2s] = 0.;
		}

		/***********************************************************************
		 * 
		 * collisional strengths for all hydrogen lines
		 * 
		 ************************************************************************/

		/* evaluate collisional deexcitation rates,
		 * Hydcs123 evaluates downward collision strength, which is stored in
		 * hydrogen lines vector
		 * Hydcs123, upon call 1=1s, 2=2s, 3=2p, 4=3 */
		for( ipLo=ipH1s; ipLo <= (nHLevels[ipZ] - 1); ipLo++ )
		{
			for( ipHi=ipLo + 1; ipHi <= nHLevels[ipZ]; ipHi++ )
			{
				if( ipHi == 1 )
				{
					/* 2s - 1s */
					if( ipZ==0 )
					{
						CStemp = HCSAR_interp(ipLo,ipHi);
					}
					else
					{
						CStemp = Hydcs123(1,2,ipZ,'e');
					}
				}
				else if( ipHi == 2 )
				{
					if( ipLo == 0 )
					{
						/* 2p - 1s */
						if( ipZ==0 )
						{
							CStemp = HCSAR_interp(ipLo,ipHi);
						}
						else
						{
							CStemp = Hydcs123(1,3,ipZ,'e');
						}
					}
					/* this is 2s - 2p */
					else if( ipLo == 1 )
					{
						/* option to kill 2s2p collisions */
						if( hydro.lgC2psOn )
						{
							/* 2s - 2p for electrons ONLY */
							CStemp = Hydcs123(2,3,ipZ,'e');
							/* 2s - 2p for protons ONLY */
							HCol2s2pp = Hydcs123(2,3,ipZ,'p');
							/* trick, since will later be multiplied by electron,
							 * not proton density */
							/*CStemp += HCol2s2pp*h.hii/phycon.eden;*/
							/* NB - all collision rates will be multiplied by this number */
							CStemp += HCol2s2pp*h.hii/phycon.EdenHCorr;
						}
						else
						{
							CStemp = 0.;
						}
					}
					else
					{
						/* this is not possible */
						CStemp = -DBL_MAX;
					}
				}

				else if( ipHi == 3 )
				{
					/* 2p - 1s */
					if( ipZ==0 )
					{
						CStemp = HCSAR_interp(ipLo,ipHi);
					}
					else
					{
						if( ipLo == 0 )
						{
							/* 3 - 1 */
							CStemp = Hydcs123(1,4,ipZ,'e');
						}
						else if( ipLo == 1 )
						{
							/* 3 - 2s */
							CStemp = Hydcs123(2,4,ipZ,'e');
						}
						else if( ipLo == 2 )
						{
							/* 3 - 2p */
							CStemp = Hydcs123(3,4,ipZ,'e');
						}
						else
						{
							/* this is not possible */
							CStemp = -DBL_MAX;
						}
					}
				}
				else if( ipZ==0 && ipHi <= 5 )
				{
					CStemp = HCSAR_interp(ipLo,ipHi);
				}
				else
				{
					/* highly excited levels */
					CStemp = HydColDwn(ipHi,ipLo,ipZ+1);
				}
				HydroLines[ipZ][ipHi][ipLo].cs = (float)CStemp;
			}
		}

		/* save 2s-2p collisions since we always want to include, even when other
		 * collisions are turned off, since 2s-2p act as one level.  there is a
		 * separate no 2s2p command, which is implemented below */
		CStemp = HydroLines[ipZ][ipH2p][ipH2s].cs;

		/* kill ALL collisions if this set with hydrogen collisional off set */
		if( hydro.HColExcOn == 0. )
		{
			for( ipLo=ipH1s; ipLo <= (nHLevels[ipZ]-1); ipLo++ )
			{
				for( ipHi=ipLo + 1; ipHi <= nHLevels[ipZ]; ipHi++ )
				{
					HydroLines[ipZ][ipHi][ipLo].cs = 0.;
				}
			}
		}

		/* case b hummer&storey does not have any collisions from n=1 or 2 
		 * just kill those collisions here */
		if( casbhs.lgCasBhs )
		{
			for( ipLo=ipH1s; ipLo <= ipH2p; ipLo++ )
			{
				for( ipHi=3; ipHi <= nHLevels[ipZ]; ipHi++ )
				{
					HydroLines[ipZ][ipHi][ipLo].cs = 0.;
				}
			}
		}

		/* reset the 2s-2p collsions unless turned off with hydrogen collisions 2s2p
		 * command, since we usually want to include these even when line excit
		 * is turned off */
		if( hydro.lgC2psOn )
		{
			HydroLines[ipZ][ipH2p][ipH2s].cs = (float)CStemp;
		}

		/***********************************************************************
		 *                                                                     *
		 * collisional deexcitation for all lines                              *
		 *                                                                     *
		 ***********************************************************************/

		for( ipLo=ipH1s; ipLo <= (nHLevels[ipZ] - 1); ipLo++ )
		{
			for( ipHi=ipLo + 1; ipHi <= nHLevels[ipZ]; ipHi++ )
			{
				/* collisional deexcitation rate */
				HydroLines[ipZ][ipHi][ipLo].ColUL = 
					(float)(HydroLines[ipZ][ipHi][ipLo].cs/
				  phycon.sqrte*8.629e-6/HydroLines[ipZ][ipHi][ipLo].gHi );
				/*This is downward collision rate */
			}
		}

		if( (trace.lgTrace && trace.lgHBugFull) && (ipZ == trace.ipZTrace) )
		{
			fprintf( ioQQQ, "       HydroCollid rate n->n-1:" );
			for( i=ipH1s; i <= (nHLevels[ipZ] - 1); i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i+1][i].ColUL ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid col ion cof:" );
			for( i=ipH1s; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e",  hydro.hcolnc[ipZ][i] ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid rate dn2 1s:" );
			for( i=ipH2s; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i][ipH1s].ColUL ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid rate dn2 2s:" );
			for( i=ipH2p; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i][ipH2s].ColUL ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid rate dn2 2p:" );
			for( i=3; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i][ipH2p].ColUL ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid rate dwn2 3:" );
			for( i=4; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i][3].ColUL ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid rate dwn2 4:" );
			for( i=5; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i][4].ColUL ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid rate dwn2 5:" );
			for( i=6; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i][5].ColUL) );
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid rate dwn2 6:" );
			for( i=7; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i][6].ColUL ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid rate dwn2 7:" );
			for( i=8; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", HydroLines[ipZ][i][7].ColUL) );
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid boltz fac c:" );
			for( i=ipH1s; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", hydro.hcbolt[ipZ][i] ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, "       HydroCollid HLTE(i,Z):  " );
			for( i=ipH1s; i <= nHLevels[ipZ]; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%10.3e", hcolrt.hlte[ipZ][i] ));
			}
			fprintf( ioQQQ, "\n" );
		}
	}

	/* following code executed even when temperature did not change*/

	if( ipZ == 0 )
	{
		/* hydrogen charge transfer charge exchange
		 * following only for hydrogen, only other array element is
		 * 2, all set zero in zero,
		 * ChargTran.HCTOn set to 0 with "no charge transfer" command */
		if( nzone <= 1 || 
			ChargTran.CTHrec[0][1]*ChargTran.CTHion[0][1] == 0. )
		{
			if( !conv.nTotalIoniz )
			{
				/* this is zone 0, the first time hydro is called
				 * ct rates have not been evaluated yet by the heavies */
				ChargTran.CTHrec[0][1] = 0.;
				ChargTran.CTHion[0][1] = 0.;
				ChargTran.CTHrec[0][0] = 0.;
				ChargTran.CTHion[0][0] = 0.;
			}
			else
			{
				ChargTran.CTHrec[0][1] = ChargTran.CTHrec[0][0]*ChargTran.HCTOn;
				ChargTran.CTHion[0][1] = ChargTran.CTHion[0][0]*ChargTran.HCTOn;
			}
		}
		else if( nzone != nzUsed )
		{
			ChargTran.CTHrec[0][1] = (float)(ChargTran.CTHrec[0][1]*0.75 + 0.25*ChargTran.CTHrec[0][0]*
			  ChargTran.HCTOn);
			ChargTran.CTHion[0][1] = (float)(ChargTran.CTHion[0][1]*0.75 + 0.25*ChargTran.CTHion[0][0]*
			  ChargTran.HCTOn);
		}
		nzUsed = nzone;
	}


#	ifdef DEBUG_FUN
	fputs( " <->HydroCollid()\n", debug_fp );
#	endif
	return;
}


#define HCSAR(ilo , ihi , nte ) (*(HCS+(nte)+(ilo)*48+(ihi)*8))

static float HCSAR_interp( int ipLo , int ipHi )
{

	static int ip=1;
	float cs;

#	ifdef DEBUG_FUN
	fputs( "<+>HCSAR_interp()\n", debug_fp );
#	endif

	if( ipLo==1 && ipHi==2 )
	{
		fprintf(ioQQQ,"HCSAR_interp was called for the 2s-2p transition, which it cannot do\n");
		exit(1);
	}
	if( phycon.te <= HCSTE[0] )
	{
		cs = HCSAR( ipLo , ipHi , 0 );
	}
	else if( phycon.te >= HCSTE[NHCSTE-1] )
	{
		cs = HCSAR( ipLo , ipHi , NHCSTE-1 );
	}
	else
	{
		/* the ip index is most likely correct since it points to the last temperature */
		if( !(HCSTE[ip-1] < phycon.te ) && ( phycon.te <= HCSTE[ip]) )
		{
			/* we must find the temperature in the array */
			for( ip=1; ip<NHCSTE ; ++ip )
			{
				if( (HCSTE[ip-1] < phycon.te ) && ( phycon.te <= HCSTE[ip]) )
					break;
			}
		}
		/* we now have the index */
		cs = HCSAR( ipLo , ipHi , ip-1 ) + 
			(HCSAR( ipLo , ipHi , ip ) - HCSAR( ipLo , ipHi , ip-1 ) ) / (HCSTE[ip]-HCSTE[ip-1] ) *
			(phycon.te - HCSTE[ip-1] );
		if( cs <= 0.)
		{
			fprintf(ioQQQ," insane cs returned by HCSAR_interp, values are\n");
			fprintf(ioQQQ,"%.3f %.3f \n", HCSAR( ipLo , ipHi , ip-1 ),HCSAR( ipLo , ipHi , ip ) );
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->HCSAR_interp()\n", debug_fp );
#	endif
	return(cs) ;
}

#if 0

/*HCS[lo][up][te]*/
#define NHCSTE 8
#define NHCS 6
/* HCS[lo][up][NHCSTE]*/
static float 
/*HCS[NHCS-1][NHCS][NHCSTE] =*/
HCS[NHCSTE*NHCS*(NHCS-1)] =
{0., 0., 0., 0., 0., 0., 0., 0., /* 1s - 1s */
2.600E-01f, 2.960E-01f, 3.250E-01f, 3.370E-01f, 3.560E-01f, 3.680E-01f, 3.750E-01f, 3.800E-01f,/* 1s-2s*/
4.270E-01f, 5.360E-01f, 8.570E-01f, 1.150E+00f, 1.750E+00f, 2.130E+00f, 2.350E+00f, 2.460E+00f,/* 1s-2p*/
2.372E-01f, 2.605E-01f, 3.413E-01f, 4.104E-01f, 5.063E-01f, 5.351E-01f, 5.335E-01f, 5.236E-01f,/* 1s-3*/
9.990E-02f, 1.144E-01f, 1.564E-01f, 1.857E-01f, 2.263E-01f, 2.422E-01f, 2.445E-01f, 2.428E-01f,/* 1s-4*/
7.325E-02f, 8.263E-02f, 9.641E-02f, 1.064E-01f, 1.202E-01f, 1.233E-01f, 1.219E-01f, 1.191E-01f,/* 1s-5*/
0., 0., 0., 0., 0., 0., 0., 0., /* 2s - 1s */
0., 0., 0., 0., 0., 0., 0., 0., /* 2s - 2s */
0., 0., 0., 0., 0., 0., 0., 0., /* 2s - 2p */
5.860E+00f, 7.640E+00f, 1.419E+01f, 2.015E+01f, 3.156E+01f, 3.969E+01f, 4.598E+01f, 5.114E+01f,/* 2s - 3 */
2.276E+00f, 2.747E+00f, 4.365E+00f, 5.609E+00f, 7.803E+00f, 9.267E+00f, 1.038E+01f, 1.126E+01f,/* 2s - 4 */
1.610E+00f, 1.897E+00f, 2.421E+00f, 2.799E+00f, 3.512E+00f, 4.023E+00f, 4.450E+00f, 4.790E+00f,/* 2s - 5 */
0., 0., 0., 0., 0., 0., 0., 0., /* 2p - 1s */
0., 0., 0., 0., 0., 0., 0., 0., /* 2p - 2s */
0., 0., 0., 0., 0., 0., 0., 0., /* 2p - 2p */
2.228E+01f, 2.818E+01f, 5.026E+01f, 7.193E+01f, 1.178E+02f, 1.542E+02f, 1.828E+02f, 2.078E+02f,/* 2p - 3 */
8.772E+00f, 1.042E+01f, 1.600E+01f, 2.058E+01f, 2.898E+01f, 3.482E+01f, 3.923E+01f, 4.280E+01f,/* 2p - 4 */
6.161E+00f, 7.209E+00f, 9.091E+00f, 1.049E+01f, 1.308E+01f, 1.488E+01f, 1.626E+01f, 1.736E+01f,/* 2p - 5 */
0., 0., 0., 0., 0., 0., 0., 0., /* 3 - 1s */
0., 0., 0., 0., 0., 0., 0., 0., /* 3 - 2s */
0., 0., 0., 0., 0., 0., 0., 0., /* 3 - 2p */
0., 0., 0., 0., 0., 0., 0., 0., /* 3 - 3 */
8.648E+01f, 1.366E+02f, 3.252E+02f, 4.834E+02f, 7.715E+02f, 9.692E+02f, 1.123E+03f, 1.247E+03f,/* 3 - 4 */
6.229E+01f, 8.263E+01f, 1.255E+02f, 1.527E+02f, 1.951E+02f, 2.211E+02f, 2.400E+02f, 2.544E+02f,/* 3 - 5 */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 1s */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 2s */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 2p */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 3 */
0., 0., 0., 0., 0., 0., 0., 0., /* 4 - 4 */
4.625E+02f, 8.070E+02f, 1.707E+03f, 2.269E+03f, 3.122E+03f, 3.675E+03f, 4.083E+03f, 4.414E+03f /* 4 - 5 */
};

/*HCS[lo][up][te]*/
#define NHCSTE 8
#define NHCS 6
/* HCS[lo][up][NHCSTE]*/
static float 
HCS[NHCS-1][NHCS][NHCSTE] =
{
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 1s - 1s */
{2.600E-01f, 2.960E-01f, 3.250E-01f, 3.370E-01f, 3.560E-01f, 3.680E-01f, 3.750E-01f, 3.800E-01f},/* 1s-2s*/
{4.270E-01f, 5.360E-01f, 8.570E-01f, 1.150E+00f, 1.750E+00f, 2.130E+00f, 2.350E+00f, 2.460E+00f},/* 1s-2p*/
{2.372E-01f, 2.605E-01f, 3.413E-01f, 4.104E-01f, 5.063E-01f, 5.351E-01f, 5.335E-01f, 5.236E-01f},/* 1s-3*/
{9.990E-02f, 1.144E-01f, 1.564E-01f, 1.857E-01f, 2.263E-01f, 2.422E-01f, 2.445E-01f, 2.428E-01f},/* 1s-4*/
{7.325E-02f, 8.263E-02f, 9.641E-02f, 1.064E-01f, 1.202E-01f, 1.233E-01f, 1.219E-01f, 1.191E-01f},/* 1s-5*/
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 2s - 1s */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 2s - 2s */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 2s - 2p */
{5.860E+00f, 7.640E+00f, 1.419E+01f, 2.015E+01f, 3.156E+01f, 3.969E+01f, 4.598E+01f, 5.114E+01f},/* 2s - 3 */
{2.276E+00f, 2.747E+00f, 4.365E+00f, 5.609E+00f, 7.803E+00f, 9.267E+00f, 1.038E+01f, 1.126E+01f},/* 2s - 4 */
{1.610E+00f, 1.897E+00f, 2.421E+00f, 2.799E+00f, 3.512E+00f, 4.023E+00f, 4.450E+00f, 4.790E+00f},/* 2s - 5 */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 2p - 1s */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 2p - 2s */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 2p - 2p */
{2.228E+01f, 2.818E+01f, 5.026E+01f, 7.193E+01f, 1.178E+02f, 1.542E+02f, 1.828E+02f, 2.078E+02f},/* 2p - 3 */
{8.772E+00f, 1.042E+01f, 1.600E+01f, 2.058E+01f, 2.898E+01f, 3.482E+01f, 3.923E+01f, 4.280E+01f},/* 2p - 4 */
{6.161E+00f, 7.209E+00f, 9.091E+00f, 1.049E+01f, 1.308E+01f, 1.488E+01f, 1.626E+01f, 1.736E+01f},/* 2p - 5 */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 3 - 1s */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 3 - 2s */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 3 - 2p */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 3 - 3 */
{8.648E+01f, 1.366E+02f, 3.252E+02f, 4.834E+02f, 7.715E+02f, 9.692E+02f, 1.123E+03f, 1.247E+03f},/* 3 - 4 */
{6.229E+01f, 8.263E+01f, 1.255E+02f, 1.527E+02f, 1.951E+02f, 2.211E+02f, 2.400E+02f, 2.544E+02f},/* 3 - 5 */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 4 - 1s */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 4 - 2s */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 4 - 2p */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 4 - 3 */
{0., 0., 0., 0., 0., 0., 0., 0.}, /* 4 - 4 */
{4.625E+02f, 8.070E+02f, 1.707E+03f, 2.269E+03f, 3.122E+03f, 3.675E+03f, 4.083E+03f, 4.414E+03f} /* 4 - 5 */
}
#endif
