/*Hydrogenic main routine to call HydroLevel and determine model hydrogen atom level balance */
#include "cddefines.h"
#include "taulines.h"
#include "hydrogenic.h"
#include "hphoto.h"
#include "trace.h"
#include "ionfracs.h"
#include "secondaries.h"
#include "h.h"
#include "phycon.h"
#include "struc.h"
#include "hmi.h"
#include "hionfraccom.h"
#include "hcolrec.h"
#include "colion.h"
#include "abundances.h"
#include "ionrange.h"
#include "hcolrt.h"
#include "hcaseb.h"
#include "hmole.h"

void Hydrogenic(void)
{
	long int i ,
		ipLo ,
		ipHi , 
		ipZ;
	double  coltot, 
	  gamtot, 
	  grddst ,
	  photex;
	float hmovh1 = 0.; 

#	ifdef DEBUG_FUN
	fputs( "<+>Hydrogenic()\n", debug_fp );
#	endif

	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		/* do not consider elements that have been turned off */
		if( abundances.lgElmtOn[ipZ] )
		{
			/* note that ipZ scale is totally on c not physical scale, so 0 is h */
			/* evaluate hydrogenic balance if ionization reaches this high */
			if( (IonRange.IonHigh[ipZ] == ipZ + 1)  )
			{
				/* evaluate collisional rates */
				HydroCollid(ipZ);

				/* evaluate photoionization rates  */
				HydroPhoto(ipZ);

				/* evaluate recombination rates */
				HydroRecom(ipZ);

				/* solve for the level populations */
				HydroLevel(ipZ);

				if( trace.lgTrace )
				{
					fprintf( ioQQQ, "     Hydrogenic Z=%2ld H2OVH1=",ipZ);
					fprintf( ioQQQ,PrintEfmt("%10.3e", HIonFraccom.HIonFrac[ipZ]));
					fprintf( ioQQQ, " simple=");
					fprintf( ioQQQ,PrintEfmt("%10.3e", HIonFraccom.HIonSimple[ipZ]));
					fprintf( ioQQQ, " H-ovH1:");
					fprintf( ioQQQ,PrintEfmt("%10.2e", hmovh1 ) );
					fprintf( ioQQQ, "\n"); 
				}

			}
			else
			{
				HIonFraccom.HIonFrac[ipZ] = 0.;
				HIonFraccom.HIonSimple[ipZ] = 0.;

				/* zero it out since no population*/
				for( ipHi=ipH2s; ipHi <= nHLevels[ipZ]; ipHi++ )
				{
					for( ipLo=ipH1s; ipLo < ipHi; ipLo++ )
					{
						/* population of lower level rel to ion */
						HydroLines[ipZ][ipHi][ipLo].PopLo = 0.;

						/* population of upper level rel to ion */
						HydroLines[ipZ][ipHi][ipLo].PopHi =  0.;

						/* population of lower level rel to ion, corrected for stim em */
						HydroLines[ipZ][ipHi][ipLo].PopOpc =  0.;
					}
				}
			}
		}
	}

	/* rest is for hydrogen only */

	/* do molecular balance 
	 * hmovh1 will be ratio of molecular to atomic hydrogen
	 * HIonFrac is fration of H that is ionized, ratio of ion to atom */
	hmole(&hmovh1,HIonFraccom.HIonFrac[0]);

	/* get ionization balance */
	h.hii = (float)(phycon.hden*HIonFraccom.HIonFrac[0]/
		(1. + HIonFraccom.HIonFrac[0] + hmovh1));

	/* save it into the master ionization balance array */
	xIonFracs[2][ipHYDROGEN] = h.hii;

	h.hi = (float)(phycon.hden/(1. + HIonFraccom.HIonFrac[0] + hmovh1));
	xIonFracs[1][ipHYDROGEN] = h.hi;

	/* is hydrogen atom in condition of collisional rec? if so then
	 * only very small dT's can be tolerated
	 * HRecEffec(ipZ) is sum of totcap(i)
	 * htotrc(ipZ) is sum of radiative recombination */
	if( hcaseb.HRecCol[ipHYDROGEN]/hcaseb.htotrc[ipHYDROGEN] > 100. )
	{
		HColRec.lgHColRec = TRUE;
	}
	else
	{
		HColRec.lgHColRec = FALSE;
	}

	/*fprintf(ioQQQ,"ratio %.1e\n", HPhoto.hgamnc[ipHELIUM][ipH2s]/HydroLines[ipHELIUM][ipH2s][ipH1s].Aul);*/

	/* this is test whether collisions are important first define ratio of exits
	 * from ground due to coll, rel to everthing else, then flag is large */
	if( hcolrt.hlte[0][ipH1s] > 1e-30 )
	{
		coltot = 
			HydroLines[ipHYDROGEN][ipH2p][ipH1s].ColUL*
			hcolrt.hlte[ipHYDROGEN][ipH2p]/hcolrt.hlte[ipHYDROGEN][ipH1s]*
		  phycon.eden*(HPhoto.hgamnc[ipHYDROGEN][ipH2p] + hydro.hcolnc[ipHYDROGEN][ipH2p] + 
		  HydroLines[ipHYDROGEN][3][ipH2p].ColUL*hcolrt.hlte[ipHYDROGEN][3]/hcolrt.hlte[ipHYDROGEN][ipH2p])/
		  (HydroLines[ipHYDROGEN][ipH2p][ipH1s].ColUL*phycon.eden + 
		  HydroLines[ipHYDROGEN][ipH2p][ipH1s].Aul*
		  (HydroLines[ipHYDROGEN][ipH2p][ipH1s].Pesc + HydroLines[ipHYDROGEN][ipH2p][ipH1s].Pdest) );

		/* caution that collisions are important (and so only small changes in
		 * temperature should happen) if more than 20% of the total */
		if( coltot > 0.2 )
		{
			colion.lgHColionImp = TRUE;
		}
	}
	else
	{
		colion.lgHColionImp = FALSE;
	}

	/* remember the ratio of pops of 2p to 1s for possible printout in prtComment
	 * and to obtain Lya excitation temps.  the pop of ground is not defined if
	 * NO ionization at all since these pops are relative to ion */
	/* >>chng 99 jun 03, added MAX2 to protect against totally neutral gas */
	if( hydro.hn[0][ipH2p]/MAX2(SMALLFLOAT,hydro.hn[0][ipH1s]) > 0.1 )
	{
		hydro.lgHiPop2 = TRUE;
		hydro.pop2mx = (float)MAX2(hydro.hn[ipHYDROGEN][ipH2p]/hydro.hn[ipHYDROGEN][ipH1s],
		  hydro.pop2mx);
	}
	/*fprintf(ioQQQ,"hdebuggg %li %.2e %.2e\n" ,
		nzone,
		hydro.hn[0][ipH2p]/MAX2(SMALLFLOAT,hydro.hn[0][ipH1s]) , 
		HPhoto.hgamnc[0][ipH2p] );*/

	photex = 0.;
	for( i=ipH2s; i <= nHLevels[ipHYDROGEN]; i++ )
	{
		photex += hydro.hn[ipHYDROGEN][i]*HPhoto.hgamnc[ipHYDROGEN][i];
	}

	gamtot = HPhoto.hgamnc[ipHYDROGEN][ipH1s] + Secondaries.csupra;

	coltot = hydro.hcolnc[ipHYDROGEN][ipH1s] + 
		HydroLines[ipHYDROGEN][ipH2p][ipH1s].ColUL*4.*
	  hydro.hlbolt[ipHYDROGEN][ipH1s][ipH2p];

	/* ground state destruction rate*/
	grddst = gamtot + coltot*phycon.eden + Secondaries.x12tot + 
		  photex/MAX2(1e-37,hydro.hn[ipHYDROGEN][ipH1s] ) ;

	if( grddst > 0. )
	{
		hydro.photn2 = 
			(float)(HPhoto.hgamnc[ipHYDROGEN][ipH1s]/grddst );

		/* fraction of ionizations of H from ground, due to thermal collisions */
		hydro.HCollIonFrac = (float)(hydro.hcolnc[ipHYDROGEN][ipH1s]*phycon.eden/grddst);

		/* this flag is used in ionize to decide whether we
		 * really need to converge the secondary ionization rates */
		Secondaries.sec2total = (float)(Secondaries.csupra / grddst) ;
	}
	else
	{
		hydro.HCollIonFrac = 0.;
		hydro.photn2 = 0.;
		Secondaries.sec2total = 0. ;
	}

	/* now do 21 cm, will define populations here */
	TauLines[ipH21cm].PopHi = HydroLines[ipHYDROGEN][ipH2p][ipH1s].PopLo*0.75f;	
	TauLines[ipH21cm].PopLo = HydroLines[ipHYDROGEN][ipH2p][ipH1s].PopLo*0.25f;
	/* assume spin temperature is gas kinetic temperature */
	TauLines[ipH21cm].PopOpc = HydroLines[ipHYDROGEN][ipH2p][ipH1s].PopLo * 
		0.25f * TauLines[ipH21cm].EnergyK / phycon.te;

	/* now do 3 cm, the HeII equivalent of H 21 cm will define populations here */
	TauLines[ipHe3cm].PopHi = HydroLines[ipHELIUM][ipH2p][ipH1s].PopLo*0.75f;	
	TauLines[ipHe3cm].PopLo = HydroLines[ipHELIUM][ipH2p][ipH1s].PopLo*0.25f;
	/* assume spin temperature is gas kinetic temperature */
	TauLines[ipHe3cm].PopOpc = HydroLines[ipHELIUM][ipH2p][ipH1s].PopLo * 
		0.25f * TauLines[ipHe3cm].EnergyK / phycon.te;


	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "     Hydrogenic retrn H,2,NE=");
		fprintf(ioQQQ,"%.2e ", h.hi);
		fprintf(ioQQQ,"%.2e ", h.hii);
		fprintf(ioQQQ,"%.2e ", phycon.eden);
		fprintf( ioQQQ, " REC, COL, GAMT= ");
		/* recomb rate coef, cm^3 s-1 */
		fprintf(ioQQQ,"%.2e ", hcaseb.htotrc[0] );
		fprintf(ioQQQ,"%.2e ", coltot);
		fprintf(ioQQQ,"%.2e ", gamtot);
		fprintf( ioQQQ, " CSUP=");
		PrintE82( ioQQQ, Secondaries.csupra);
		fprintf( ioQQQ, " Htwo=");
		fprintf(ioQQQ,"%.2e ", hmi.htwo);
		fprintf( ioQQQ, " Te=");
		fprintf(ioQQQ,"%.2e ", phycon.te );
		fprintf( ioQQQ, "\n");
	}

#	ifdef DEBUG_FUN
	fputs( " <->Hydrogenic()\n", debug_fp );
#	endif
	return;
}
