/*HydroPhoto photoionization, recombination, radiative rates for model hydrogen atom */
#include "cddefines.h"
#include "hydrogenic.h"
#include "rfield.h"
#include "opacity.h"
#include "trace.h"
#include "hphoto.h"
#include "photrate.h"
#include "hpheat.h"
#include "hgamsc.h"
#include "heat.h"
#include "hcolrt.h"
#include "hlteok.h"
#include "gammas.h"

void HydroPhoto(long int ipZ)
{
	long int n, 
	  limit;

#	ifdef DEBUG_FUN
	fputs( "<+>HydroPhoto()\n", debug_fp );
#	endif

	/* check that we were called with valid charge */
	assert( ipZ >= 0);
	assert( ipZ < LIMELM );

	/* do photoionization rates */
	/* induced recombination; FINDUC is integral of
	 * pho rate times EXP(-hn/kt) for induc rec
	 * CIND is this times hnu-hnu0 to get ind rec cooling
	 * PhotScaleOn is 1, set to 0 with 'no photoionization' command
	 * ipSecIon points to 7.353 Ryd, lowest energy where secondary ioniz
	 * of hydrogen is possible */

	HPhoto.hgamnc[ipZ][ipH1s] = GammaBn(hydro.ipHLevNIonCon[ipZ][ipH1s],
		rfield.nflux,
		opac.ipHOpac[ipZ][ipH1s],
	  hydro.HLevNIonRyd[ipZ][ipH1s],
	  &HPhoto.finduc[ipZ][ipH1s],
	  &HPhoto.cind[ipZ][ipH1s])*
	  PhotRate.PhotScaleOn;

	HPHeat.HPhotHeat[ipZ][ipH1s] = heat.HeatNet*PhotRate.PhotScaleOn;

	PhotRate.PhotoRate[0][0][ipZ][ipZ] = HPhoto.hgamnc[ipZ][ipH1s];
	PhotRate.PhotoRate[1][0][ipZ][ipZ] = heat.HeatLowEnr*PhotRate.PhotScaleOn;
	PhotRate.PhotoRate[2][0][ipZ][ipZ] = heat.HeatHiEnr*PhotRate.PhotScaleOn;

	/* option to print photoionization rates */
	if( trace.lgTrace && trace.lgHBug )
	{
		GammaPrt(hydro.ipHLevNIonCon[ipZ][ipH1s],
			rfield.nflux,
			opac.ipHOpac[ipZ][ipH1s],
		  ioQQQ,
		  HPhoto.hgamnc[ipZ][ipH1s],HPhoto.hgamnc[ipZ][ipH1s]*0.05);
	}

	/* hgamsc is direct photioniz rate due to 
	 * bound compton scattering of very hard x-rays+Compton scat */
	HPhoto.hgamnc[ipZ][ipH1s] += hgamsc.BoundCompton;

	/* all other continua, n<NHPLPHOT are actual fits to photo cross sections,
	 * NHPLPHOT is 10 in opacity.h */
	limit = MIN2((long)(NHPLPHOT-1),nHLevels[ipZ]);
	for( n=ipH2s; n <= limit; n++ )
	{
		if( n>4 && !opac.lgRedoStatic )
			break;
		if( HPhoto.lgHInducImp )
		{
			HPhoto.hgamnc[ipZ][n] = 
				GammaBn(
				hydro.ipHLevNIonCon[ipZ][n],
			  hydro.ipHLevNIonCon[ipZ][ipH1s]-1,
			  opac.ipHOpac[ipZ][n],
			  hydro.HLevNIonRyd[ipZ][n],
			  &HPhoto.finduc[ipZ][n],
			  &HPhoto.cind[ipZ][n])*
			  PhotRate.PhotScaleOn;
		}
		else
		{
			HPhoto.hgamnc[ipZ][n] = 
				(float)(GammaK(hydro.ipHLevNIonCon[ipZ][n],
				hydro.ipHLevNIonCon[ipZ][ipH1s]-1,
				opac.ipHOpac[ipZ][n],1.)*
				PhotRate.PhotScaleOn);
			HPhoto.finduc[ipZ][n] = 0.;
			HPhoto.cind[ipZ][n] = 0.;
		}
		HPHeat.HPhotHeat[ipZ][n] = heat.HeatNet*PhotRate.PhotScaleOn;
		{
			/*@-redef@*/
			enum {DEBUG=FALSE};
			/*@+redef@*/
			if( DEBUG )
			{
				if( ipZ==0 && n==ipH2s && nzone > 267 && iteration==2 )
				{
					GammaPrt(hydro.ipHLevNIonCon[ipZ][n],
						hydro.ipHLevNIonCon[ipZ][ipH1s]-1,
						opac.ipHOpac[ipZ][n],
					  ioQQQ,
					  HPhoto.hgamnc[ipZ][n],HPhoto.hgamnc[ipZ][n]*0.05);
				}
			}
		}
	}
	{
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG )
		{
			if( ipZ==0 && nzone > 267 && iteration==2)
			{
				fprintf(ioQQQ," blr89bugg hphotodebugg%li\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\n",
					nzone,
				  HPhoto.hgamnc[ipZ][0],
				  HPhoto.hgamnc[ipZ][1],
				  HPhoto.hgamnc[ipZ][3],
				  HPhoto.hgamnc[ipZ][4],
				  HPhoto.hgamnc[ipZ][5],
				  HPhoto.hgamnc[ipZ][6]);
			}
		}
	}

	/* need not redo in most cases */
	if( opac.lgRedoStatic )
	{
		/* these are nu^-3 powerlaws */
		for( n=NHPLPHOT; n <= nHLevels[ipZ]; n++ )
		{
			if( HPhoto.lgHInducImp )
			{
				HPhoto.hgamnc[ipZ][n] = GammaBnPL(n,ipZ,
					&HPhoto.finduc[ipZ][n],
				  &HPhoto.cind[ipZ][n])*
				  PhotRate.PhotScaleOn;
			}
			else
			{
				HPhoto.hgamnc[ipZ][n] = (float)GammaPL(n,ipZ)*PhotRate.PhotScaleOn;
				HPhoto.finduc[ipZ][n] = 0.;
				HPhoto.cind[ipZ][n] = 0.;
			}
			/* hgamnc(n,ipZ) = hgamnc(n,ipZ) */
			HPHeat.HPhotHeat[ipZ][n] = heat.HeatNet*PhotRate.PhotScaleOn;
		}
	}

	/* now set induced recombination rate and cooling */
	if( HlteOk.lgHlteOk[ipZ] )
	{
		for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
		{
			/* induced recombination rate */
			HPhoto.rinduc[ipZ][n] = 
				(float)(HPhoto.finduc[ipZ][n]*hcolrt.hlte[ipZ][n]);

			/* cooling due to induced recombination */
			HPhoto.cinduc[ipZ][n] = 
				(float)(HPhoto.cind[ipZ][n]*hcolrt.hlte[ipZ][n]);
		}
	}
	else
	{
		for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
		{
			HPhoto.cinduc[ipZ][n] = 0.;
			HPhoto.rinduc[ipZ][n] = 0.;
		}
	}

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "     HydroPhoto%2ld low, hi=",ipZ);
		fprintf( ioQQQ,PrintEfmt("%9.2e", HPhoto.hgamnc[ipZ][ipH1s]));
		fprintf( ioQQQ,PrintEfmt("%9.2e", hgamsc.BoundCompton));
		fprintf( ioQQQ, " total=");
		fprintf( ioQQQ,PrintEfmt("%9.2e",HPhoto.hgamnc[ipZ][ipH1s] ));
		fprintf( ioQQQ, "\n");
	}

#	ifdef DEBUG_FUN
	fputs( " <->HydroPhoto()\n", debug_fp );
#	endif
	return;
}

