/*lines_molecules put energetics, H, and He lines into line intensity stack */
#include "cddefines.h"
#include "taulines.h"
#include "physconst.h"
#include "nhe1lvl.h"
#include "linesave.h"
#include "coolheavy.h"
#include "cooling.h"
#include "secondaries.h"
#include "hydrogenic.h"
#include "grainvar.h"
#include "opacity.h"
#include "totlum.h"
#include "heots.h"
#include "h.h"
#include "hmi.h"
#include "heat.h"
#include "comneg.h"
#include "heatlmax.h"
#include "hsrate.h"
#include "he1ex.h"
#include "he.h"
#include "smbeta.h"
#include "phycon.h"
#include "ionfracs.h"
#include "dphoht.h"
#include "compton.h"
#include "phe1lv.h"
#include "heinwd.h"
#include "he3c.h"
#include "he3tau.h"
#include "he3lines.h"
#include "dtrans.h"
#include "sphere.h"
#include "cionhe.h"
#include "codish.h"
#include "hfbnet.h"
#include "rfield.h"
#include "hcaseb.h"
#include "predcont.h"
#include "trace.h"
#include "photrate.h"
#include "ip626.h"
#include "iphe1l.h"
#include "ionrange.h"
#include "linadd.h"
#include "lindst.h"
#include "iwavlen.h"
#include "radius.h"
#include "putline.h"
#include "putextra.h"
#include "getmaxhline.h"
#include "lines.h"

void lines_molecules(void)
{
	long int i;

#	ifdef DEBUG_FUN
	fputs( "<+>lines_molecules()\n", debug_fp );
#	endif

	/* molecules */
	i = StuffComment( "molecules" );
	linadd( 0., i , "####", 'i',' ',' ');

	/* H2 rotation lines from Lepp and Shull ApJ 270, 578. */
	linadd(CoolHeavy.h2line,2,"H2 l",'c',' ',' ');

	/* H2 dissociation by H atoms (not e) */
	linadd(CoolHeavy.h2dis,0,"H2dC",'c',' ',' ');

	/* molecular hydrogen heating */
	hmi.h2dtot += hmi.HeatH2Dish*(float)radius.dVeff;
	hmi.h2dfrc = (float)MAX2(hmi.HeatH2Dish/heat.htot,hmi.h2dfrc);

	/* heating by H2 dissociation by Lyman continuum */
	linadd(hmi.HeatH2Dish,0,"H2dH",'h',' ',' ');

	/* heating by coll deexcit of vib-excited H2 */
	linadd( MAX2(0.,hmi.HeatH2Dexc),0,"H2vH",'h',' ',' ');

	/* cooling by coll deexcit of vib-excited H2 */
	linadd( MAX2(0.,-hmi.HeatH2Dexc) ,0,"H2vC",'c',' ',' ');

	/* line emission by vib-excited H2 */
	linadd( hmi.htwo_star*2e-7*4.17e-12,0,"H2 v",'c',' ',' ');

	/* neg H ion free-bound emission */
	linadd(CoolHeavy.hmfb,0,"H-FB",'c',' ',' ');

	/* H-alpha produce by H- mutual neutralization */
	linadd(hmi.HalphaHmin*3.032e-12,6563,"H-CT",'i',' ',' ');

	/* H- heating */
	linadd(MAX2(0.,hmi.hmihet),0,"H- H",'h',' ',' ');

	/* induced H- cooling */
	linadd(MAX2(0.,-hmi.hmihet),0,"H-Hc",'c',' ',' ');

	/* H+ + H => H2+ + photon continuum cooling */
	linadd(CoolHeavy.H2PlsCool,0,"H2+ ",'c',' ',' ');

	/* HeH+ formation cooling */
	linadd(MAX2(3.27e-12+phycon.te*BOLTZMANN,0.)*h.hii*he.hei*1e-20+
	  (1.76e-11+phycon.te*BOLTZMANN)*h.hi*he.heii*1e-16,0,"HEH+",'i',' ',' ');

	/* carbon monoxide heating */
	codish.codtot += codish.CODissHeat*(float)radius.dVeff;
	codish.codfrc = (float)MAX2(codish.CODissHeat/heat.htot,codish.codfrc);

	/* carbon monoxide co photodissociation */
	linadd(codish.CODissHeat,0,"COdh",'h',' ',' ');

	/* old cooling due to coll of vib rot levels */
	linadd(CoolHeavy.COVibCoolOld,0,"CO C",'i',' ',' ');

	/* c12o16 cooling */
	linadd(CoolHeavy.C12O16Rot,12,"CO  ",'c',' ',' ');

	/* c13o16 cooling */
	linadd(CoolHeavy.C13O16Rot,13,"CO  ",'c',' ',' ');

	/* remember most amount of CO cooling */
	comneg.COCoolBigFrac = MAX2( comneg.COCoolBigFrac, (CoolHeavy.C12O16Rot+CoolHeavy.C13O16Rot)/cooling.ctot);

	/* add in CO carbon monoxide lines */
	for( i=0; i < nCORotate; i++ )
	{
		PutLine(&C12O16Rotate[i]);
	}
	for( i=0; i < nCORotate; i++ )
	{
		PutLine(&C13O16Rotate[i]);
	}

#	ifdef DEBUG_FUN
	fputs( " <->lines_molecules()\n", debug_fp );
#	endif
	return;
}

