/*NextDR use adaptive logic to find next zone thickness */
/*ContRate called by nextdr to find energy of maximum continuum-gas interaction */
/*GrainRateDr called by nextdr to find grain heating rate dr */
/*ChkRate called by nextdr to check how rates of destruction of various species changes */
#include <float.h>
#include "cddefines.h"
#define	DNGLOB	0.10
#define	Z	1.0001
#include "nhe3lvl.h"
#include "angllum.h"
#include "opacity.h"
#include "drmin.h"
#include "destcrt.h"
#include "tff.h"
#include "hydrogenic.h"
#include "abundances.h"
#include "heavy.h"
#include "grainvar.h"
#include "sdrmin.h"
#include "photrate.h"
#include "converge.h"
#include "rfield.h"
#include "thermal.h"
#include "hmi.h"
#include "coolheavy.h"
#include "hextra.h"
#include "doppvel.h"
#include "fluct.h"
#include "stopcalc.h"
#include "colden.h"
#include "ionfracs.h"
#include "heat.h"
#include "h.h"
#include "phycon.h"
#include "radius.h"
#include "drminu.h"
#include "trace.h"
#include "filfac.h"
#include "didz.h"
#include "wind.h"
#include "ipdr.h"
#include "drneg.h"
#include "taulines.h"
#include "pressure.h"
#include "itercnt.h"
#include "dtmase.h"
#include "struc.h"
#include "he3gams.h"
#include "fndlineht.h"
#include "tabden.h"
#include "fabden.h"
#include "chlinelbl.h"
#include "nextdr.h"

/*ChkRate called by nextdr to check how rates of destruction of various species changes */
static void ChkRate(
	  /* element number on physical scale */
	  long int nelem, 
	  /* change in destruction rate */
	  double *dDestRate, 
	  /* old and new destruction rates */
	  double *DestRateOld,
	  double *DestRateNew,
	  /* stage of ionization on the physical scale */
	  long int *istage);

/*ContRate called by nextdr to find energy of maximum continuum-gas interaction */
static void ContRate(double *freqm, 
  double *opacm);

/*GrainRateDr called by nextdr to find grain heating rate dr */
static void GrainRateDr(double *grfreqm, 
  double *gropacm);

void NextDR(void)
{
	char chDestAtom[9], 
	  chLbl[11];
	int lgDoPun, 
	  lgOscil;

	long int icarstag, 
	  iironstag, 
	  initstag, 
	  ioxystag, 
	  ipStrong, 
	  istage, 
	  k, 
	  level, 
	  limit;

	double ThicknessSet , dThickness , drThickness , DepthToGo ;

	double DrGrainHeat, 
	  GlobDr, 
	  Out2Tot, 
	  SaveOHIIoHI, 
	  SpecDr, 
	  Strong, 
	  SvHe2oHe1, 
	  TauDTau, 
	  TauInwd, 
	  coleff, 
	  dDRCarbon, 
	  dDRIron, 
	  dDRNitrogen, 
	  dDROxygen, 
	  dDestRate, 
	  dEden, 
	  dHdStep, 
	  dRTaue, 
	  dTdStep, 
	  dnew, 
	  drConPres, 
	  drConvergeIoniz ,
	  drDepth, 
	  drDest, 
	  drEden, 
	  drFail, 
	  drFluc, 
	  drH2Abund, 
	  drH2Band, 
	  drHMase, 
	  drHe1ion, 
	  drHion, 
	  drInter, 
	  drLineHeat, 
	  drOutFrac, 
	  drPressure, 
	  drSphere, 
	  drTab, 
	  drdHdStep, 
	  drdTdStep, 
	  drmax, 
	  dt, 
	  error, 
	  fac2, 
	  factor, 
	  freqm, 
	  grfreqm, 
	  gropacm, 
	  hdnew, 
	  opacm, 
	  OldDR ,
	  OldestEden,
	  winddr, 
	  x;
	double DestOldCarb,DestNewCarb, DestOldNit, DestNewNit , DestOldOxy,DestNewOxy,
		DestOldIron,DestNewIron, DestOld, DestNew;

	static double OHIIoHI, 
	  OHe2oHe1 = 0., 
	  OldEden, 
	  OldHeat = 0., 
	  OldTe = 0.,
	  OldH2Abund=0.;

	static double BigRadius = 1e30;

#	ifdef DEBUG_FUN
	fputs( "<+>NextDR()\n", debug_fp );
#	endif


	/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	 *
	 * changes in logic
	 * 95 oct 19, drSphere now 3% of radius, was 2%, fewer zone
	 * 95 oct 19, subtracted grain opacity from total opacity used
	 * to get thickness in routine ContRate
	 *
	 *>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	 *
	 * free statement labels >= 13
	 *
	 *-----------------------------------------------------------------------
	 *
	 * this sub determines the thickness of the next zone
	 * if is called one time for each zone
	 * flag lgNxtDROff is true if this is initialization of NextDR,
	 * is false if we are to use logic to find dr
	 *
	 *----------------------------------------------------------------------- */

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "   NextDR called\n" );
	}

	/* save current dr */
	OldDR = radius.drad;

	/*  1    '' NextDR keys from change in H ionization'',e11.3)')
	 * check on change in hydrogen ionizaiton */
	if( nzone <= 1 )
	{
		if( h.hii > 0. && h.hi > 0. )
		{
			OHIIoHI = h.hii/h.hi;
		}
		else
		{
			OHIIoHI = 0.;
		}
		drHion = BigRadius;
		SaveOHIIoHI = h.hii/h.hi;
		/* else if(hii.gt.0. .and. hi.gt.0. .and. OHIIoHI.gt.0. ) then
		 * >>chng 97 jul 9, for deep in PDR vastly now ionz H slowed down works */
	}

	else if( (h.hii > 0. && h.hi > 0.) && OHIIoHI > 1e-15 )
	{
		/* ratio of current HII/HI to old value - < 1 when becoming more neutral */
		x = (h.hii/h.hi)/OHIIoHI;
		if( x < 1. )
		{
			x = 1. - x;
			/* >>chng 96 feb 23 from 0.7 to 0.3 because punched through H i-front
			 * >>chng 97 may 5, from 0.3 to 0.2 since punched through H i-front */
			drHion = radius.drad*MAX2( 0.2 , 0.2/MAX2(1e-10,x) );
		}
		else
		{
			drHion = BigRadius;
		}
		SaveOHIIoHI = OHIIoHI;
		OHIIoHI = h.hii/h.hi;
	}

	else
	{
		SaveOHIIoHI = OHIIoHI;
		if( h.hii > 0. && h.hi > 0. )
		{
			OHIIoHI = h.hii/h.hi;
		}
		else
		{
			OHIIoHI = 0.;
		}
		drHion = BigRadius;
	}

	/* "NextDR keys from H maser, dt, ij=" possible hydrogen maser action */
	if( dtMase.dTauMase < -0.01 )
	{
		/* maser so powerful, must limit inc in tay
		 * >>chng 96 aug 08, from 0.3 to 0.1 due to large maser crash */
		drHMase = radius.drad*MAX2(0.1,-0.2/dtMase.dTauMase);
	}
	else
	{
		drHMase = BigRadius;
	}

	/* NextDR keys from change in He2/He1 ionization, old=%11.3e sv=%11.3e FrLya*/
	/* check on change in HeI ionization */
	SvHe2oHe1 = OHe2oHe1;
	if( nzone <= 1 )
	{
		if( xIonFracs[2][ipHELIUM] > 0. && xIonFracs[1][ipHELIUM] > 0. )
		{
			OHe2oHe1 = xIonFracs[2][ipHELIUM]/xIonFracs[1][ipHELIUM];
		}
		else
		{
			OHe2oHe1 = 0.;
		}
		drHe1ion = BigRadius;
	}

	else if( (xIonFracs[2][ipHELIUM] > 0. && xIonFracs[1][ipHELIUM] > 0.) && OHe2oHe1 > 0. )
	{
		/* >>chng 97 jul 14, from 1e-2 to 1e-3 */
		/* >>chng 99 feb 16, this logic included a test for whether
		 * the ots fields of Lya were an important destruction mechanism 
		 * for the triplets.  changes in the helium ionization were NOT
		 * counted when ionization fraction was small and lya important.
		 * changed to simpler logic, only count changes when there is
		 * a significant amount of He+ */
		/*if( (he3gams.He3FrLya > 1e-3) && (he.heii/he.hei <= 1e-3) )*/
		if( xIonFracs[2][ipHELIUM]/xIonFracs[1][ipHELIUM] < 1e-3 )
		{
			/* >>chng 97 jul 12, He mostly neutral but Lya photoionized,
			 * oscillation could
			 * take place - do not trigger dr from this oscillation */
			drHe1ion = BigRadius;
		}
		else
		{
			/* this is the ratio of the current to old He2/He1 ratio */
			x = (xIonFracs[2][ipHELIUM]/xIonFracs[1][ipHELIUM])/OHe2oHe1;
			if( x < 1. )
			{
				x = 1. - x;
				/* >>chng 97 may 05, ran into i-fronts,
				 * >>chng 99 feb 24, from 0.02 to 0.03 in below */
				drHe1ion = radius.drad*
					MAX2(0.3,0.3/MAX2(1e-10,x));
			}
			else
			{
				drHe1ion = BigRadius;
			}
		}
		OHe2oHe1 = xIonFracs[2][ipHELIUM]/xIonFracs[1][ipHELIUM];
	}

	else
	{
		if( xIonFracs[2][ipHELIUM] > 0. && xIonFracs[1][ipHELIUM] > 0. )
		{
			OHe2oHe1 = xIonFracs[2][ipHELIUM]/xIonFracs[1][ipHELIUM];
		}
		else
		{
			OHe2oHe1 = 0.;
		}
		drHe1ion = BigRadius;
	}

	/* check on how much outward flux is being added relative to
	 * attenuated incident beam
	 *  1    '' NextDR keys from ratio out2 tot continuua'',e9.1)') */
	if( (rfield.SumIncCon > 0. && rfield.SumOutCon > 0.) && 
	  !thermal.lgTSetOn )
	{
		/* evaluated in metdif */
		Out2Tot = rfield.SumOutCon/rfield.SumIncCon;
		/* Out2HTot = SumHOutCon / SumHIncCon
		 * try to keep outward frac less than 5% of total */
		drOutFrac = radius.drad*MAX2(0.7,0.05/MAX2(1e-10,Out2Tot));
	}
	else
	{
		Out2Tot = FLT_MAX;
		drOutFrac = BigRadius;
	}

	/* check how heating is changing
	 * '' NextDR keys from change in heating; current, delta='', */
	if( nzone <= 1 || thermal.lgTSetOn )
	{
		drdHdStep = BigRadius;
		dHdStep = FLT_MAX;
	}
	else
	{
		dHdStep = fabs(heat.htot-OldHeat)/heat.htot;
		if( dHdStep > 0. )
		{
			if( phycon.hden >= 1e13 )
			{
				/* drdHdStep = drad * MAX( 0.8 , 0.05/dHdStep ) */
				drdHdStep = radius.drad*MAX2(0.8,0.075/dHdStep);
			}
			else if( phycon.hden >= 1e11 )
			{
				/* drdHdStep = drad * MAX( 0.8 , 0.075/dHdStep ) */
				drdHdStep = radius.drad*MAX2(0.8,0.100/dHdStep);
			}
			else
			{
				/* changed from .15 to .12 for outer edge of coolhii too steep dT
				 * changed to .10 for symmetry, big change in some rates, 95nov14
				 * changed from .10 to .125 since parispn seemed to waste zones
				 * >>chng 96 may 21, from .125 to .15 since pn's still waste zones */
				drdHdStep = radius.drad*MAX2(0.8,0.15/dHdStep);
			}
		}
		else
		{
			drdHdStep = BigRadius;
		}
	}
	OldHeat = heat.htot;

	/* pressure due to incident continuum if in eos */
	if( strcmp(pressure.chCPres,"CPRE") == 0 && pressure.lgConPres )
	{
		if( nzone > 2 && pressure.pinzon > 0. )
		{
			/* pinzon is pressrue from acceleration onto previos zone
			 * want this less than some fraction of total pressure */
			drConPres = 0.05*pressure.PressureInit/(wind.AccelTot*
			  phycon.xMassDensity*filfac.FillFac);
		}
		else
		{
			drConPres = BigRadius;
		}
	}
	else
	{
		drConPres = BigRadius;
	}

	/* check how temperature is changing
	 *  1    '' NextDR keys from change in temperature; current, delta='', */
	if( nzone <= 1 )
	{
		drdTdStep = BigRadius;
		dTdStep = FLT_MAX;
	}
	else
	{
		dTdStep = fabs(phycon.te-OldTe)/phycon.te;
		if( phycon.te > 1e4 )
		{
			/* >>chng 96 may 30, for high temps with flat cooling fcn,
			 * do not want too fine a test */
			x = 0.05;
		}
		else
		{
			/* >>chng 95 oct 13 from 0.05 to 0.03 for big te changes at
			 * outer edges of pn's and hii regions */
			x = 0.03;
		}
		if( dTdStep > 0. )
		{
			/* >>chng 96 may 30, variable depending on temp
			 * >>chng 96 may 31, allow 0.7 smaller, was 0.8
			 * >>chng 97 may 05, from 0.7 to 0.5 stop from punching through thermal front */
			drdTdStep = radius.drad*MAX2(0.7,x/dTdStep);
		}
		else
		{
			drdTdStep = BigRadius;
		}
	}
	OldTe = phycon.te;

	/* find freq where opacity largest and interaction rate is fastest
	 *  1  '' NextDR keys from cont inter nu='',e9.2, */
	ContRate(&freqm,&opacm);

	/* drChange is optical depth at max interaction energy
	 * >>chng 96 jun 06 was drChange=0.15 changed to 0.3 for high Z models
	 * taking too many zones
	 * drInter = drChange / MAX(1e-30,opacm*FillFac) */

	drInter = 0.3/MAX2(1e-30,opacm*filfac.FillFac*Angllum.AngleIllum);

	/* check on changes in destruction rates for various atoms
	 * call ChkRate( 1 , dDRHydro  ,istage )
	 * call ChkRate( 2 , dDRHelium ,istage ) */
	ChkRate(6,&dDRCarbon,&DestOldCarb, &DestNewCarb,&icarstag);
	ChkRate(7,&dDRNitrogen,&DestOldNit, &DestNewNit,&initstag);
	ChkRate(8,&dDROxygen,&DestOldOxy, &DestNewOxy,&ioxystag);
	ChkRate(26,&dDRIron,&DestOldIron, &DestNewIron,&iironstag);

	dDestRate = vfmax(dDROxygen,dDRIron,dDRCarbon,dDRNitrogen,FEND);

	if( dDRCarbon == dDestRate )
	{
		dDestRate = dDRCarbon;
		DestOld = DestOldCarb;
		DestNew = DestNewCarb;
		istage = icarstag;
		strcpy( chDestAtom, "Carbon  " );
	}

	else if( dDRNitrogen == dDestRate )
	{
		dDestRate = dDRNitrogen;
		DestOld = DestOldNit;
		DestNew = DestNewNit;
		istage = initstag;
		strcpy( chDestAtom, "Nitrogen" );
	}

	else if( dDROxygen == dDestRate )
	{
		dDestRate = dDROxygen;
		DestOld = DestOldOxy;
		DestNew = DestNewOxy;
		strcpy( chDestAtom, "Oxygen  " );
		istage = ioxystag;
	}

	else if( dDRIron == dDestRate )
	{
		dDestRate = dDRIron;
		DestOld = DestOldIron;
		DestNew = DestNewIron;
		istage = iironstag;
		strcpy( chDestAtom, "Iron    " );
	}

	else
	{
		fprintf( ioQQQ, " insanity following ChkRate\n" );
		ShowMe();
		puts( "[Stop in nextdr]" );
		cdEXIT(1);exit(1);
	}

	/*  NextDR keys from change in dest rates, atom= */
	if( dDestRate > 0. )
	{
		/* if( te.gt.40 000. ) then
		 * added different accuracy for hot gas since tend to jump over
		 * big te range for small change in heat (intrinsically unstable)
		 * drDest = drad * MAX( 0.5 , 0.10/dDestRate )
		 * else
		 * was drChange, changed to .15 for parishii go through HeII=HeI I front
		 * drDest = drad * MAX( 0.5 , 0.15/dDestRate )
		 * >>chng 95 dec 18 from above to below to stop oscillations
		 * >>chng 95 dec 27 from min of .5 to .75 to stop zone size from changing rapidly
		 * drDest = drad * MAX( 0.8 , 0.20 /dDestRate )
		 * >>chng 96 jan 14 from .2 to .25 to use less zones
		 * >>chng 96 may 30 from min of 0.8 to 0.5 to prevent crashing into He i-front */
		drDest = radius.drad*MAX2(0.5,0.20/dDestRate);
		/* endif */
	}
	else
	{
		drDest = BigRadius;
	}

	/* check whether change in wind velocity constrains DRAD,
	 * this is usual hypersonic wind */
	if( wind.windv > 0. )
	{
		if( wind.dVeldRad != 0. )
		{
			/* dVeldRad is D(vel)/vel / DRAD, computed in convpres */
			winddr = 0.03/wind.dVeldRad;
		}
		else
		{
			winddr = 1.1*radius.drad;
		}
	}
	/* this is D-critical wind, dVdr is negative, and we want
	 * small changes */
	else if( wind.windv < 0. )
	{
		if( wind.dVeldRad != 0. )
		{
			/* this is change in velocity per zone, in cm/sec */
			const double dVel=3e4;
			/* dVeldRad is D(vel)/vel / DRAD, computed in convpres */
			winddr = dVel / fabs(wind.dVeldRad);
		}
		else
		{
			winddr = 1.1*radius.drad;
		}
	}
	else
	{
		winddr = BigRadius;
	}

	/* inside out globule */
	if( strcmp(pressure.chCPres,"GLOB") == 0 )
	{
		if( radius.glbdst < 0. )
		{
			fprintf( ioQQQ, " Globule distance is negative, internal overflow has occured,  sorry.\n" );
			fprintf( ioQQQ, " This is routine NextDR, GLBDST is%10.2e\n", 
			  radius.glbdst );
			puts( "[Stop in nextdr]" );
			cdEXIT(1);exit(1);
		}
		factor = radius.glbden*pow(radius.glbrad/radius.glbdst,radius.glbpow);
		fac2 = radius.glbden*pow(radius.glbrad/(radius.glbdst - radius.drad),radius.glbpow);
		if( fac2/factor > 1. + DNGLOB )
		{
			/* DNGLOB is relative change in density allowed this zone, 0.10 */
			GlobDr = radius.drad*DNGLOB/(fac2/factor - 1.);
		}
		else
		{
			GlobDr = BigRadius;
		}
		/* GlobDr = GLBDST * DNGLOB * (GLBRAD/GLBDST)**(-GLBPOW) / GLBPOW */
		GlobDr = MIN2(GlobDr,radius.glbdst/20.);
	}
	else
	{
		GlobDr = BigRadius;
	}

	hdnew = 0.;
	if( strncmp( pressure.chCPres , "DLW" , 3) == 0 )
	{
		/* one of the special density laws, first get density at possible next dr */
		if( strcmp(pressure.chCPres,"DLW1") == 0 )
		{
			hdnew = fabden(radius.Radius+radius.drad,radius.depth+
			  radius.drad);
		}
		else if( strcmp(pressure.chCPres,"DLW2") == 0 )
		{
			hdnew = tabden(radius.Radius+radius.drad,radius.depth+
			  radius.drad);
		}
		else
		{
			fprintf( ioQQQ, " dlw insanity in NextDR\n" );
			puts( "[Stop in nextdr]" );
			cdEXIT(1);exit(1);
		}
		drTab = fabs(hdnew-phycon.hden)/MAX2(hdnew,phycon.hden);
		drTab = radius.drad*MAX2(0.2,0.10/MAX2(0.01,drTab));
	}
	else
	{
		drTab = BigRadius;
	}

	/* special density law */
	if( strcmp(pressure.chCPres,"DLW1") == 0 )
	{
		dnew = fabs(fabden(radius.Radius+radius.drad,radius.depth+
		  radius.drad)/phycon.hden-1.);
		/* DNGLOB is relative change in density allowed this zone, 0.10 */
		if( dnew == 0. )
		{
			SpecDr = radius.drad*3.;
		}
		else if( dnew/DNGLOB > 1.0 )
		{
			SpecDr = radius.drad/(dnew/DNGLOB);
		}
		else
		{
			SpecDr = BigRadius;
		}
	}
	else
	{
		SpecDr = BigRadius;
	}

	/* check grain line heating dominates
	 * this is important in PDR and HII region calculations
	 * >>chng 97 jul 03, added following check */
	if( heat.heating[13][0]/heat.htot > 0.2 )
	{
		/* >>chng 01 jan 03, following returns 0 when NO light at all,
		 * had failed with botched assert */
		GrainRateDr(&grfreqm,&gropacm);
		DrGrainHeat = 1.0/MAX2(1e-30,gropacm*filfac.FillFac*Angllum.AngleIllum);
	}
	else
	{
		gropacm = 0.;
		grfreqm = 0.;
		DrGrainHeat = BigRadius;
	}

	/* check if line heating dominates
	 * this is important in high metallicity models */
	if( heat.heating[22][0]/heat.htot > 0.2 )
	{
		FndLineHt(&level,&ipStrong,&Strong);
		if( Strong/heat.htot > 0.1 )
		{
			if( level == 1 )
			{
				/* a level1 line was the heat source (usual case)
				 * drLineHeat = MAX(1.0,TauLines(ipLnTauIn,ipStrong)*0.2) /
				 *  1      TauLines(ipLnDTau,ipStrong) */
				TauInwd = TauLines[ipStrong].TauIn;
				TauDTau = TauLines[ipStrong].dTau;
			}
			else if( level == 2 )
			{
				/* a level2 line was the heat source
				 * (bad case since not as well treated)
				 * drLineHeat = MAX(1.0,WindLine(ipLnTauIn,ipStrong)*0.2) /
				 *  1      WindLine(ipLnDTau,ipStrong) */
				TauInwd = TauLine2[ipStrong].TauIn;
				TauDTau = TauLine2[ipStrong].dTau;
			}
			else if( level == 3 )
			{
				/* a 12CO line was the heat source
				 * (bad case since not as well treated)
				 * drLineHeat = MAX(1.0,WindLine(ipLnTauIn,ipStrong)*0.2) /
				 *  1      WindLine(ipLnDTau,ipStrong) */
				TauInwd = C12O16Rotate[ipStrong].TauIn;
				TauDTau = C12O16Rotate[ipStrong].dTau;
			}
			else if( level == 4 )
			{
				/* a 13CO line was the heat source
				 * (bad case since not as well treated)
				 * drLineHeat = MAX(1.0,WindLine(ipLnTauIn,ipStrong)*0.2) /
				 *  1      WindLine(ipLnDTau,ipStrong) */
				TauInwd = C13O16Rotate[ipStrong].TauIn;
				TauDTau = C13O16Rotate[ipStrong].dTau;
			}
			else
			{
				/* this is insane, since Strong was set, but not level */
				insane();
				fprintf( ioQQQ, " NextDR Strong line heating set, but not level.\n" );
				drLineHeat = BigRadius;
				ipStrong = 0;
				Strong = 0.;
				puts( "[Stop in nextdr]" );
				cdEXIT(1);exit(1);
			}
			/* in following logic cannot use usual inverse opacity,
			 * since line heating competes with escape probability,
			 * so is effective at surprising optical depths */
			if( TauDTau > 0. )
			{
				drLineHeat = MAX2(1.,TauInwd)*0.4/TauDTau;
			}
			else
			{
				drLineHeat = BigRadius;
			}
		}
		else
		{
			TauInwd = 0.;
			drLineHeat = BigRadius;
			ipStrong = 0;
			Strong = 0.;
		}
	}
	else
	{
		TauInwd = 0.;
		drLineHeat = BigRadius;
		ipStrong = 0;
		level = 0;
		Strong = 0.;
	}

	/* Lyman band optical depth
	 * second term total lyman band optical depth, do not check if thick
	 * H2Opacity is opacity evaluated in hmole */
	if( hmi.htwo*hmi.H2Opacity > 1e-20 && coldenCom.colden[IPCOLH2-1]*
	  hmi.H2Opacity < 5. )
	{
		if( coldenCom.colden[IPCOLH2-1]*hmi.H2Opacity < 2. )
		{
			/* special extra 2 since H2 will change by this amount early on */
			drH2Band = (didz.drChange/2.)/(hmi.htwo*hmi.H2Opacity*
			  filfac.FillFac);
		}
		else
		{
			drH2Band = didz.drChange/(hmi.htwo*hmi.H2Opacity*filfac.FillFac);
		}
	}
	else
	{
		drH2Band = BigRadius;
	}

	/* if H2 cooling - if important cooling source then keep track of abundance */
	if( nzone <= 1 || thermal.lgTSetOn  )
	{
		drH2Abund = BigRadius;
	}
	else
	{
		if( OldH2Abund > SMALLFLOAT )
		{
			dHdStep = fabs(hmi.htwo/phycon.hden -OldH2Abund)/OldH2Abund;
			dHdStep = MAX2( SMALLFLOAT , dHdStep );
			/* do this only if significant */
			if( CoolHeavy.h2line/heat.htot > 0.05 )
			{
				drH2Abund = radius.drad*MAX2(0.1,0.075/dHdStep);
			}
			else
			{
				drH2Abund = radius.drad*MAX2(0.8,0.2/dHdStep);
				drH2Abund = BigRadius;
			}
		}
		else 
		{
			drH2Abund = BigRadius;
		}
		/*fprintf(ipdr.ipDRout,"h2dr dr%.2e\t%.2e\t%.2e\n",CoolHeavy.h2line/heat.htot, dHdStep, 0.075/dHdStep);*/
	}
	OldH2Abund = hmi.htwo/phycon.hden;

	/* can't make drmax large deep in model since causes feedback
	 * oscillations with changes in heating or destruction rates
	 * >>chng 96 oct 15, change from 5 to 10 */
	if( nzone < 11 )
	{
		lgOscil = FALSE;
		if( nzone < 5 )
		{
			/* >>chng 96 nov 11, had been 4 * drad up to 11, change to following
			 * to be similar to c90.01, max of 1.3 between 5 and 11 */
			drmax = 4.*radius.drad/Angllum.AngleIllum;
		}
		else
		{
			drmax = 1.3*radius.drad/Angllum.AngleIllum;
		}
	}
	else
	{
		/* >>chng 96 oct 15, do not let zones increase if oscillations present */
		lgOscil = FALSE;
		/* this routine called before RTOptDepthIncre stores variables
		 * >>chng 96 oct 31, error to declare oscillation propto toler, the 
		 *heating cooling tolerance */
		error = conv.HeatCoolRelError*conv.HeatCoolRelError;
		limit = MIN2((long)NZLIM-2,nzone-2);
		for( k=nzone - 10; k < limit; k++ )
		{
			/* small residiual is to allow 0.01 rel error */
			if( (struc.testr[k-1] - struc.testr[k])/struc.testr[k]*
			  (struc.testr[k] - struc.testr[k+1])/struc.testr[k] < 
			  -(float)error )
			{
				lgOscil = TRUE;
			}
			/* small residiual is to allow 0.01 rel error */
			if( (struc.ednstr[k-1] - struc.ednstr[k])/struc.ednstr[k]*
			  (struc.ednstr[k] - struc.ednstr[k+1])/struc.ednstr[k] < 
			  -(float)error )
			{
				lgOscil = TRUE;
			}
		}
		if( lgOscil )
		{
			/* >>chng 96 oct 15, do not let zones increase if oscillations present */
			drmax = radius.drad;
		}
		else
		{
			/* >>chng 96 jan 15 was 1.15 - why so small? */
			drmax = 1.3*radius.drad;
		}
	}

	/* check on several convergence criteria */

	/* is current pressure ok? */
	if( conv.lgConvPres )
	{
		drPressure = BigRadius;
	}
	else if( nzone > 0 )
	{
		/* first zone usually has pressure fail due to bug in ptot */
		drPressure = radius.drad/2.;
	}
	else
	{
		drPressure = BigRadius;
	}

	/* >>chng 01 apr 05, add this logic */
	/* an ionization convergence failure of some kind did occur, although
	 * it may no longer be a problem.  Do not let zone get larger */
	if( conv.lgConvIonizThisZone )
	{
		drConvergeIoniz = radius.drad;
	}
	else 
	{
		drConvergeIoniz = BigRadius;
	}

	if( !conv.lgConvTemp )
	{
		drFail = radius.drad/2.;
	}
	else
	{
		drFail = BigRadius;
	}

	/* change in electron density - NextDR keys from change in elec den,
	 * remember old electron density during first call */
	if( nzone == 0 )
	{
		OldEden = phycon.eden;
	}

	/* this is low ionization solution */
	if( phycon.eden/phycon.hden <= 1. )
	{
		dEden = fabs(phycon.eden-OldEden)/phycon.eden;
		if( dEden > 0. )
		{
			/* >>chng 96 may 15, down to .03 from 0.04
			 * >>chng 96 may 17, up to 0.1 from .03 */
			/* >>chng 99 nov 23, smallest change is 0.3, had been 0.7,
			 * smaller value needed due to large initial changes in ionization
			 * in 94 since code does not make attempt to identify drnext
			 * and stay in middle of zone, as result some near-lte dense models
			 * can have rapid changes in ionization when due to pumped lines,
			 * see dalton.in for examples */
			/* >>chng 99 nov 25, allow dr to become as much as 0.2 smaller,
			 * for dense moels where initial change in eden is large at ill face.
			 * had been 0.2 */
			/* >>chng 01 mar 24, ratio had allowed 10% changes in electron fraction,
			 * in pdr models this was too big, and ConvIoniz did not converge,
			 * change to 0.04 */
			/*drEden = radius.drad*MAX2(0.2,0.10/dEden);*/
			drEden = radius.drad*MAX2(0.2,0.04/dEden);
		}
		else
		{
			drEden = BigRadius;
		}
	}
	else
	{
		dEden = 0.;
		drEden = BigRadius;
	}
	OldestEden = OldEden;
	OldEden = phycon.eden;

	/* do not let thickness get too large
	 *  1    '' NextDR keys from relative depth'',e11.3)') */
	if( nzone > 20 )
	{
		drDepth = radius.depth/5.;
	}
	else
	{
		drDepth = BigRadius;
	}
	
	/* case where stopping thickness or edge specified, need to approach slowly */
	ThicknessSet = -1.;
	dThickness = -1.;
	DepthToGo = -1.;
	if( StopCalc.HColStop < 5e29 )
	{
		ThicknessSet = StopCalc.HColStop;
		dThickness = phycon.hden*filfac.FillFac;
		DepthToGo = StopCalc.HColStop-coldenCom.colden[IPCOLUMNDENSITY-1];
	}

	else if( StopCalc.colpls < 5e29 )
	{
		ThicknessSet = StopCalc.colpls;
		dThickness = (double)(h.hii)*filfac.FillFac;
		DepthToGo = StopCalc.colpls-coldenCom.colden[IPCHII-1];
	}

	else if( StopCalc.colnut < 5e29 )
	{
		ThicknessSet = StopCalc.colnut;
		dThickness = (double)(h.hi)*filfac.FillFac;
		DepthToGo = StopCalc.colnut - coldenCom.colden[IPCHI-1] ;
	}

	/* this is case where outer radius is set */
	if( radius.router[iteration-1] < 5e29 )
	{
		ThicknessSet = radius.router[iteration-1] ;
		dThickness = 1.;
		DepthToGo = radius.router[iteration-1] - radius.depth ;
	}

	/* set dr if one of above tests have triggered */
	if( ThicknessSet > 0. && DepthToGo > 0.)
	{
		/* want to approach the outer edge slowly,
		 * the need for this logic is most evident in brems.in - 
		 * HI fraction varies across coronal model */
		DepthToGo = MIN2( ThicknessSet/10. , DepthToGo );
		drThickness = DepthToGo / dThickness ;
	}
	else
	{
		drThickness = BigRadius ;
	}
	/*fprintf(ioQQQ," drThickness = %e %e %e \n", drThickness, DepthToGo , dThickness );*/

	/* spherical models, do not want delta R/R big */
	drSphere = radius.Radius*0.03;
	drSphere = radius.Radius*0.04;

	/* optical depth to electron scattering */
	dRTaue = didz.drChange/(phycon.eden*6.65e-25);

	if( fluct.flong != 0. )
	{
		/* FacAbunSav = (cfirst * COS( depth*flong ) + csecnd)
		 * this is option for rapid density fluctuations
		 * set DR to roughly 0.1 of fluctuation length
		 * read flcsub first parameter (log of period) is used to get flong
		 * flong = 6.2831853 / period in flcsub */
		drFluc = 0.628/2./fluct.flong;
	}
	else
	{
		drFluc = BigRadius;
	}

	/* if density fluctuations in place then override change in heat
	 * for dr set */
	if( strcmp(pressure.chCPres,"SINE") == 0 && fluct.lgDenFluc )
	{
		drdHdStep = BigRadius;
	}

	/*active dr sets */
	/* we are deep into model, use logic since we have several zones
	 * of old data */
	radius.drNext = vfmin(drmax,drInter,drLineHeat,winddr,drFluc,GlobDr,
	  DrGrainHeat,FEND);
	radius.drNext = vfmin(radius.drNext,SpecDr,drFail,drSphere,sdrminCom.sdrmax,
	  drPressure,drConvergeIoniz,dRTaue,FEND);
	radius.drNext = vfmin(radius.drNext,drH2Band,drH2Abund,drDest,drdTdStep,drdHdStep,
	  drConPres,drTab,FEND);
	radius.drNext = vfmin(radius.drNext,drOutFrac,drHion,drHe1ion,drDepth,
	  drEden,drHMase,drThickness,FEND);

	/* keep dr constant in first two zones, if it wants to increase,
	 * but always allow it to decrease.
	 * to guard against large increases in efrac for balmer cont photo dominated models,
	 */
	if( nzone <= 1 && radius.drNext > OldDR)
	{
		radius.drNext = OldDR;
	}

	/* option to force min drad */
	if( radius.drNext < sdrminCom.sdrmin )
	{
		radius.drNext = sdrminCom.sdrmin;
	}

	/* dr = zero is a logical mistake */
	if( radius.drNext <= 0. )
	{
		fprintf( ioQQQ, " NextDR finds insane drNext:%10.2e\n", 
		  radius.drNext );
		fprintf( ioQQQ, " all drs follow:%10.2e%10.2e%10.2e%10.2e%10.2e%10.2e%10.2e\n all drs follow:%10.2e%10.2e%10.2e%10.2e%10.2e\n all drs follow:%10.2e%10.2e%10.2e%10.2e%10.2e\n", 
		  drmax, drInter, drLineHeat, winddr, drFluc, GlobDr, SpecDr, 
		  drFail, drSphere, sdrminCom.sdrmax, drPressure, drConvergeIoniz,dRTaue, 
		  drH2Band, OldH2Abund,drOutFrac, drDepth );
		puts( "[Stop in nextdr]" );
		cdEXIT(1);exit(1);
	}

	/* all this is to only punch on last iteration
	 * the punch dr command is not really a punch command, making this necessary
	 * lgDRon is set true if "punch dr" entered */
	if( ipdr.lgDROn )
	{
		if( ipdr.lgDRPLst )
		{
			/* lgDRPLst was set true if "punch" had "last" on it */
			if( IterCnt.lgLastIt )
			{
				lgDoPun = TRUE;
			}
			else
			{
				lgDoPun = FALSE;
			}
		}
		else
		{
			lgDoPun = TRUE;
		}
	}
	else
	{
		lgDoPun = FALSE;
	}
	if( (trace.lgTrace && trace.lgDrBug) || lgDoPun )
	{
		if( !conv.lgConvTemp && nzone > 0 )
		{
			fprintf( ipdr.ipDRout, " >>>> A temperature failure occured.\n" );
		}
		if( !conv.lgConvPres && nzone > 0 )
		{
			fprintf( ipdr.ipDRout, " >>>> A pressure failure occured.\n" );
		}

		/* this is common part of each line, the zone count, depth, chosen dr, and depth2go */
		fprintf( ipdr.ipDRout , "%ld\t%.3e\t%.3e\t%.3e\t", nzone, radius.depth, radius.drNext, radius.Depth2Go );

		/*=======begin active dr sets */
		if( radius.drNext == drLineHeat )
		{
			if( level == 1 )
			{
				strcpy( chLbl, chLineLbl(&TauLines[ipStrong]) );
				fprintf( ipdr.ipDRout, "level 1 line heating,%10.10s TauIn%10.2e%10.2e%10.2e\n", 
				  chLbl, TauInwd, TauLines[ipStrong].pump, 
				  TauLines[ipStrong].Pesc );
			}
			else if( level == 2 )
			{
				strcpy( chLbl, chLineLbl(&TauLine2[ipStrong]));
				fprintf( ipdr.ipDRout, "level 2 line heating,%10.10s TauIn%10.2e%10.2e%10.2e\n", 
				  chLbl, TauInwd, TauLine2[ipStrong].pump, 
				  TauLine2[ipStrong].Pesc );
			}
			else
			{
				fprintf( ioQQQ, " insanity pr line heat\n" );
				puts( "[Stop in nextdr]" );
				cdEXIT(1);exit(1);
			}
		}

		else if( radius.drNext == drDepth )
		{
			fprintf( ipdr.ipDRout, "relative depth\n");
		}

		else if( radius.drNext == drThickness )
		{
			fprintf( ipdr.ipDRout, "depth to go\n");
		}

		else if( radius.drNext == drTab )
		{
			fprintf( ipdr.ipDRout, "spec den law, new old den%10.2e%10.2e\n", 
			  hdnew, phycon.hden );
		}

		else if( radius.drNext == drHMase )
		{
			fprintf( ipdr.ipDRout, "H maser, dt, ij=%10.2e%5ld\n", 
			  dtMase.dTauMase, dtMase.ijMase );
		}

		else if( radius.drNext == drHion )
		{
			fprintf( ipdr.ipDRout, "change in H ionization fm to%11.3e%11.3e\n", 
			  SaveOHIIoHI, OHIIoHI );
		}

		else if( radius.drNext == drEden )
		{
			fprintf( ipdr.ipDRout, 
				"change in elec den, rel chng:%11.3e, cur %g old=%g\n", 
			  dEden , OldEden , OldestEden );
		}

		else if( radius.drNext == drHe1ion )
		{
			fprintf( ipdr.ipDRout, 
				"change in He2/He1 ionization, new=%11.3e old=%11.3e FrLya%11.3e\n", 
			  OHe2oHe1, 
			  SvHe2oHe1, 
			  he3gams.He3FrLya );
		}

		else if( radius.drNext == drDest )
		{
			fprintf( ipdr.ipDRout, 
				"change in dest rates, atom=%s%3ld old rate %.2e new %.2e\n", 
			  chDestAtom, istage ,DestOld , DestNew );
		}

		else if( radius.drNext == drOutFrac )
		{
			fprintf( ipdr.ipDRout, 
				"ratio out2 tot continuua%10.2e%10.2e%7ld %4.4s %4.4s\n", 
			  Out2Tot, 
			  rfield.SumOutMax, 
			  rfield.ipSumOutMax, 
			  rfield.chLineLabel[rfield.ipSumOutMax-1], 
			  rfield.chContLabel[rfield.ipSumOutMax-1] );
			/* in above SumOutMax is anu where peak occured */
		}

		else if( radius.drNext == drdHdStep )
		{
			fprintf( ipdr.ipDRout, 
				"change in heating; current %10.3e delta=%10.3e\n", 
			  heat.htot, dHdStep );
		}

		else if( radius.drNext == drConPres )
		{
			fprintf( ipdr.ipDRout, "change in con accel\n" 
			  );
		}

		else if( radius.drNext == drdTdStep )
		{
			fprintf( ipdr.ipDRout, 
				"change in temperature; current= %10.3e, delta= %10.3e\n", 
			  phycon.te, dTdStep );
		}

		else if( radius.drNext == sdrminCom.sdrmin )
		{
			fprintf( ipdr.ipDRout, "sdrmin\n" 
			  );
		}

		else if( radius.drNext == drH2Band )
		{
			fprintf( ipdr.ipDRout, "H2 Lyman Band - opacity=%10.2e\n", 
			  hmi.htwo*hmi.H2Opacity );
		}

		else if( radius.drNext == drH2Abund )
		{
			fprintf( ipdr.ipDRout, "H2 abundance - %10.2e\n", 
			  drH2Abund );
		}

		else if( radius.drNext == sdrminCom.sdrmax )
		{
			fprintf( ipdr.ipDRout, "sdrmax\n" 
			  );
		}

		else if( radius.drNext == drSphere )
		{
			fprintf( ipdr.ipDRout, "sphericity\n" 
			  );
		}

		else if( radius.drNext == dRTaue )
		{
			fprintf( ipdr.ipDRout, 
				"optical depth to electron scattering\n" 
			  );
		}

		else if( radius.drNext == drPressure )
		{
			fprintf( ipdr.ipDRout, 
				"pressure failure\n" 
			   );
		}

		else if( radius.drNext == drConvergeIoniz )
		{
			fprintf( ipdr.ipDRout, 
				"ionization convergence inhibited\n" 
			   );
		}

		else if( radius.drNext == drFail )
		{
			fprintf( ipdr.ipDRout, 
				"temperature failure.\n" 
			  );
		}

		else if( radius.drNext == drmax )
		{
			fprintf( ipdr.ipDRout, 
				"DRMAX; nu opc dr=%10.2e%10.2e%10.2e oscil=%2c\n", 
			  freqm, opacm, didz.drChange/
			  opacm, TorF(lgOscil) );
		}

		else if( radius.drNext == drInter )
		{
			fprintf( ipdr.ipDRout, 
				"cont inter nu=%10.2e opac=%10.2e\n", 
			  freqm, opacm );
		}

		else if( radius.drNext == DrGrainHeat )
		{

			fprintf( ipdr.ipDRout, 
				"grain heating nu=%10.2e opac=%10.2e\n", 
			  grfreqm, gropacm );
		}

		else if( radius.drNext == winddr )
		{
			fprintf( ipdr.ipDRout, 
				"Wind, dVeldRad=%10.3e\n", 
			   wind.dVeldRad );
		}

		else if( radius.drNext == drFluc )
		{
			fprintf( ipdr.ipDRout, 
				"density fluctuations\n" 
			   );
		}

		else if( radius.drNext == GlobDr )
		{
			fprintf( ipdr.ipDRout, 
				"GLOB law new dr=%10.2e HDEN=%10.2e\n", 
				GlobDr,
			  phycon.hden );
		}

		else if( radius.drNext == SpecDr )
		{
			fprintf( ipdr.ipDRout, 
				"special law new dr=%10.2e HDEN=%10.2e\n", 
				SpecDr,
			  phycon.hden );
		}

		else if( radius.drNext == OldDR )
		{
			fprintf( ipdr.ipDRout, "old DR.\n" 
			  );
		}

		else
		{
			fprintf( ipdr.ipDRout, 
				" %4ld NextDR keys from insanity %10.2e\n", 
			  nzone, radius.drNext );

			fprintf( ioQQQ, 
				" %4ld NextDR keys from insanity %10.2e\n", 
			  nzone, radius.drNext );
			puts( "[Stop in nextdr]" );
			cdEXIT(1);exit(1);
		}

		/*======end active dr sets */
	}

	/* this is general code that prevents zone thickness drNext from
	 * becoming too thin, something that can happen for various bad reasons
	 * HOWEVER we do not want to do this test for certain density laws,
	 * for which very small zone thicknesses are unavoidable
	 * the special cases are:
	 * special density law,
	 * globule density law,
	 * not at carbon +-0 i front
	 * not flucuations command
	 * drMinimum was set in FirstDR to either sdrmin (set drmin) or
	 * some fraction of the initial radius - it is always set
	 * to something non-trivial.  
	 * sdrmin is only set wih the "set dr" command */

	if( ((strcmp(pressure.chCPres,"DLW1") != 0 && 
		strcmp(pressure.chCPres ,"GLOB") != 0) && 
		xIonFracs[1][5]/xIonFracs[0][5] < 0.05) && 
		(fluct.flong == 0.) )
	{
		/* don't let dr get smaller than drMinimum, if this resets drNext
		 * then code stops with warning that zones got too thin */
		if( radius.drNext < drmin.drMinimum )
		{
			radius.drNext = drmin.drMinimum;
			/* leaving this at true will cause the model to stop with a warning
			 * that the zone thickness is too small */
			drminu.lgDrMinUsed = TRUE;
		}
	}

	/* following is to make thickness of model exact
	 * n.b., on last zone, drNext can be NEGATIVE!!
	 * DEPTH was incremented at start of zone calc in newrad,
	 * has been outer edge of zone all throughout */
	radius.drNext = MIN2(radius.drNext,(radius.router[iteration-1]-
	  radius.depth)*Z);

	/* this means outer limit exceeded */
	if( radius.drNext < 0. )
	{
		drneg.lgDrNeg = TRUE;
	}
	else
	{
		drneg.lgDrNeg = FALSE;
	}

	if( StopCalc.HColStop < 5e29 )
	{
		coleff = phycon.hden*filfac.FillFac;
		radius.drNext = 
			MIN2(
			radius.drNext,
			(StopCalc.HColStop-coldenCom.colden[IPCOLUMNDENSITY-1]-radius.drad*coleff)*Z/coleff);
	}

	else if( StopCalc.colpls < 5e29 )
	{
		coleff = h.hii*filfac.FillFac;
		radius.drNext = 
			MIN2(
			radius.drNext,
			(StopCalc.colpls-coldenCom.colden[IPCHII-1]- radius.drad*coleff)*Z/coleff);
	}

	else if( StopCalc.colnut < 5e29 )
	{
		coleff = h.hi*filfac.FillFac;
		/* >>chng 97 oct 30, prevent overflow for very high U models */
		if( (StopCalc.colnut - coldenCom.colden[IPCHI-1] - (float)(radius.drad)*
		  coleff) < coleff*radius.drNext )
		{
			radius.drNext = (StopCalc.colnut - coldenCom.colden[IPCHI-1] - 
			  radius.drad*coleff)*Z/coleff;
		}
	}

	if( StopCalc.iptnu != NCELL )
	{
		/* end optical depth has been specified */
		dt = opac.opac[StopCalc.iptnu-1]*filfac.FillFac;
		radius.drNext = 
			MIN2(radius.drNext,(StopCalc.tauend-opac.TauAbsGeo[0][StopCalc.iptnu-1]-
		  radius.drad*dt)/MAX2(SMALLFLOAT,dt) );
	}

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, " NextDR chooses next drad=%12.4e; this drad was%12.4e\n", 
		  radius.drNext, radius.drad );
	}

#	ifdef DEBUG_FUN
	fputs( " <->NextDR()\n", debug_fp );
#	endif
	return;
}

/*ContRate called by nextdr to find energy of maximum continuum-gas interaction */
static void ContRate(double *freqm, 
  double *opacm)
{
	long int i, 
	  ipHi,
	  iplow, 
	  limit;
	double FreqH, 
	  FreqSub, 
	  OpacH, 
	  OpacSub, 
	  xMaxH, 
	  xMaxSub;

#	ifdef DEBUG_FUN
	fputs( "<+>ContRate()\n", debug_fp );
#	endif

	/* 
	 * find maximum continuum interaction rate,
	 * these should be reset in following logic without exception,
	 * if they are still zero at the end we have a logical error 
	 */
	xMaxSub = 0.;
	FreqSub = 0.;
	OpacSub = 0.;

	/* do up to carbon photo edge if carbon is turned on */
	/* >>>chng 00 apr 07, add test for whether element is turned on */
	if( abundances.lgElmtOn[5] )
	{
		/* carbon is turned on, use carbon 1 edge */
		ipHi = Heavy.ipHeavy[0][5] - 1;
	}
	else
	{
		/* carbon truned off, use hydrogen balmer continuum */
		ipHi = hydro.ipHLevNIonCon[ipHYDROGEN][ipH2s]-1;
	}

	for( i=1; i < ipHi; i++ )
	{
		/* this does not have grain opacity since grains totally passive
		 * at energies below CI edge */
		if( rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*(opac.opac[i] - 
		  gv.dstab[i]*phycon.hden) > xMaxSub )
		{
			xMaxSub = rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*
			  (opac.opac[i] - gv.dstab[i]*phycon.hden);
			FreqSub = rfield.anu[i];
			OpacSub = opac.opac[i] - gv.dstab[i]*phycon.hden;
		}
	}

	/* not every continuum extends beyond C edge-this whole loop can add to zero
	 * use total opacity here
	 * test is to put in fir continuum if free free heating is important */
	if( hydro.FreeFreeHeat/heat.htot > 0.05 )
	{
		/* this is pointer to energy where cloud free free optical depth is unity */
		iplow = tffCom.ntff;
	}
	else
	{
		/* >>>chng 00 apr 07, from Heavy.ipHeavy[0][5] to ipHi defined above, since
		 * would crash if element not defined */
		iplow = ipHi;
	}

	limit = MIN2(rfield.nflux,hydro.ipHLevNIonCon[ipHYDROGEN][ipH1s]-1);
	for( i=iplow-1; i < limit; i++ )
	{
		if( rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*(opac.opac[i] - 
		  gv.dstab[i]*phycon.hden) > xMaxSub )
		{
			xMaxSub = rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*
			  (opac.opac[i] - gv.dstab[i]*phycon.hden);
			FreqSub = rfield.anu[i];
			OpacSub = opac.opac[i] - gv.dstab[i]*phycon.hden;
		}
	}

	/* variables to check continuum interactions over lyman continuum */
	xMaxH = 0.;
	OpacH = 0.;
	FreqH = 0.;

	/* not every continuum extends beyond 1 Ryd-this whole loop can add to zero */
	for( i=hydro.ipHLevNIonCon[0][ipH1s]-1; i < rfield.nflux; i++ )
	{
		if( rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*(opac.opac[i] - 
		  gv.dstab[i]*phycon.hden) > xMaxH )
		{
			/* xMaxH = anu(i)*flux(i)/widflx(i)*opac(i) */
			xMaxH = rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*
			  (opac.opac[i] - gv.dstab[i]*phycon.hden);
			FreqH = rfield.anu[i];
			OpacH = opac.opac[i] - gv.dstab[i]*phycon.hden;
		}
	}

	/* use lyman continuum if its opacity is larger than non-h ion */
	if( xMaxSub < 1e-30 )
	{
		/* this happens for laser source - use Lyman continuum */
		*opacm = OpacH;
		*freqm = FreqH;
	}
	else if( OpacH > OpacSub && xMaxH/xMaxSub > 1e-10 )
	{
		/* use Lyman continuum */
		*opacm = OpacH;
		*freqm = FreqH;
	}
	else
	{
		/* not much rate in the lyman continuum, stick with low energy */
		*opacm = OpacSub;
		*freqm = FreqSub;
	}

	if( xMaxH > xMaxSub )
	{
		*opacm = OpacH;
		*freqm = FreqH;
	}
	else
	{
		*opacm = OpacSub;
		*freqm = FreqSub;
	}

	/*begin sanity check
	 * these were set to zero at start, and must have been reset if one of the
	 * two loops.  Logic error if still zero. 
	 */
	if( *opacm <= 0. || *freqm <= 0. )
	{
		fprintf( ioQQQ, " ContRate found non positive interaction, or energy.\n" );
		fprintf( ioQQQ, " This is impossible.\n" );
		ShowMe();
		puts( "[Stop in contrate]" );
		cdEXIT(1);
	}
	/*end sanity check*/

#	ifdef DEBUG_FUN
	fputs( " <->ContRate()\n", debug_fp );
#	endif
	return;
}

/*GrainRateDr called by nextdr to find grain heating rate dr */
static void GrainRateDr(double *grfreqm, 
  double *gropacm)
{
	long int i, 
	  iplow;
	double xMax;

#	ifdef DEBUG_FUN
	fputs( "<+>GrainRateDr()\n", debug_fp );
#	endif

	/* in all following changed from anu2 to anu  july 25 95
	 *
	 * find maximum continuum interaction rate */

	/* not every continuum extends beyond C edge-this whole loop can add to zero
	 * use total opacity here
	 * test is to put in fir continuum if free free heating is important */
	if( hydro.FreeFreeHeat/heat.htot > 0.05 )
	{
		/* this is pointer to energy where cloud free free optical depth is unity */
		iplow = tffCom.ntff;
	}
	else
	{
		/* do up to carbon photo edge if carbon is turned on */
		/* >>>chng 00 apr 07, add test for whether element is turned on */
		if( abundances.lgElmtOn[5] )
		{
			/* carbon is turned on, use carbon 1 edge */
			iplow = Heavy.ipHeavy[0][5];
		}
		else
		{
			/* carbon truned off, use hydrogen balmer continuum */
			iplow = hydro.ipHLevNIonCon[ipHYDROGEN][ipH2s];
		}

	}

	xMax = -1.;
	/* integrate up to H edge */
	for( i=iplow-1; i < Heavy.ipHeavy[0][0]; i++ )
	{
		if( rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*opac.opac[i] > xMax )
		{
			xMax = rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*
			  opac.opac[i];
			*grfreqm = rfield.anu[i];
			*gropacm = opac.opac[i];
		}
	}
	/* integrate up to heii edge if he is turned on,
	 * this logic will not make sense if grains on but he off, which in itself makes no sense*/
	if( abundances.lgElmtOn[1] )
	{
		for( i=Heavy.ipHeavy[0][0]-1; i < Heavy.ipHeavy[1][1]; i++ )
		{
			if( rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*opac.opac[i] > xMax )
			{
				xMax = rfield.anu[i]*rfield.flux[i]/rfield.widflx[i]*
				  opac.opac[i];
				*grfreqm = rfield.anu[i];
				*gropacm = opac.opac[i];
			}
		}
	}

	/* possible that there is NO ionizing radiation, in extreme cases,
	 * if so then xMax will still be negative */
	if( xMax <= 0. )
	{
		*gropacm = 0.;
		*grfreqm = 0.;
	}

#	ifdef DEBUG_FUN
	fputs( " <->GrainRateDr()\n", debug_fp );
#	endif
	return;
}

/*ChkRate called by nextdr to check how rates of destruction of various species changes */
static void ChkRate(
	  /* element number on physical scale */
	  long int nelem, 
	  /* change in destruction rate */
	  double *dDestRate, 
	  /* old and new destruction rates */
	  double *DestRateOld,
	  double *DestRateNew,
	  /* stage of ionization on the physical scale */
	  long int *istage)
{
	long int i;

	double average, 
	  dDest;

	static double OldDest[LIMELM][LIMELM];

#	ifdef DEBUG_FUN
	fputs( "<+>ChkRate()\n", debug_fp );
#	endif

	/* return if this element is not turned on */
	if( !abundances.lgElmtOn[nelem-1] )
	{
		*dDestRate = 1e-3;
		
#		ifdef DEBUG_FUN
		fputs( " <->ChkRate()\n", debug_fp );
#		endif
		return;
	}

	/* for first zone, and during search, we will do nothing but
	 * still must return finite numbers */
	*istage = 1;
	*dDestRate = 0.;
	*DestRateOld = 0.;
	*DestRateNew = 0.;
	*dDestRate = 0.;

	if( nzone <= 1 )
	{
		for( i=0; i < nelem; i++ )
		{
			OldDest[i][nelem-1] = DestCrt.xIonize[i][nelem-1];
		}
	}

	else if( xIonFracs[1][nelem-1]/xIonFracs[0][nelem-1] <  0.9 )
	{
		/* do not use this method if everything is atomic */
		for( i=0; i < (nelem - 1); i++ )
		{
			/* last check below, .5 chosen so that do not key off
			 * predominantly neutral species where self-opacity
			 * could cause oscillations */
			if( ((xIonFracs[i+1][nelem-1]/xIonFracs[0][nelem-1] > 0.01 && 
				xIonFracs[i+1][nelem-1]/xIonFracs[0][nelem-1] < 0.9) && 
				xIonFracs[i+2][nelem-1]/xIonFracs[0][nelem-1] > .05) && 
				OldDest[i][nelem-1] > 0. )
			{
				/* last check on old dest in case just lowered ionization
				 * stage, so no history */
				/* check that rate is positive */
				if( DestCrt.xIonize[i][nelem-1] <= 0. )
				{
					fprintf( ioQQQ, " ChkRate gets insane destruction rate for ion%4ld%4ld%10.2e\n", 
					  nelem, i, DestCrt.xIonize[i][nelem-1] );
					puts( "[Stop in chkrate]" );
					cdEXIT(1);
				}

				/* do not consider unless of middling ionization, and
				 * rate is going down (to prevent dr osciallating)
				 * no absolute value in following since do not want to
				 * consider cases where ionization rate increases */
				average = (OldDest[i][nelem-1] + DestCrt.xIonize[i][nelem-1])* 0.5;

				dDest = (OldDest[i][nelem-1] - DestCrt.xIonize[i][nelem-1])/ average;
				/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ term + if rate going down */

				if( *dDestRate < dDest )
				{
					/* biggest rate so far, remember change in rates and ionization stage */
					*dDestRate = dDest;
					*istage = i+1;
					*DestRateOld = OldDest[i][nelem-1];
					*DestRateNew = DestCrt.xIonize[i][nelem-1];
				}

			}
			OldDest[i][nelem-1] = DestCrt.xIonize[i][nelem-1];
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->ChkRate()\n", debug_fp );
#	endif
	return;
}
