/*OpacityCreate1Element generate ionic subshell opacities by calling phfit */
#include "cddefines.h"
#include "physconst.h"
#include "rfield.h"
#include "ph1com.h"
#include "punpoint.h"
#include "nsshells.h"
#include "kshllenr.h"
#include "phfiton.h"
#include "phfit.h"
#include "opacity.h"

void OpacityCreate1Element(
		  /* atomic number on the C scale, lowest ever called will be Li=2 */
		  long int nelem)
{
	long int ihi, 
	  ip, 
	  ipop, 
	  limit, 
	  low, 
	  need, 
	  nelec, 
	  nion, 
	  nshell;
	double cs; 
	double energy;

#	ifdef DEBUG_FUN
	fputs( "<+>OpacityCreate1Element()\n", debug_fp );
#	endif

	/* confirm range of validity of atomic number, Li=2 should be the lightest */
	assert( nelem >= 2 );
	assert( nelem < LIMELM );

	/*>>chng 99 jan 27, no longer redo hydrogenic opacity here */
	/* for( nion=0; nion <= nelem; nion++ )*/
	for( nion=0; nion < nelem; nion++ )
	{

		/* will be used for a sanity check on number of hits in a cell*/
		for( ip=0; ip < NCELL; ip++ )
		{
			opac.opac[ip] = 0.;
		}

		/* number of bound electrons */
		nelec = nelem+1 - nion;

		for( nshell=0; nshell < nsShellsCom.nsShells[nion][nelem]; nshell++ )
		{
			opac.ipElement[2][nshell][nion][nelem] = opac.nOpacTot +  1;

			/* this is continuum pointer upper limit for energy range of this shell */
			limit = opac.ipElement[1][nshell][nion][nelem];

			/* this is number of cells in continuum needed to store opacity */
			need = limit - opac.ipElement[0][nshell][nion][nelem] + 1;

			/* check that opac will have enough frequeny cells */
			if( opac.nOpacTot + need > NOPSV )
			{
				fprintf( ioQQQ, " OpacityCreate1Element needs more opacity cells than NOPSV,  increase it.\n" );
				fprintf( ioQQQ, " element, Ion, are%3ld%3ld\n", 
				  nelem, nion );
				puts( "[Stop in makeopacity]" );
				cdEXIT(1);
			}

			/* set lower and upper limits to this range */
			low = opac.ipElement[0][nshell][nion][nelem];
			ihi = opac.ipElement[1][nshell][nion][nelem];
			ipop = opac.ipElement[2][nshell][nion][nelem];

			/* make sure indices are within correct bounds,
			 * mainly check on logic for detecting mission shells */
			assert( low <= ihi || low<5 );

			for( ip=low-1; ip < ihi; ip++ )
			{
				/* >>chng 96 oct 08 add min to ip, so never eval below threshold */
				energy = MAX2(rfield.AnuOrg[ip]*EVRYD , 
					PH1COM.PH1[nshell][nelec-1][nelem][0]);

				cs = phfit(nelem+1,nelec,nshell+1,energy,PhFitOn.lgPhFit);
				/* cannot assert that cs is positive since, at edge of shell,
				 * energy might be slightly below threshold and hence zero,
				 * due to finite size of continuum bins */
				opac.OpacStack[ip-low+ipop] = cs*1e-18;

				/* add this to total opacity, which we will confirm to be greater than zero below */
				opac.opac[ip] += cs;
			}

			opac.nOpacTot += ihi - low + 1;

			/* punch pointers option */
			if( PunPoint.lgPunPoint )
			{
				fprintf( PunPoint.ipPoint, "%3ld%3ld%3ld%10.2e%10.2e%10.2e%10.2e\n", 
				  nelem, nion, nshell, rfield.anu[low-1], rfield.anu[ihi-1], 
				  opac.OpacStack[ipop-1], opac.OpacStack[ihi-low+ipop-1] );
			}
		}

		/*confirm that total opacity is greater than zero  */
		for( 
			ip=opac.ipElement[0][nsShellsCom.nsShells[nion][nelem]-1][nion][nelem]-1;
		    ip < KshllEnr.KshellLimit; ip++ )
		{
			assert( opac.opac[ip] > 0. );
		}
		/*end sanity check */

	}

#	ifdef DEBUG_FUN
	fputs( " <->OpacityCreate1Element()\n", debug_fp );
#	endif
	return;
}

