/*OpacityCreateAll compute initial set of opacities for all species */
/******************************************************************************
 *NB NB NB  NB NB NB NB NB NB NB  NB NB NB NB
 * everything set here must be written to the opacity store files
 *
 ****************************************************************************** */
/*lint -e713 loss of precission unsigned int to long */
/*lint -e737 loss of sign in promotion */
#include "cddefines.h"
#include "taulines.h"
#define	NCSH2P	10
#include "physconst.h"
#include "nhe1lvl.h"
#include "abundances.h"
#include "i3727.h"
#include "ph1com.h"
#include "trace.h"
#include "hydrogenic.h"
#include "rfield.h"
#include "path.h"
#include "he.h"
#include "nhe1.h"
#include "iphmin.h"
#include "iphe1l.h"
#include "stopcalc.h"
#include "phfiton.h"
#include "ipoint.h"
#include "grain.h"
#include "phfit.h"
#include "hypho.h"
#include "hmiopc.h"
#include "hpfit.h"
#include "rayleh.h"
#include "opacity.h"
/* number of char in magic number */
#define NMAGIC 7
/* the magic number itself, the current date (YYMMDD) */
#define MAGIC "010118"


/*OpacityCreateReilMan generate photoionization cross sections from Reilman and Manson points */
static void OpacityCreateReilMan(long int low, 
  long int ihi, 
  float cross[], 
  long int ncross, 
  long int *ipop, 
  char *chLabl);

/*OpacityCreatePowerLaw generate array of cross sections using a simple power law fit */
static void OpacityCreatePowerLaw(
	/* lower energy limit on continuum mesh */
	long int ilo, 
	/* upper energy limit on continuum mesh */
	long int ihi, 
	/* threshold cross section */
	double cross, 
	/* power law index */
	double s, 
	/* pointer to opacity offset where this starts */
	long int *ip);

/*ofit compute cross sections for all shells of atomic oxygen */
double ofit(double e, 
	  float opart[]);

void OpacityCreateAll(void)
{
	long int i, 
	  ipZ, 
	  n ,
	  nelem;

	float opart[7];

	double crs, 
	  dx,
	  eps, 
	  thom, 
	  thres, 
	  x, 
	  z;

	/* this will be the file name we will try to open for the opacities */
	char chFileName[81];

	FILE *ioOPAC; /* will be pointer to opacity.opc data file */

	/* remember whether opacities have ever been evaluated in this coreload */
	static int lgOpEval = FALSE;

	/* fits to cross section for photo dist of H_2^+ */
	static float csh2p[NCSH2P]={6.75f,0.24f,8.68f,2.5f,10.54f,7.1f,12.46f,
	  6.0f,14.28f,2.7f};

#	ifdef DEBUG_FUN
	fputs( "<+>OpacityCreateAll()\n", debug_fp );
#	endif

	/* H2+ h2plus h2+ */

	/* this is option to stop at certain optical depth */
	if( StopCalc.taunu > 0. )
	{
		StopCalc.iptnu = ipoint(StopCalc.taunu);
		StopCalc.iptnu = MIN2(StopCalc.iptnu,NCELL-1);
	}
	else
	{
		StopCalc.iptnu = NCELL;
		StopCalc.tauend = 1e30f;
	}

	/* make and print dust opacities
	 * fill in dstab and dstsc, totals, zero if no dust
	 * may be different if different grains are turned on */
	InitGrains();

	/* flag lgOpEval says whether opacity stack has been generated
	 * only do this one time per core load  */
	if( lgOpEval )
	{
		/* this is not the first time code called */
		if( trace.lgTrace )
		{
			fprintf( ioQQQ, " OPAC0 called but NOT evaluated.\n" );
		}
		
#		ifdef DEBUG_FUN
		fputs( " <->OpacityCreateAll()\n", debug_fp );
#		endif
		return;
	}

	lgOpEval = TRUE;

	/* should we even try to bring in opacity file? */
	if( opac.lgUseFileOpac )
	{
		/* option to compile opacities into file for later use */
		/* first try local directory */
		strcpy( chFileName , "opacity.opc" );
		ioOPAC = fopen( chFileName , "rb" );
		if( ioOPAC == NULL )
		{
			/* did not get it here, lets try the path if it was set */
			if( lgDataPathSet )
			{
				/* first stuff the path name itself */
				strcpy( chFileName , chDataPath );
				strcat( chFileName , "opacity.opc" );
				/* now try to open this file */
				ioOPAC = fopen( chFileName , "rb" );
				if( ioOPAC == NULL )
				{
					/* still not here, generate new opacities */
					opac.lgOpacExist = FALSE;
				}
				else
				{
					/* found it, will use this file */
					opac.lgOpacExist = TRUE;
				}
			}
			else
			{
				opac.lgOpacExist = FALSE;
			}
		}
		else
		{
			opac.lgOpacExist = TRUE;
		}

		/*	opac.lgCompileOpac is set TRUE with compile opacities command */
		if( !opac.lgCompileOpac && opac.lgOpacExist && (ioOPAC!=NULL) )
		{
			char chMagic[NMAGIC];
			/* opacity file exists and have not been asked to compile it */

			/* first get the magic number */
			n = (long)fread( chMagic , NMAGIC , sizeof(char), ioOPAC );
			if( strcmp( chMagic , MAGIC ) != 0 )
			{
				fprintf( ioQQQ," The starting magic number in the compiled opacity file is incorrect.\n");
				fprintf( ioQQQ," Please recompile opacities, or delete opacity.opc in the data directory.\n");
				fprintf( ioQQQ," The magic number was %s\n", MAGIC );
				puts( "[Stop in OpacityCreateAll]" );
				cdEXIT(1);
			}

			/* next get the big stack of opacity data */
			n = (long)fread( opac.OpacStack , 1, sizeof(opac.OpacStack), ioOPAC );
			if( ((unsigned)n-sizeof(opac.OpacStack)) != 0 )
			{
				fprintf( ioQQQ, " problem trying to read opacity.opc\n" );
				fprintf( ioQQQ, " I expected to read %li words, but fread returned only %li\n",
					(long)sizeof(opac.OpacStack),n);
				fprintf( ioQQQ, " Try recompiling the opacities with the COMPILE OPACITY command,\n" );
				fprintf( ioQQQ, " or tell the code not to use the compiled opacities with the NO FILE OPACITY command.\n" );

				/* did we hit end of file? */
				if( feof(ioOPAC ) )
				{
					fprintf( ioQQQ, " end of file hit\n" );
				}
				/* some other error condition? */
				if( ferror(ioOPAC ) )
				{
					fprintf( ioQQQ, " end of file hit\n" );
				}
				puts( "[Stop in OpacityCreateAll]" );
				cdEXIT(1);
			}

			/* NG following must exactly mirror contents of common opac */
			/* we will add unwritten bytes to this, then check if still zero at end */
			n = 0;
			/* abs cannot be used here since it takes an int, not a long */
			n -= (long)(fread( opac.ipHOpac,	1,sizeof(opac.ipHOpac),
				ioOPAC ) - sizeof(opac.ipHOpac));

			n -= (long)(fread( &opac.ipRayScat,1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( &opac.iopcom,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( &opac.ippr,		1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( &opac.ioppr,		1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( &opac.ipBrems,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( &opac.iphmra,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( &opac.iphmop,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( opac.ih2pnt,		1,sizeof(opac.ih2pnt),
				ioOPAC ) - sizeof(opac.ih2pnt));

			n -= (long)(fread( &opac.ih2pof,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( opac.iophe1,		1,sizeof(opac.iophe1),
				ioOPAC ) - sizeof(opac.iophe1));

			n -= (long)(fread( &opac.ioptri,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( opac.ipElement,	1,sizeof(opac.ipElement),
				ioOPAC ) - sizeof(opac.ipElement));

			n -= (long)(fread( opac.in1,			1,sizeof(opac.in1),
				ioOPAC ) - sizeof(opac.in1));

			n -= (long)(fread( opac.ipo3exc,	1,sizeof(opac.ipo3exc),
				ioOPAC ) - sizeof(opac.ipo3exc));

			n -= (long)(fread( opac.ipo3exc3,	1,sizeof(opac.ipo3exc3),
				ioOPAC ) - sizeof(opac.ipo3exc3));

			n -= (long)(fread( opac.ipo1exc,	1,sizeof(opac.ipo1exc),
				ioOPAC ) - sizeof(opac.ipo1exc));

			n -= (long)(fread( &opac.iopo2d,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( &opac.ipmgex,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( &opac.ipOpMgEx,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			n -= (long)(fread( opac.ica2ex,		1,sizeof(opac.ica2ex),
				ioOPAC ) - sizeof(opac.ica2ex));

			n -= (long)(fread( &opac.ica2op,	1,sizeof(long) ,
				ioOPAC ) - sizeof(long));

			/* following contains AnuOrg and will be checked against it as a sanity check*/
			n -= (long)(fread( opac.tmn,		1,sizeof(opac.tmn),
				ioOPAC ) - sizeof(opac.tmn));

			/* >>chng 01 jan 18, remove two photon from stored opacity file since will create on the fly */
#			if 0
			n -= (long)(fread( em2nu.Hy2nu,			1,sizeof(em2nu.Hy2nu),
				ioOPAC ) - sizeof(em2nu.Hy2nu));

			n -= (long)(fread( em2nu.he12nu,			1,sizeof(em2nu.he12nu),
				ioOPAC ) - sizeof(em2nu.he12nu));

			n -= (long)(fread( em2nu.he22nu ,			1,sizeof(em2nu.he22nu),
				ioOPAC ) - sizeof(em2nu.he22nu));
#			endif

			/* now check whether n is still zero, problem if not */
			if( n!= 0 )
			{
				fprintf( ioQQQ, " problem trying to read opacity.opc pointers\n" );
				fprintf( ioQQQ, " fread was short by %li\n",
					n);
				fprintf( ioQQQ, " Try recompiling the opacities with the COMPILE OPACITY command,\n" );
				fprintf( ioQQQ, " or tell the code not to use the compiled opacities with the NO FILE OPACITY command.\n" );
				puts( "[Stop in OpacityCreateAll]" );
				cdEXIT(1);
			}

			if( opac.tmn[rfield.nupper-1] != rfield.AnuOrg[rfield.nupper-1] )
			{
				fprintf( ioQQQ, " Energy grid in opacity file opacity.opc opacity.pnt do not agree with this version of the code - please recompile.\n" );
				fprintf( ioQQQ, " Values are:%12.4e%12.4e\n", 
				  rfield.AnuOrg[rfield.nupper-1], opac.tmn[rfield.nupper-1] );
				puts( "[Stop in OpacityCreateAll]" );
				cdEXIT(1);
			}

			if( trace.lgTrace )
			{
				fprintf( ioQQQ, " OPAC0 called, values from file =%s=\n",chFileName );
			}

			/* note that opacities now exist */
			opac.lgOpacExist = TRUE;

			/* zero out opac since this array sometimes addressed before OpacityAddTotal called */
			for( i=0; i < NCELL; i++ )
			{
				opac.opac[i] = 0.;
			}
			
#			ifdef DEBUG_FUN
			fputs( " <->OpacityCreateAll()\n", debug_fp );
#			endif
			return;
		}
		else
		{
			opac.lgOpacExist = FALSE;
		}
	}
	else 
	{
		/* this branch if NO FILE OPACITY command entered */
		opac.lgOpacExist = FALSE;
	}

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, " OPAC0 called, evaluating.\n" );
	}

	/* zero out opac since this array sometimes addressed before OpacityAddTotal called */
	for( i=0; i < NCELL; i++ )
	{
		opac.opac[i] = 0.;
	}

	/* nOpacTot is number of opacity cells in OpacStack filled so far by opacity generting routines */
	opac.nOpacTot = 0;

	/* hydrogenic photoionization  */
	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		if( abundances.lgElmtOn[ipZ] )
		{
			/* this is special pointer to the opacity offset for hydrogenic species only */
			opac.ipHOpac[ipZ][ipH1s] = opac.nOpacTot + 1;

			/* this is the same number, but in the general purpose pointer array */
			opac.ipElement[2][0][ipZ][ipZ] = opac.ipHOpac[ipZ][ipH1s];

			/* the lyman continuum, which extends to the high energy limit of the code */
			for( i=hydro.ipHLevNIonCon[ipZ][ipH1s]-1; i < rfield.nupper; i++ )
			{
				thres = MAX2(rfield.AnuOrg[i]*EVRYD , PH1COM.PH1[0][0][ipZ][0]);

				crs = phfit(ipZ+1,1,1,thres,PhFitOn.lgPhFit);

				opac.OpacStack[i-hydro.ipHLevNIonCon[ipZ][ipH1s]+opac.ipHOpac[ipZ][ipH1s]] = 
					crs* 1e-18f;
			}
			opac.nOpacTot += rfield.nupper - hydro.ipHLevNIonCon[ipZ][ipH1s] + 1;

			/* Balmer, Paschen, etc  continua, hbf only goes up to 10, 
			 * and NHPLPHOT is defined to 10 in opacity.h, 
			 * hpfit resolves 2s and 2p */
			for( n=ipH2s; n < MIN2( nHLevels[ipZ], NHPLPHOT); n++ )
			{
				double energy;
				opac.ipHOpac[ipZ][n] = opac.nOpacTot + 1;
				/* first make sure that first energy point is at least near the limit */
				assert( rfield.AnuOrg[hydro.ipHLevNIonCon[ipZ][n]-1] > 0.98f * hydro.HLevNIonRyd[ipZ][n] );

				for( i=hydro.ipHLevNIonCon[ipZ][n]-1; i < hydro.ipHLevNIonCon[ipZ][ipH1s]; i++ )
				{
					/* for first cell, depending on the curent resolution of the energy mesh,
					 * the center of the first cell can be below the ionization limit of the
					 * level.  do not let the energy fall below this limit */
					energy = MAX2( hydro.HLevNIonRyd[ipZ][n]*1.001 , rfield.AnuOrg[i] );

					opac.OpacStack[i-hydro.ipHLevNIonCon[ipZ][n]+opac.ipHOpac[ipZ][n]] = 
						(float)hpfit(ipZ+1,n,energy*EVRYD);
						/*(float)hpfit(ipZ+1,n,rfield.AnuOrg[i]*EVRYD);*/

					/* make sure it is positive */
					assert( opac.OpacStack[i-hydro.ipHLevNIonCon[ipZ][n]+opac.ipHOpac[ipZ][n]] > 0.);
				}

				/* this checks that the photo cross sections we just generated agree
				 * with newly computed exact results, at threshold */
#				if !defined(NDEBUG)
				{
					/*@-redef@*/
					enum {DEBUG=TRUE};
					/*@+redef@*/
					if( DEBUG && ipZ==0 && n>ipH2p )
					{
						float anu[1]={1.} , cs[1] ;
						double error ;

						/* photon energy where cross section will be evaluated,
						 * this is in Ryd */
						energy = hydro.HLevNIonRyd[ipZ][n];
						anu[0] = (float)(energy*1.01);

						hypho(
							/* Z-Nelec, the residual charge, 1 for hydrogen, 2 for helium */
							(double)(ipZ+1), 
							/* principle quantum number */
						  n, 
						  /* lowest angular momentum */
						  0, 
						  /* highest angular momentum */
						  n-1, 
						  /* scaled lowest photon energy, 
						   * => incorrect?? in units of zed^2/n^2, 
						   * at which the cs will be done */
						  energy, 
						  /* number of points to do */
						  1, 
						  /* array of energies (in units given above) */
						  anu , 
						  /* calculated photoionization cross section in cm^-2 */
						  cs);

						error = fabs(cs[0] - opac.OpacStack[opac.ipHOpac[ipZ][n]-1] )/
							(        (cs[0] + opac.OpacStack[opac.ipHOpac[ipZ][n]-1] )/2.);
						/*fprintf(ioQQQ,"z=%ld n=%ld error %g old %e new %e\n",ipZ,n, error,
							opac.OpacStack[opac.ipHOpac[ipZ][n]-1] ,cs[0] );*/
						assert( error < 0.05 );
					}
				}
#				endif

				/* possible that center of first cell is below IP of level,
				 * even though high end of level has edge - hpfit returned
				 * zero in this case - update to next higher cell */

				if( opac.OpacStack[opac.ipHOpac[ipZ][n]-1] == 0. )
					opac.OpacStack[opac.ipHOpac[ipZ][n]-1] = 
					opac.OpacStack[opac.ipHOpac[ipZ][n]];

				opac.nOpacTot += hydro.ipHLevNIonCon[ipZ][ipH1s] - hydro.ipHLevNIonCon[ipZ][n] + 1;

				if( opac.nOpacTot > NOPSV - NCELL )
				{
					fprintf( ioQQQ, " hydrogenic opacities need NOPSV larger.\n" );
					fprintf( ioQQQ, " ipZ, n, NOPSV=%3ld%3ld%9ld\n", 
					  ipZ, n, NOPSV );
					puts( "[Stop in OpacityCreateAll]" );
					cdEXIT(1);
				}
			}
		}
	}

	/* Lyman alpha damping wings - Rayleigh scattering */
	opac.ipRayScat = opac.nOpacTot + 1;
	for( i=0; i < hydro.ipHLevNIonCon[0][ipH1s]; i++ )
	{
		opac.OpacStack[i-1+opac.ipRayScat] = rayleh(rfield.AnuOrg[i]);
	}
	opac.nOpacTot += hydro.ipHLevNIonCon[0][ipH1s];

	/* assume Thomson scattering up to ipCKshell, 20.6 Ryd=0.3 keV */
	thom = 6.65e-25;
	opac.iopcom = opac.nOpacTot + 1;
	for( i=0; i < opac.ipCKshell; i++ )
	{
		opac.OpacStack[i-1+opac.iopcom] = thom;
	}

	/* Klein-Nishina from eqn 7.5, Rybicki and Lightman */
	for( i=opac.ipCKshell; i < rfield.nupper; i++ )
	{
		dx = rfield.AnuOrg[i]/3.7573e4;

		opac.OpacStack[i-1+opac.iopcom] = thom*3.e0/4.e0*((1.e0 + 
		  dx)/POW3(dx)*(2.e0*dx*(1.e0 + dx)/(1.e0 + 2.e0*dx) - log(1.e0+
		  2.e0*dx)) + 1.e0/2.e0/dx*log(1.e0+2.e0*dx) - (1.e0 + 3.e0*
		  dx)/POW3(1.e0 + 2.e0*dx));
	}
	opac.nOpacTot += rfield.nupper - 1 + 1;

	/* pair production */
	opac.ioppr = opac.nOpacTot + 1;
	for( i=opac.ippr-1; i < rfield.nupper; i++ )
	{
		/* pair production heating rate for unscreened H + He
		 * fit to figure 41 of Experimental Nuclear Physics,
		 * Vol1, E.Segre, ed */

		x = rfield.AnuOrg[i]/7.512e4*2.;

		opac.OpacStack[i-opac.ippr+opac.ioppr] = 5.793e-28*
		  POW2((-0.46737118 + x*(0.349255416 + x*0.002179893))/(1. + 
		  x*(0.130471301 + x*0.000524906)));
	}
	opac.nOpacTot += rfield.nupper - opac.ippr + 1;

	/* brems (free-free) opacity */
	opac.ipBrems = opac.nOpacTot + 1;

	for( i=0; i < rfield.nupper; i++ )
	{
		/* missing factor of 1E-20 to avoid underflow
		 * free free opacity needs g(ff)*(1-exp(hn/kT))/SQRT(T)*1E-20 */
		opac.OpacStack[i-1+opac.ipBrems] = 
			/*(float)(1.03680e-18/POW3(rfield.AnuOrg[i]));*/
			/* >>chng 00 jun 05, descale by 1e10 so that underflow at high-energy
			 * end does not occur */
			1.03680e-8/POW3(rfield.AnuOrg[i]);
	}
	opac.nOpacTot += rfield.nupper - 1 + 1;

	opac.iphmra = opac.nOpacTot + 1;
	for( i=0; i < rfield.nupper; i++ )
	{
		/* following is ratio of h minus to neut h opacity */
		opac.OpacStack[i-1+opac.iphmra] = 0.1175*rfield.anusqr[i];
	}
	opac.nOpacTot += rfield.nupper - 1 + 1;

	opac.iphmop = opac.nOpacTot + 1;
	for( i=iphminCom.iphmin-1; i < nhe1Com.nhe1[0]; i++ )
	{
		/* H- hminus H minus bound-free opacity */
		opac.OpacStack[i-iphminCom.iphmin+opac.iphmop] = 
			hmiopc(rfield.AnuOrg[i]);
	}
	opac.nOpacTot += nhe1Com.nhe1[0] - iphminCom.iphmin + 1;

	/* H2+ H2P h2plus photoabsorption
	 * cs from Buckingham Reid and Spence, MN 112m 382, 0 K temp */
	OpacityCreateReilMan(opac.ih2pnt[0],opac.ih2pnt[1],csh2p,NCSH2P,&opac.ih2pof,
	  "H2+ ");

	/* Helium, triplet 23S from Steward 1979, quoted in Mathisen compendium */
	OpacityCreatePowerLaw(he.nhei3,nhe1Com.nhe1[0],5.5e-18,1.35,&opac.ioptri);

	/* HeI singlets neutral helium ground */
	opac.iophe1[0] = opac.nOpacTot + 1;
	opac.ipElement[2][0][0][1] = opac.iophe1[0];
	for( i=nhe1Com.nhe1[0]-1; i < rfield.nupper; i++ )
	{
		crs = phfit(2,2,1,rfield.AnuOrg[i]*EVRYD,PhFitOn.lgPhFit);
		opac.OpacStack[i-nhe1Com.nhe1[0]+opac.iophe1[0]] = 
			crs*1e-18;
	}
	opac.nOpacTot += rfield.nupper - nhe1Com.nhe1[0] + 1;

	/* HeI neutral Helium 21s from Stewart JPhysB 11, L431 */
	OpacityCreatePowerLaw(nhe1Com.nhe1[1],nhe1Com.nhe1[0],0.4*8.7e-18,1.5,&opac.iophe1[1]);

	/* HeI helium singlet bound free excited states */
	z = 1.;
	for( n=3; n <= NHE1LVL; n++ )
	{
		crs = 7.906e-18*(float)(n)/(z*z);
		OpacityCreatePowerLaw(nhe1Com.nhe1[n-1],nhe1Com.nhe1[0],crs,3.0,&opac.iophe1[n-1]);
	}

	/* these are opacity offset points that would be defined in OpacityCreate1Element,
	 * but this routine will not be called for H and He
	 * generate all heavy element opacities, everything heavier than He,
	 * nelem is on the C scale, so Li is 2 */
	/*>>chng 99 jan 27, do not reevaluate hydrogenic opacity below */
	for( nelem=2; nelem < LIMELM; nelem++ )
	{
		if( abundances.lgElmtOn[nelem] )
		{
			OpacityCreate1Element(nelem);
		}
	}

	/* now add on some special cases, where exicted states, etc, come in */

	/* Nitrogen
	 * >>refer Henry, R., ApJ 161, 1153.
	 * photoionization of excited level of N+ */
	OpacityCreatePowerLaw(opac.in1[0],opac.in1[1],9e-18,1.75,&opac.in1[2]);

	/* atomic Oxygen
	 * only do this if 1996 Verner results are used */
	if( abundances.lgElmtOn[7] && !PhFitOn.lgPhFit )
	{
		/* integrate over energy range of the valence shell of atomic oxygen*/
		for( i=opac.ipElement[0][2][0][7]-1; i < opac.ipElement[1][2][0][7]; i++ )
		{
			/* call special routine to evaluate partial cross section for OI shells */
			eps = rfield.AnuOrg[i]*EVRYD;
			crs = ofit(eps,opart);

			/* this will be total cs of all processes leaving shell 3 */
			crs = opart[0];
			for( n=1; n < 6; n++ )
			{
				/* add up table of cross sections */
				crs += opart[n];
			}
			/* convert to cgs and overwrite cross sections set by OpacityCreate1Element */
			crs *= 1e-18;
			opac.OpacStack[i-opac.ipElement[0][2][0][7]+opac.ipElement[2][2][0][7]] = crs;
		}
	}

	/* Henry nubmers for 1S excit state of OI, OP data very sparse */
	OpacityCreatePowerLaw(opac.ipo1exc[0],opac.ipo1exc[1],4.64e-18,0.,&opac.ipo1exc[2]);

	/* photoionization of excited level of O2+ 1D making 5007
	 * fit to TopBase Opacity Project cs */
	OpacityCreatePowerLaw(opac.ipo3exc[0],opac.ipo3exc[1],3.8e-18,0.,&opac.ipo3exc[2]);

	/* photoionization of excited level of O2+ 1S making 4363 */
	OpacityCreatePowerLaw(opac.ipo3exc3[0],opac.ipo3exc3[1],5.5e-18,0.01,
	  &opac.ipo3exc3[2]);

	/* photoionization to excited states of O+ */
	opac.iopo2d = opac.nOpacTot + 1;
	thres = rfield.AnuOrg[i3727.i2d-1];
	for( i=i3727.i2d-1; i < hydro.ipHLevNIonCon[1][ipH1s]; i++ )
	{
		crs = 3.85e-18*(4.4*pow(rfield.AnuOrg[i]/thres,-1.5) - 3.38*
		  pow(rfield.AnuOrg[i]/thres,-2.5));

		opac.OpacStack[i-i3727.i2d+opac.iopo2d] = crs;
	}
	opac.nOpacTot += hydro.ipHLevNIonCon[1][ipH1s] - i3727.i2d + 1;

	/* magnesium
	 * photoionization of excited level of Mg+
	 * fit to opacity project data Dima got */
	opac.ipOpMgEx = opac.nOpacTot + 1;
	for( i=opac.ipmgex-1; i < hydro.ipHLevNIonCon[0][ipH1s]; i++ )
	{
		opac.OpacStack[i-opac.ipmgex+opac.ipOpMgEx] = 
			(0.2602325880970085 + 
		  445.8558249365131*exp(-rfield.AnuOrg[i]/0.1009243952792674))*
		  1e-18;
	}
	opac.nOpacTot += hydro.ipHLevNIonCon[0][ipH1s] - opac.ipmgex + 1;

	/* Calcium
	 * excited states of Ca+ */
	OpacityCreatePowerLaw(opac.ica2ex[0],opac.ica2ex[1],4e-18,1.,&opac.ica2op);

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, " OPAC0 return OK, number of opacity cells used in OPSC=%7ld and OPSV is dimensioned%7ld\n", 
		  opac.nOpacTot, NOPSV );
	}

	if( NOPSV - opac.nOpacTot < NCELL )
	{
		fprintf( ioQQQ, " Warning, only%4ld cells left in OPSV.  Increase NOPSV.\n", 
		  NOPSV - opac.nOpacTot );
	}

	/* option to compile opacities into file for later use 
	 * this is executed if the 'compile opacities' command is entered */
	if( opac.lgCompileOpac )
	{
		ioOPAC = fopen("opacity.opc","wb");
		if( ioOPAC == NULL )
		{
			fprintf( ioQQQ, " problem trying to open opacity.opc\n" );
			puts( "[Stop in OpacityCreateAll]" );
			cdEXIT(1);
		}

		/* first get the magic number */
		n = (long)fwrite( MAGIC , NMAGIC , sizeof(char), ioOPAC );

		/* write the data, remembering how many bytes were dumped */
		n = (long)fwrite( opac.OpacStack , 1, sizeof(opac.OpacStack), ioOPAC );
		if( (n-sizeof(opac.OpacStack)) != 0 )
		{
			fprintf( ioQQQ, " problem trying to write opacity.opc\n" );
			fprintf( ioQQQ, " I expected to write %li words, but fwrite returned only %li\n",
				(long)sizeof(opac.OpacStack),n);
			puts( "[Stop in OpacityCreateAll]" );
			cdEXIT(1);
		}

		/* NG following must exactly mirror contents of common opac */
		/* we will add unwritte bytes to this, then check if still zero at end */
		n = 0;
		/* abs cannot be used here since it takes an int, not a long */
		n -= (long)( fwrite( opac.ipHOpac, 1,sizeof(opac.ipHOpac),
			ioOPAC ) - sizeof(opac.ipHOpac));

		n -= (long)( fwrite( &opac.ipRayScat, 1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( &opac.iopcom, 1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( &opac.ippr, 1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( &opac.ioppr,		1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( &opac.ipBrems,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( &opac.iphmra,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( &opac.iphmop,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( opac.ih2pnt,		1,sizeof(opac.ih2pnt),
			ioOPAC ) - sizeof(opac.ih2pnt));

		n -= (long)( fwrite( &opac.ih2pof,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( opac.iophe1,		1,sizeof(opac.iophe1),
			ioOPAC ) - sizeof(opac.iophe1));

		n -= (long)( fwrite( &opac.ioptri,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( opac.ipElement,	1,sizeof(opac.ipElement),
			ioOPAC ) - sizeof(opac.ipElement));

		n -= (long)( fwrite( opac.in1,			1,sizeof(opac.in1),
			ioOPAC ) - sizeof(opac.in1));

		n -= (long)( fwrite( opac.ipo3exc,	1,sizeof(opac.ipo3exc),
			ioOPAC ) - sizeof(opac.ipo3exc));

		n -= (long)( fwrite( opac.ipo3exc3,	1,sizeof(opac.ipo3exc3),
			ioOPAC ) - sizeof(opac.ipo3exc3));

		n -= (long)( fwrite( opac.ipo1exc,	1,sizeof(opac.ipo1exc),
			ioOPAC ) - sizeof(opac.ipo1exc));

		n -= (long)( fwrite( &opac.iopo2d,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( &opac.ipmgex,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( &opac.ipOpMgEx,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( opac.ica2ex,		1,sizeof(opac.ica2ex),
			ioOPAC ) - sizeof(opac.ica2ex));

		n -= (long)( fwrite( &opac.ica2op,	1,sizeof(long) ,
			ioOPAC ) - sizeof(long));

		n -= (long)( fwrite( rfield.AnuOrg,		1,sizeof(rfield.AnuOrg),
			ioOPAC ) - sizeof(rfield.AnuOrg));

		/* >>chng 01 jan 18, remove two photon from stored opacity file since will create on the fly */
#		if 0
		n -= (long)( fwrite( em2nu.Hy2nu,			1,sizeof(em2nu.Hy2nu),
			ioOPAC ) - sizeof(em2nu.Hy2nu));

		n -= (long)( fwrite( em2nu.he12nu,			1,sizeof(em2nu.he12nu),
			ioOPAC ) - sizeof(em2nu.he12nu));

		n -= (long)( fwrite( em2nu.he22nu ,			1,sizeof(em2nu.he22nu),
			ioOPAC ) - sizeof(em2nu.he22nu));
#		endif

		/* now check whether n is still zero, problem if not */
		if( n!= 0 )
		{
			fprintf( ioQQQ, " problem trying to write opacity.opc pointers\n" );
			fprintf( ioQQQ, " fwrite was short by %li\n",
				n);
			puts( "[Stop in OpacityCreateAll]" );
			cdEXIT(1);
		}

		fclose(ioOPAC);
		fprintf( ioQQQ, "\n\nCompile opacity completed ok  - stopping.\n" );
		fprintf( ioQQQ, "The file opacity.opc was created.\n" );
		fprintf( ioQQQ, "Make sure this file lies somewhere on the path.\n\n\n" );
		puts( "[Stop in OpacityCreateAll]" );
		cdEXIT(1);
	}

#	ifdef DEBUG_FUN
	fputs( " <->OpacityCreateAll()\n", debug_fp );
#	endif
	return;
}
/*OpacityCreatePowerLaw generate array of cross sections using a simple power law fit */
static void OpacityCreatePowerLaw(
	/* lower energy limit on continuum mesh */
	long int ilo, 
	/* upper energy limit on continuum mesh */
	long int ihi, 
	/* threshold cross section */
	double cross, 
	/* power law index */
	double s, 
	/* pointer to opacity offset where this starts */
	long int *ip)
{
	long int i;
	double thres;

#	ifdef DEBUG_FUN
	fputs( "<+>OpacityCreatePowerLaw()\n", debug_fp );
#	endif

	/* non-positive cross section is unphysical */
	assert( cross > 0. );

	/* place in the opacity stack where we will stuff cross sections */
	*ip = opac.nOpacTot + 1;
	assert( *ip > 0 );
	assert( ilo > 0 );
	thres = rfield.anu[ilo-1];

	for( i=ilo-1; i < ihi; i++ )
	{
		opac.OpacStack[i-ilo+*ip] = cross*pow(rfield.anu[i]/thres,-s);
	}

	opac.nOpacTot += ihi - ilo + 1;

	if( opac.nOpacTot > NOPSV )
	{
		fprintf( ioQQQ, " OpacityCreatePowerLaw too many opacity points.\n" );
		fprintf( ioQQQ, " Increase parameter variable NOPSV everywhere it occurs in the code.\n" );
		fprintf( ioQQQ, " The current value is%6ld.\n", NOPSV );
		puts( "[Stop in OpacityCreatePowerLaw]" );
		cdEXIT(1);
	}


#	ifdef DEBUG_FUN
	fputs( " <->OpacityCreatePowerLaw()\n", debug_fp );
#	endif
	return;
}

/*OpacityCreateReilMan generate photoionization cross sections from Reilman and Manson points */
static void OpacityCreateReilMan(long int low, 
  long int ihi, 
  float cross[], 
  long int ncross, 
  long int *ipop, 
  char *chLabl)
{
	long int i, 
	  ics, 
	  j, 
	  ncr;

#define	NOP	100
	float cs[NOP], 
	  en[NOP], 
	  slope;

#	ifdef DEBUG_FUN
	fputs( "<+>OpacityCreateReilMan()\n", debug_fp );
#	endif

	/* this is the opacity entering routine designed for
	 * the Reilman and Manson tables.  It works with incident
	 * photon energy (entered in eV) and cross sections in megabarns
	 * */
	*ipop = opac.nOpacTot + 1;
	assert( *ipop > 0 );

	ncr = ncross/2;
	if( ncr > NOP )
	{
		fprintf( ioQQQ, " Too many opacities were entered into OpacityCreateReilMan.  Increase the value of NOP.\n" );
		fprintf( ioQQQ, " chLabl was %4.4s\n", chLabl );
		puts( "[Stop in OpacityCreateReilMan]" );
		cdEXIT(1);
	}

	/* the array CROSS has ordered pairs of elements.
	 * the first is the energy in eV (not Ryd)
	 * and the second is the cross section in megabarns */
	for( i=0; i < ncr; i++ )
	{
		en[i] = (float)(cross[i*2]/13.6);
		cs[i] = (float)(cross[(i+1)*2-1]*1e-18);
	}

	assert( low>0 );
	if( en[0] > rfield.anu[low-1] )
	{
		fprintf( ioQQQ, 
			" OpacityCreateReilMan: The entered opacity energy bandwidth is not large enough (low fail).\n" );
		fprintf( ioQQQ, 
			" The desired energy (Ryd) was%12.5eeV and the lowest entered in the array was%12.5e eV\n", 
		  rfield.anu[low-1]*13.606, en[0]*13.606 );

		fprintf( ioQQQ, " chLabl was %4.4s\n", chLabl );
		fprintf( ioQQQ, " The original energy (eV) and cross section (mb) arrays follow:\n" );
		fprintf( ioQQQ, " " );

		for( i=0; i < ncross; i++ )
		{
			fprintf( ioQQQ, "%11.4e", cross[i] );
		}

		fprintf( ioQQQ, "\n" );
		puts( "[Stop in OpacityCreateReilMan]" );
		cdEXIT(1);
	}

	slope = (cs[1] - cs[0])/(en[1] - en[0]);
	ics = 1;

	/* now fill in the opacities using linear interpolation */
	for( i=low-1; i < ihi; i++ )
	{
		if( rfield.anu[i] > en[ics-1] && rfield.anu[i] <= en[ics] )
		{
			opac.OpacStack[i-low+*ipop] = cs[ics-1] + slope*(rfield.anu[i] - 
			  en[ics-1]);
		}

		else
		{
			ics += 1;
			if( ics + 1 > ncr )
			{
				fprintf( ioQQQ, " OpacityCreateReilMan: The entered opacity energy bandwidth is not large enough (high fail).\n" );
				fprintf( ioQQQ, " The entered energy was %10.2eeV and the highest in the array was %10.2eeV\n", 
				  rfield.anu[i]*13.6, en[ncr-1]*13.6 );
				fprintf( ioQQQ, " chLabl was %4.4s\n", chLabl
				   );
				fprintf( ioQQQ, " The lowest energy enterd in the array was%10.2e eV\n", 
				  en[0]*13.65 );
				fprintf( ioQQQ, " The highest energy ever needed would be%10.2eeV\n", 
				  rfield.anu[ihi-1]*13.6 );
				fprintf( ioQQQ, " The lowest energy needed was%10.2eeV\n", 
				  rfield.anu[low-1]*13.6 );
				puts( "[Stop in OpacityCreateReilMan]" );
				cdEXIT(1);
			}

			slope = (cs[ics] - cs[ics-1])/(en[ics] - en[ics-1]);
			if( rfield.anu[i] > en[ics-1] && rfield.anu[i] <= en[ics] )
			{
				opac.OpacStack[i-low+*ipop] = cs[ics-1] + slope*(rfield.anu[i] - 
				  en[ics-1]);
			}
			else
			{
				assert( i > 0);
				fprintf( ioQQQ, " Internal logical error in OpacityCreateReilMan.\n" );
				fprintf( ioQQQ, " The desired energy (%10.2eeV), I=%5ld, is not within the next energy bound%10.2e%10.2e\n", 
				  rfield.anu[i]*13.6, i, en[ics-1], en[ics] );

				fprintf( ioQQQ, " The previous energy (eV) was%10.2e\n", 
				  rfield.anu[i-1]*13.6 );

				fprintf( ioQQQ, " Here comes the energy array.  ICS=%4ld\n", 
				  ics );

				for( j=0; j < ncr; j++ )
				{
					fprintf( ioQQQ, "%10.2e", en[j] );
				}
				fprintf( ioQQQ, "\n" );

				fprintf( ioQQQ, " chLabl was %4.4s\n", chLabl );
				puts( "[Stop in OpacityCreateReilMan]" );
				cdEXIT(1);
			}
		}
	}

	opac.nOpacTot += ihi - low + 1;
	if( opac.nOpacTot > NOPSV )
	{
		fprintf( ioQQQ, " OpacityCreateReilMan too many opacity points.\n" );
		fprintf( ioQQQ, " Increase parameter variable NOPSV everywhere it occurs in the code.\n" );
		fprintf( ioQQQ, " The current value is%6ld.\n", NOPSV );
		puts( "[Stop in OpacityCreateReilMan]" );
		cdEXIT(1);
	}

#	ifdef DEBUG_FUN
	fputs( " <->OpacityCreateReilMan()\n", debug_fp );
#	endif
	return;
}


/*ofit compute cross sections for all shells of atomic oxygen */
double ofit(double e, 
	  float opart[])
{
	long int i, 
	  _r;

	double otot,
		q, 
	  x;

	static double y[7][5];
	static double eth[7]={13.62,16.94,18.79,28.48,50.,110.5,538.};
	static long l[7]={1,1,1,0,1,1,0};
	static int _aini = 1;

	if( _aini ){ /* Do 1 TIME INITIALIZATIONS! */
		{ static double _itmp0[] = {8.915,3995.,3.242,10.44,0.0};
		for( i=1, _r = 0; i <= 5; i++ )
		{
			y[0][i-1] = _itmp0[_r++];
			}
		}
		{ static double _itmp1[] = {11.31,1498.,5.27,7.319,0.0};
		for( i=1, _r = 0; i <= 5; i++ )
		{
			y[1][i-1] = _itmp1[_r++];
			}
		}
		{ static double _itmp2[] = {10.5,1.059e05,1.263,13.04,0.0};
		for( i=1, _r = 0; i <= 5; i++ )
		{
			y[2][i-1] = _itmp2[_r++];
			}
		}
		{ static double _itmp3[] = {19.49,48.47,8.806,5.983,0.0};
		for( i=1, _r = 0; i <= 5; i++ )
		{
			y[3][i-1] = _itmp3[_r++];
			}
		}
		{ static double _itmp4[] = {50.,4.244e04,0.1913,7.012,4.454e-02};
		for( i=1, _r = 0; i <= 5; i++ )
		{
			y[4][i-1] = _itmp4[_r++];
			}
		}
		{ static double _itmp5[] = {110.5,0.1588,148.3,-3.38,3.589e-02};
		for( i=1, _r = 0; i <= 5; i++ )
		{
			y[5][i-1] = _itmp5[_r++];
			}
		}
		{ static double _itmp6[] = {177.4,32.37,381.2,1.083,0.0};
		for( i=1, _r = 0; i <= 5; i++ )
		{
			y[6][i-1] = _itmp6[_r++];
			}
		}
		_aini = 0;
	}

#	ifdef DEBUG_FUN
	fputs( "<+>ofit()\n", debug_fp );
#	endif
	/*compute cross sections for all shells of atomic oxygen
	 * Photoionization of OI
	 * Input parameter:   e - photon energy, eV
	 * Output parameters: otot - total photoionization cross section, Mb
	 *  opart(1) - 2p-shell photoionization, goes to 4So
	 *  opart(2) - 2p-shell photoionization, goes to 2Do
	 *  opart(3) - 2p-shell photoionization, goes to 2Po
	 *  opart(4) - 2s-shell photoionization
	 *  opart(5) - double photoionization, goes to O++
	 *  opart(6) - triple photoionization, goes to O+++
	 *  opart(7) - 1s-shell photoionization */

	otot = 0.0;

	for( i=0; i < 7; i++ )
	{
		opart[i] = 0.0;
	}

	for( i=0; i < 7; i++ )
	{
		if( e >= eth[i] )
		{
			q = 5.5 - 0.5*y[i][3] + l[i];

			x = e/y[i][0];

			opart[i] = (float)(y[i][1]*(POW2(x - 1.0) + POW2(y[i][4]))/
			  pow(x,q)/pow(1.0 + sqrt(x/y[i][2]),y[i][3]));

			otot += opart[i];
		}
	}


#	ifdef DEBUG_FUN
	fputs( " <->ofit()\n", debug_fp );
#	endif
	return(otot);
}

/*lint +e713 loss of precission unsigned int to long */
/*lint +e737 loss of sign in promotion */
