/*ParseAtomFeII parse the FeII command */
#include "cddefines.h"
#include "pop371.h"
#include "parse.h"

void ParseAtomFeII(char *chCard )
{
	long int i;
	int lgEOL=FALSE;

#	ifdef DEBUG_FUN
	fputs( "<+>ParseAtomFeII()\n", debug_fp );
#	endif

	/* turn on the large verner atom */
	FeII.lgFeIION = TRUE;

	/* levels keyword is to adjust number of levels.  But this only has effect
	 * BEFORE space is allocated for the FeII arrays */
	if( lgMatch("LEVE",chCard) )
	{

		/* do option only if space not yet allocated */
		if( !lgFeIIMalloc )
		{
			i = 5;
			/* number of levels for hydrogen at, 2s is this plus one */
			nFeIILevel = (long int)FFmtRead(chCard,&i,LINELENGTH,&lgEOL);
			if( lgEOL )
			{
				/* hit eol with no number - this is a problem */
				NoNumb(chCard);
			}
			if( nFeIILevel <16 )
			{
				fprintf( ioQQQ, " This would be too few levels, must have at least 16.\n" );
				puts( "[Stop in ParseAtomFeII]" );
				cdEXIT(1);
			}
			else if( nFeIILevel > NFE2LEVN )
			{
				fprintf( ioQQQ, " This would be too many levels.\n" );
				puts( "[Stop in ParseAtomFeII]" );
				cdEXIT(1);
			}
		}
	}

	/* slow keyword means do not try to avoid evaluating atom */
	if( lgMatch("SLOW",chCard) )
	{
		FeII.lgSlow = TRUE;
	}

	/* print keyword print comment for each call to pop371 */
	if( lgMatch("PRIN",chCard) )
	{
		FeII.lgPrint = TRUE;
	}

	/* only simulate the feii atom, do not actually call it */
	if( lgMatch("SIMU",chCard) )
	{
		/* option to only simulate calls to pop371 */
		FeII.lgSimulate = TRUE;
	}

	/* only simulate the feii atom, do not actually call it */
	if( lgMatch("CONT",chCard) )
	{
		/* the continuum output with the punch feii continuum command */
		i=5;

		/* number of levels for hydrogen at, 2s is this plus one */
		FeII.fe2con_wl1 = (float)FFmtRead(chCard,&i,LINELENGTH,&lgEOL);
		FeII.fe2con_wl2 = (float)FFmtRead(chCard,&i,LINELENGTH,&lgEOL);
		FeII.nfe2con = (long int)FFmtRead(chCard,&i,LINELENGTH,&lgEOL);
		if( lgEOL )
		{
			fprintf(ioQQQ," there are three numbers on the FeII continuum command, start and end wavelengths, and number of intervals.\n");
			/* hit eol with no number - this is a problem */
			NoNumb(chCard);
		}
		/* check that all are positive */
		if( FeII.fe2con_wl1<=0. || FeII.fe2con_wl2<=0. || FeII.nfe2con<= 0 )
		{
			fprintf(ioQQQ," there are three numbers on the FeII continuum command, start and end wavelengths, and number of intervals.\n");
			fprintf(ioQQQ," all three must be greater than zero, sorry.\n");
			puts( "[Stop in ParseAtomFeII]" );
			cdEXIT(1);
		}
		/* make sure that wl1 is less than wl2 */
		if( FeII.fe2con_wl1>= FeII.fe2con_wl2 )
		{
			fprintf(ioQQQ," there are three numbers on the FeII continuum command, start and end wavelengths, and number of intervals.\n");
			fprintf(ioQQQ," the second wavelength must be greater than the first, sorry.\n");
			puts( "[Stop in ParseAtomFeII]" );
			cdEXIT(1);
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->ParseAtomFeII()\n", debug_fp );
#	endif
	return;
}

