/* phycon.h */
EXTERN struct t_phycon {

	/*te, electron temperature
	 *NB, this is used in zero to check whether 32 bit environment in use,
	 *if ever declared double will need to change that test, setting lgBit32*/
	float te;

	/* the electron temperature in eV */
	float te_eV;

	/* 1/TE */
	float teinv;

	/* T^2 */
	float tesqrd;

	/* electron density */
	double eden, 

	/* hydrogen density */
	hden;

	/* ratio of electron to nuclei density, could be >1 because of he, set in esum */
	float ElecFrac;

	/*total number of colliders (atoms, ions, molecules) in gas, no electrons */
	double CollidDensity;

	/*xMassDensity grams per cc */
	float xMassDensity, 

	  /* number of particles per cubic centimeter */
	  pden, 

	  /* mean AMU per particle */
	  wmole ,

	  /* total number of nuclei, set in PressureTotal */
		TotalNuclei;


	/*
	 * EdenMet is sum of electrons from Lit through n=30, all ions
	 * EdenNotH is total electron density less the proton density
	 * used in hydrogen loop for charge conservation
	 * EdenHCorr is eden + hi * 1.7e-4, includes correction for H atom collisions */
	float EdenMet, 
	  EdenNotH, 
	  EdenHCorr;

	/* total ionization energy of gas, erg/cm^3, evaluated in PressureTotal */
	double EnergyIonization;

	/* this is relative error in the electron density we want
	 * set in block data to 0.01
	 * reset with set EdenError command */
	float EdenError;

	/* this is the true eden as set in esum, we will try to converge eden to this */
	float EdenTrue;

	/*flags set when bad electron density is detected */
	int lgEdenBad;

	/* flag saying whether to use temperature predictor for next zone, 
	 * in constant density models */
	int lgPredNextTe;

	/*zone where bad electron density was detected */
	long int nzEdenBad;

	/*EdenExtra extra electrons added with eden command*/
	float EdenExtra;

	/*EdenSet electron density set with set eden command*/
	float EdenSet;

	/* sum of heavy element free electrons, set in esum */
	float EdenFFSum;

	/* edensqte is eden/sqrte */
	float edensqte; 

	/* cdsqte is eden/sqrte times 8.629e-6 */
	float cdsqte;

	/* these are simple powers of the electron temperature, are
	 * evaluated in tfidle, and can be used to avoid exponentials*/
	float sqrte, 
	  te32, 
	  te70, 
	  te30, 
	  te20, 
	  te10, 
	  te05, 
	  te03, 
	  te02, 
	  te01, 
	  te005, 
	  te003, 
	  te001;
	/*1 is log Te, 2 is (log T)^2, etc*/
	float telogn[7] ,
		/* this is used to rescale telogn array for other charge,
		 * ith element is log(i+1)^(i+1)*/
		sqlogz[LIMELM];

	/* alogte is base 10 log of temperature */
	float alogte;

	/* alogte is base e log of temperature */
	float alogete;

	/* alogte is base 10 log of density */
	float alogne;

	/* square root of electron density, set in tfidle */
	float SqrtEden;

	/* temperature of previous zone,used for punch */
	float tlast;

	float 
	  /* inital temperature */
	  TeInit, 
	  /* proposed temperature */
	  TeProp,
	  /* inital electron density */
	  EdenInit ,
	  /* proposed electron density */
	  EdenProp ;

	}	phycon;

