/*RadInc do work associated with geometry increments of this zone, called before RTOptDepthIncre */
/*pnegopc punch negative opacities on io unit, iff 'set negopc' command was given */
#include "cddefines.h"
#include "taulines.h"
#include "physconst.h"
#include "cooling.h"
#include "atomcwgt.h"
#include "hydrogenic.h"
#include "rfield.h"
#include "jmin.h"
#include "colden.h"
#include "filfac.h"
#include "opacity.h"
#include "thermal.h"
#include "timesc.h"
#include "csphot.h"
#include "crsnut.h"
#include "uspher.h"
#include "neutrn.h"
#include "ionfracs.h"
#include "hevmolec.h"
#include "hmi.h"
#include "trace.h"
#include "dynamics.h"
#include "sphere.h"
#include "wind.h"
#include "h.h"
#include "phycon.h"
#include "radius.h"
#include "pressure.h"
#include "angllum.h"
#include "hphoto.h"
#include "aver.h"
#include "colmas.h"
#include "grainvar.h"
#include "molcol.h"
#include "h21cm.h"
#include "mean.h"
#include "abrems.h"
#include "heat.h"
#include "struc.h"
#include "forlin.h"
#include "cmshft.h"
#include "radinc.h"

/*pnegopc punch negative opacities on io unit, iff 'set negopc' command was given */
static void pnegopc(void);

void RadInc(void)
{
#	if !defined(NDEBUG)
	int lgFlxNeg;
#	endif

	long int nelem,
		i, 
		ion,
	  ipZ,
	  nd;
	double avWeight,
	  DilutionHere ,
	  escatc, 
	  flx, 
	  opfac, 
	  relec, 
	  rforce, 
	  t;

	double ajmass, 
	  rjeans;

#	ifdef DEBUG_FUN
	fputs( "<+>RadInc()\n", debug_fp );
#	endif

	/* when this sub is called RADIUS is the outer edge of zone */

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, 
			" RadInc called; radius=%10.3e rinner=%10.3e DRAD=%10.3e drNext=%10.3e ROUTER=%10.3e DEPTH=%10.3e\n", 
		  radius.Radius, radius.rinner, radius.drad, radius.drNext, 
		  radius.router[iteration-1], radius.depth );
	}

	/* check cooling time for this zone, remember longest */
	timesc.ttherm = MAX2(timesc.ttherm,1.5*phycon.pden*BOLTZMANN*phycon.te/
	  cooling.ctot);

	/* H 21 cm equilibrium timescale */
	t = H21cm( phycon.te )* xIonFracs[ipHYDROGEN][1];
	if( t > SMALLFLOAT )
	{
		timesc.TimeH21cm = MAX2( 1./t, timesc.TimeH21cm );
	}

	/* remember longest CO timescale */
	if( xIonFracs[1][5]*xIonFracs[1][7] > SMALLFLOAT )
	{
		double timemin;
		timemin = MIN2(timesc.AgeCOMoleDest ,
			timesc.AgeCOMoleDest* hevmolec.hevmol[IPCO-1]/
			MAX2(SMALLFLOAT,xIonFracs[1][5]*xIonFracs[1][7]) );
		/* this is rate CO is destroyed, equal to formation rate in equilibrium */
		timesc.BigCOMoleForm = MAX2(  timesc.BigCOMoleForm , timemin );
			/*timesc.AgeCOMoleDest* hevmolec.hevmol[IPCO-1]/
			(xIonFracs[1][5]*xIonFracs[1][2]) );*/
	}

	/* remember longest H2 timescale */
	if( (double)xIonFracs[1][0]*(double)xIonFracs[1][0] > SMALLFLOAT )
	{
		double timemin;
		timemin = MIN2(timesc.AgeH2MoleDest ,
			timesc.AgeH2MoleDest*hmi.htwo /((double)xIonFracs[1][0]*(double)xIonFracs[1][0]));
		timesc.BigH2MoleForm = MAX2(timesc.BigH2MoleForm, timemin );
			/*timesc.AgeH2MoleDest*hmi.htwo /(xIonFracs[1][0]*xIonFracs[1][0])));*/
	}

	/* increment counter if this zone possibly thermally unstable
	 * this flag was set in ionte, deriv of heating and cooling negative */
	if( thermal.lgUnstable )
	{
		thermal.nUnstable += 1;
	}

	/* find Stromgren radius */
	if( !uspher.lgUSphON && h.hi/phycon.hden > 0.49 )
	{
		uspher.rstrom = (float)radius.Radius;
		uspher.lgUSphON = TRUE;
	}

	/* ratio of inner to outer radii, at this point
	 * radius is the outer radius of this zone */
	DilutionHere = POW2((radius.Radius - radius.drad*radius.dRadSign)/
	  radius.Radius);

	relec = 0.;
	rforce = 0.;

	if( trace.lgTrace && trace.lgConBug )
	{
		fprintf( ioQQQ, " Energy, flux, OTS:\n" );
		for( i=0; i < rfield.nflux; i++ )
		{
			fprintf( ioQQQ, "%6ld%10.2e%10.2e%10.2e", i, rfield.anu[i], 
			  rfield.flux[i] + rfield.outlin[i] + rfield.ConOutInter[i]+rfield.ConOutRecInter[i], 
			  rfield.otscon[i] + rfield.otslin[i] );
		}
		fprintf( ioQQQ, "\n" );
	}

	/* diffuse continua factor
	 * if lgSphere then all diffuse radiation is outward (COVRT=1)
	 * lgSphere not set then COVRT=0.0 */

	/* begin sanity check */
	/* only do this test in debug mode */
#	if !defined(NDEBUG)
	lgFlxNeg = FALSE;
	for( i=0; i < rfield.nflux; i++ )
	{
		if( rfield.flux[i] < 0. )
		{
			fprintf( ioQQQ, " RadInc finds negative intensity in flux.\n" );
			fprintf( ioQQQ, " Intensity, frequency, pointer=%11.3e%11.3e%6ld\n", 
			  rfield.flux[i], rfield.anu[i], i );
			lgFlxNeg = TRUE;
		}
		if( rfield.otscon[i] < 0. )
		{
			fprintf( ioQQQ, " RadInc finds negative intensity in otscon.\n" );
			fprintf( ioQQQ, " Intensity, frequency, pointer=%11.3e%11.3e%6ld\n", 
			  rfield.otscon[i], rfield.anu[i], i );
			lgFlxNeg = TRUE;
		}
		if( opac.tmn[i] < 0. )
		{
			fprintf( ioQQQ, " RadInc finds negative tmn.\n" );
			fprintf( ioQQQ, " value, frequency, pointer=%11.3e%11.3e%6ld %4.4s\n", 
			  opac.tmn[i], rfield.anu[i], i, rfield.chLineLabel[i] );
			lgFlxNeg = TRUE;
		}
		if( rfield.otslin[i] < 0. )
		{
			fprintf( ioQQQ, " RadInc finds negative intensity in otslin.\n" );
			fprintf( ioQQQ, " Intensity, frequency, pointer=%11.3e%11.3e%6ld %4.4s\n", 
			  rfield.otslin[i], rfield.anu[i], i, rfield.chLineLabel[i]  );
			lgFlxNeg = TRUE;
		}
		if( rfield.outlin[i] < 0. )
		{
			fprintf( ioQQQ, " RadInc finds negative intensity in outlin.\n" );
			fprintf( ioQQQ, " Intensity, frequency, pointer=%11.3e%11.3e%6ld %4.4s\n", 
			  rfield.outlin[i], rfield.anu[i], i, rfield.chLineLabel[i]  );
			lgFlxNeg = TRUE;
		}
		if( rfield.ConOutInter[i]+rfield.ConOutRecInter[i] < 0. )
		{
			fprintf( ioQQQ, " RadInc finds negative intensity in ConOutInter.\n" );
			fprintf( ioQQQ, " Intensity, frequency, pointer=%11.3e%11.3e%6ld %4.4s\n", 
			  rfield.ConOutInter[i]+rfield.ConOutRecInter[i], rfield.anu[i], i, rfield.chContLabel[i]  );
			lgFlxNeg = TRUE;
		}
		if( opac.opac[i] < 0. )
		{
			opac.lgOpacNeg = TRUE;
			/* this sub will punch negative opacities on io unit,
			 * iff 'set negopc' command was given */
			pnegopc();
		}
	}
	if( lgFlxNeg )
	{
		fprintf( ioQQQ, " Insanity has occurred, this is zone%4ld\n", 
		  nzone );
		ShowMe();
		puts( "[Stop in RadInc]" );
		cdEXIT(1);
	}
	/*end sanity check*/
#	endif

	/* attenuate the flux array after doing radiative acceleration */
	escatc = 6.65e-25*phycon.eden;

	for( i=0; i < rfield.nflux; i++ )
	{
		/* sum total continuous optical depths */
		opac.TauAbsGeo[0][i] += (float)(opac.opac[i]*radius.dReff);
		opac.TauScatGeo[0][i] += (float)(opac.scatop[i]*radius.dReff);

		/* following only optical depth to illuminated face */
		opac.TauAbsFace[i] += (float)(opac.opac[i]*radius.dReff);
		opac.TauScatFace[i] += (float)(opac.scatop[i]*radius.dReff);

		/* these are total in inward direction, large if sherical */
		opac.TauTotalGeo[0][i] = opac.TauAbsGeo[0][i] + opac.TauScatGeo[0][i];

		/* TMN is array of scale factors which account for attenuation
		 * of continuum across the zone (necessary to conserve energy
		 * at the 1% - 5% level.) sphere effects in
		 * drNext was set by NEXTDR and will be next dr */
		/* compute both total and thompson scat rad acceleration */
		rforce += (rfield.flux[i] + rfield.outlin[i] + rfield.ConOutInter[i] +rfield.ConOutRecInter[i]+ 
		  rfield.ConOutNoInter[i])*rfield.anu[i]*(opac.opac[i] + 
		  opac.scatop[i]);

		relec += ((rfield.flux[i] + rfield.outlin[i] + rfield.ConOutInter[i] +rfield.ConOutRecInter[i]+ 
		  rfield.ConOutNoInter[i])*escatc)*rfield.anu[i];

		/* attenuation of flux by optical depths IN THIS ZONE 
		 * AngleIllum is 1/COS(theta), is usually 1, reset with illuminate command,
		 * option for illumination of slab at an angle */
		opac.ExpZone[i] = sexp(opac.opac[i]*radius.dReff*Angllum.AngleIllum);

		/* e(-tau) in inward direction, up to illuminated face */
		opac.ExpmTau[i] *= (float)opac.ExpZone[i];

		/* e2(tau) in inward direction, up to illuminated face */
		opac.e2TauAbs[i] = (float)e2(opac.TauAbsFace[i],opac.ExpmTau[i]);
		assert( opac.e2TauAbs[i] <= 1. );

		/* DilutionHere is square of ratio of inner to outer radius */
		opfac = opac.ExpZone[i]*DilutionHere;

		rfield.flux[i] *= (float)opfac;

		/* this simulates the behavior in C90 and before,
		 * first attenuate flux acculumated so far, then
		 * add on flux from this zone */
		rfield.ConOutNoInter[i] *= (float)opfac;
		rfield.ConOutNoInter[i] += rfield.ConLocNoInter[i]*(float)radius.dVolOutwrd*opac.tmn[i]/**/;

		/* outward lines and continua */
		rfield.ConOutInter[i] *= (float)opfac;
		rfield.outlin[i] *= (float)opfac;
		rfield.ConOutRecInter[i] *= (float)opfac;

		/* set occupation numbers, first attenuated incident continuum */
		rfield.OccNumbIncidCont[i] = rfield.flux[i]*rfield.convoc[i];

		/* >>chng 00 oct 03, add diffuse continua */
		/* local diffuse continua */
		rfield.OccNumbDiffCont[i] = (rfield.ConLocInter[i] + rfield.ConLocNoInter[i])*rfield.convoc[i];

	}

	/* begin sanity check, compare total Lyman continuum optical depth 
	 * with amount of extinction there */

	/* this is amount continuum attenuated to illuminated face, 
	 * but only do test if flux positive, not counting scattering opacity,
	 * and correction for spherical dilution not important */
	if( (rfield.flux[hydro.ipHLevNIonCon[0][0]-1]/MAX2(SMALLFLOAT,rfield.FluxSave[hydro.ipHLevNIonCon[0][0]-1]) ) > SMALLFLOAT && 
		!opac.lgScatON &&
		radius.depth/radius.Radius < 1e-4 )
	{
		/* ratio of current to incident continuum, converted to optical depth */
		/* >>chng 99 apr 23, this crashed on alpha due to underflow to zero in argy
		 * to log.  defended two tways - above checks that ratio of fluxes is large enough, 
		 * and here convert to double.
		 * error found by Peter van Hoof */
		flx = -log((double)rfield.flux[hydro.ipHLevNIonCon[0][0]-1]/
			(double)opac.tmn[hydro.ipHLevNIonCon[0][0]-1]/
			(double)rfield.FluxSave[hydro.ipHLevNIonCon[0][0]-1]);

		/* this is computed absorption optical depth to illuminated face */
		t = opac.TauAbsFace[hydro.ipHLevNIonCon[0][0]-1] ;

		/* first test is relative error, second is to absolute error and comes
		 * in for very small tau, where differences are in the round-off */
		if( fabs( flx - t ) / t > 0.01 && 
			/* for very small inner optical depths, the tmn correction is major,
			 * and this test is not precise */
			fabs(flx-t)>MAX2(0.001,1.-opac.tmn[hydro.ipHLevNIonCon[0][0]-1]) )
		{
			/* in print below must add extra HI col den since this will later be
			 * incremented in RTOptDepthIncre */
			fprintf( ioQQQ,
				" RadInc lyman continuum insanity zone %li, attenuat, atomic taus=%g %g H=%g\n",
				nzone, flx , t ,
				6.327e-18*(h.hi*radius.dReff+coldenCom.colden[IPCHI-1]) );
			insane();
		}
	}
	/* end sanity check */

	/* do scattering opacity, intensity goes as 1/(1+tau) */
	if( opac.lgScatON )
	{
		for( i=0; i < rfield.nflux; i++ )
		{
			/* assume half forward scattering
			 * opfac = 1. / ( 1. + dReff*0.5 * scatop(i) )
			 * >>chng 97 apr 25, remove .5 to get agreement with
			 * Lightman and White formula */
			opfac = 1./(1. + radius.dReff*opac.scatop[i]);
			rfield.flux[i] *= (float)opfac;
			rfield.ConOutNoInter[i] *= (float)opfac;
			rfield.ConOutInter[i] *= (float)opfac;
			rfield.ConOutRecInter[i] *= (float)opfac;
			rfield.outlin[i] *= (float)opfac;
		}
	}

	/* now do slight reshuffling of energy due to compton scattering */
	cmshft();

	relec *= EN1RYD;

	/* radiative acceleration; xMassDensity is gm per cc, eval when PTOT called */
	t = 1./SPEEDLIGHT/phycon.xMassDensity;
	wind.AccelLine = (float)(forlin()*t);
	wind.AccelCont = (float)(rforce*EN1RYD*t);
	wind.AccelTot = wind.AccelCont + wind.AccelLine;
	wind.AccelMax = (float)MAX2(wind.AccelMax,wind.AccelTot);
	/* this shows that AccelLine becomes unphysicsl, presumaby due to forlin
	 * fprintf(ioQQQ," wind te %e  %e %e %e\n", phycon.te,	wind.AccelLine ,
		wind.AccelCont ,wind.AccelTot );*/

	/* force multiplier; RELEC*T can be zero for very low densities */
	if( relec*t > SMALLFLOAT )
	{
		wind.fmul = (float)((wind.AccelLine + wind.AccelCont)/(relec*t));
	}
	else
	{
		wind.fmul = 0.;
	}

	/* following is integral of radiative force */
	pressure.pinzon = (float)(wind.AccelTot*phycon.xMassDensity*radius.dReff*Angllum.AngleIllum);
	pressure.PresInteg += pressure.pinzon;

	/* sound is sound travel time, sqrt term is sound speed */
	timesc.sound += radius.dReff/sqrt(pressure.GasPres/phycon.xMassDensity);

	/* attenuate neutrons if they are present */
	if( neutrn.lgNeutrnHeatOn )
	{
		/* correct for optical depth effects */
		neutrn.totneu *= (float)sexp(crsnut.CrsSecNeutron*phycon.hden*radius.dReff*Angllum.AngleIllum);
		/* correct for spherical effects */
		neutrn.totneu *= (float)DilutionHere;
	}

	/* following radiation factors are extinguished by 1/r**2 and e- opacity
	 * also bound electrons */
	
	/* do all emergent spectrum from illuminated face if model is NOT spherical */
	if( !sphere.lgSphere )
	{
		double Reflec_Diffuse_Cont;

		for( i=0; i < rfield.nflux; i++ )
		{
			if( opac.TauAbsGeo[0][i] < 30. )
			{
				/* ConLocInter is diffuse emission per unit vol, fill fac
				 * the 1/2 comes from isotropic emission
				 * scatsv(i) = (flux(i)+ConOutInter(i))*0.5*scatop(i)
				 * >>chng 97 apr 25, remove fac of 2 to
				 * get agreement wth lightman&white
				 * Reflec_Diffuse_Cont = (ConLocInter(i)+scatsv(i)) * dReff *e2TauAbs(i)
				 * >>chng 97 may 08, needed 1/2 in front of ConLocInter 
				 * had been there, but also with scatsv, and lost both
				 * when /2 removed to get agreement with lightman and white */
				/* >>chng 99 dec 21, add ConLocNoInter to account for rec continua*/
				Reflec_Diffuse_Cont = (rfield.ConLocInter[i]/2. + rfield.ConLocNoInter[i]/2.)*
					radius.dReff * opac.e2TauAbs[i];

				/* Reflec_Diffuse_Cont = (ConLocInter(i)+scatsv(i)) * dReff/2. *e2TauAbs(i)
				 * ConRefDiff - reflected diffuse continuum */
				rfield.ConRefDiff[i] += (float)(Reflec_Diffuse_Cont*radius.r1r0sq);

				/* the reflected part of the incident continuum */
				rfield.ConRefIncid[i] += (float)(rfield.flux[i]*opac.scatop[i]*
				  radius.dReff*opac.e2TauAbs[i]*radius.r1r0sq);

				/* >>chng 97 apr 25, remove fac of 2 to get
				 * agreement wth lightman&white */
				rfield.reflin[i] += (float)(rfield.outlin[i]*opac.scatop[i]*
				  radius.dReff*opac.e2TauAbs[i]*radius.r1r0sq);
			}
		}
	}

	hydro.FreeFreeTotHeat += hydro.FreeFreeHeat*(float)radius.dVeff;


	/* following is general method to find means weighted by various functions
	 * called in StartIter to initialize to zero, called here to put in numbers
	 * results will be weighted by radius and volumn
	 * this is the only place things must be entered to create averages */
	aver("zone",1.,1.,"          ");
	aver("doit",phycon.te,1.,"    Te    ");
	aver("doit",phycon.te,phycon.eden,"  Te(Ne)  ");
	aver("doit",phycon.te,phycon.eden*xIonFracs[2][ipHYDROGEN]," Te(NeNp) ");
	aver("doit",phycon.te,phycon.eden*xIonFracs[2][ipHELIUM]," Te(NeHe+)");
	aver("doit",phycon.te,phycon.eden*xIonFracs[3][ipHELIUM],"Te(NeHe2+)");
	aver("doit",phycon.te,phycon.eden*xIonFracs[2][ipOXYGEN]," Te(NeO+) " );
	aver("doit",phycon.te,phycon.eden*xIonFracs[3][ipOXYGEN]," Te(NeO2+)");
	aver("doit",phycon.hden,1.,"    NH    ");
	aver("doit",phycon.eden,xIonFracs[3][7],"  Ne(O2+) ");
	aver("doit",phycon.eden,h.hii,"   Ne(Np) ");

	/* save information about structure of model, now used to get t^2 */
	nd = MIN2(nzone , NZLIM)-1;
	struc.testr[nd] = phycon.te;
	/* total grams per unit vol */
	struc.pdenstr[nd] = phycon.xMassDensity;
	struc.heatstr[nd] = heat.htot;
	struc.coolstr[nd] = cooling.ctot;
	struc.volstr[nd] = (float)radius.dVeff;
	struc.radstr[nd] = (float)radius.dReff;
	struc.histr[nd] = xIonFracs[1][ipHYDROGEN];
	struc.hiistr[nd] = xIonFracs[2][ipHYDROGEN];
	struc.ednstr[nd] = (float)phycon.eden;
	struc.o3str[nd] = xIonFracs[3][ipOXYGEN];
	struc.DenMass[nd] = phycon.xMassDensity;
	struc.pressure[nd] = pressure.pnow;
	struc.depth[nd] = (float)radius.depth;
	/* save absorption optical depth from illuminated face to current position */
	struc.xLyman_depth[nd] = opac.TauAbsFace[hydro.ipHLevNIonCon[ipHYDROGEN][ipH1s]];
	for( nelem=ipHYDROGEN; nelem<=ipZINC; ++nelem)
	{
		for( ion=0; ion<=nelem+2 ; ++ion )
		{
			struc.xIonFracs[nd][ion][nelem] = xIonFracs[ion][nelem];
		}
	}

	/* this is the sum of the enthalph and the kinetic energy per gram */
	struc.work[nd] = (float)(
		phycon.EnergyIonization/phycon.xMassDensity +
		0.5*POW2(dynamics.Velocity)+
		/* thermal plus PdV work term */
		5./2.*pressure.GasPres/phycon.xMassDensity);

	abrems.dlnenp += phycon.eden*(double)(h.hii)*radius.dReff;

	/* counter for number of times Lya excit temp hotter than gas */
	if( hydro.TexcLya > phycon.te )
	{
		hydro.nLyaHot += 1;
		if( hydro.TexcLya > hydro.TLyaMax )
		{
			hydro.TLyaMax = hydro.TexcLya;
			hydro.TeLyaMax = phycon.te;
			hydro.nZTLaMax = nzone;
		}
	}

	coldenCom.colden[IPCOLUMNDENSITY-1] += (float)(phycon.hden*radius.dReff);
	coldenCom.colden[IPCHMIN-1] += hmi.hminus*(float)radius.dReff;
	coldenCom.colden[IPCOLH2-1] += hmi.htwo*(float)radius.dReff;
	coldenCom.colden[IPCHEHP-1] += hmi.hehp*(float)radius.dReff;
	coldenCom.colden[IPCH2PLS-1] += hmi.h2plus*(float)radius.dReff;
	coldenCom.colden[IPCHII-1] += h.hii*(float)radius.dReff;
	coldenCom.colden[IPCHI-1] += h.hi*(float)radius.dReff;

	/* now add total molecular column densities */
	molcol("ADD ");

	/* increment forming the mean ionizatin and temperature */
	MeanInc();

	/*-----------------------------------------------------------------------*/

	gv.reftot += (float)radius.dReff;

	/* calculate average atomic weight per hydrogen of the plasma */
	avWeight = 0.;
	for( ipZ=0; ipZ < LIMELM; ipZ++ ) {
		avWeight += xIonFracs[0][ipZ]*AtomcWgt.AtomicWeight[ipZ];
	}
	avWeight /= xIonFracs[0][0];

	for( nd=0; nd < gv.nBin; nd++ )
	{
		gv.bin[nd]->avdust += gv.bin[nd]->tedust*(float)radius.dReff;
		gv.bin[nd]->avdft += gv.bin[nd]->DustDftVel*(float)radius.dReff;
		gv.bin[nd]->avdpot += (float)(gv.bin[nd]->dstpot*EVRYD*radius.dReff);
		gv.bin[nd]->avDGRatio += (float)(gv.bin[nd]->dustp[1]*gv.bin[nd]->dustp[2]*
			gv.bin[nd]->dustp[3]*gv.bin[nd]->dustp[4]*gv.bin[nd]->dstAbund/avWeight*
			radius.dReff);
	}

	colmas.TotMassColl += phycon.xMassDensity*(float)radius.dReff;
	colmas.tmas += phycon.te*phycon.xMassDensity*(float)radius.dReff;
	colmas.wmas += phycon.wmole*phycon.xMassDensity*(float)radius.dReff;

	/* now find minimum Jeans length and mass; length in cm */
	rjeans = 7.79637 + (phycon.alogte - log10(phycon.wmole) - log10(phycon.xMassDensity*
	  filfac.FillFac))/2.;

	/* minimum Jeans mass in gm */
	ajmass = 3.*(rjeans - 0.30103) + log10(4.*PI/3.*phycon.xMassDensity*
	  filfac.FillFac);

	/* now remember smallest */
	jmin.rjnmin = MIN2(jmin.rjnmin,(float)rjeans);
	jmin.ajmmin = MIN2(jmin.ajmmin,(float)ajmass);

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, " RadInc returns\n" );
	}

#	ifdef DEBUG_FUN
	fputs( " <->RadInc()\n", debug_fp );
#	endif
	return;
}

/*pnegopc punch negative opacities on io unit, iff 'set negopc' command was given */
static void pnegopc(void)
{
	long int i;
	FILE *ioFile;

#	ifdef DEBUG_FUN
	fputs( "<+>pnegopc()\n", debug_fp );
#	endif

	if( opac.lgNegOpacIO )
	{
		/* option to punch negative opacities */
		if( NULL==(ioFile=fopen("negopc.txt","w")) )
		{
			fprintf( ioQQQ,"pnegopc could not open negopc.txt for writing\n");
			puts( "[Stop in opac0]" );
			cdEXIT(1);
		}
		for( i=0; i < rfield.nflux; i++ )
		{
			fprintf( ioFile, "%10.2e %10.2e \n", rfield.anu[i], 
			  opac.opac[i] );
		}
		fclose( ioFile);
	}


#	ifdef DEBUG_FUN
	fputs( " <->pnegopc()\n", debug_fp );
#	endif
	return;
}
/* Note - when this file is compiled in fast optimized mode,
 * a good compiler will complain that the routine pnegopc is a
 * static, local routine, but that it has not been used.  It is
 * indeed used , but only when NDEBUG is not set, so it is only
 * used when the code is compiled in debug mode.  So this is
 * not a problem. */
