/* rfield.h */
EXTERN struct t_rfield {

	/* ================================================================================= */
	/* the following define the continuum energy scale and its limits */

	/* nflux is number of continuum points needed to get to high energy
	 * end of this continuum.  this is initally set to nupper in ContCreatePointers,
	 * and then trimed down in ContSetIntensity.  It is finally reset in ConvInitTemp,
	 * to make sure continuum includes all possible line and continuum emission
	 * produced by the code */
	long int nflux;

	/* number of frequency cells needed to get full energy resolution of code */
	long int nupper;

	/* the energy of the lower limit of the continuum */
	float emm;

	/* the energy of the upper limit of the continuum */
	float egamry;

	/*faintest high energy flux to consider, set with set flxfnt command */
	float FluxFaint;

	/* energy in Ryd of center of cell */
	float anu[NCELL];

	/* original set of energy units, always the same no matter what the
	 * shape of the continuum - actual energy units change with shape */
	float AnuOrg[NCELL];

	/* width of cells in ryd */
	float widflx[NCELL];

	/* these are log, sqrt, square, and cube of anu array */
	float anulog[NCELL], 
	  anusqr[NCELL], 
	  anu2[NCELL], 
	  anu3[NCELL];

	/* ================================================================================= */
	/* the following are the arrays containing the local radiation field */

	/*flux is photons per cell ND widthd of cells vary*/
	float flux[NCELL];

	/* convoc is the conversion factor from rfield to OccNumbIncidCont */
	float convoc[NCELL];

	/* OccNumbIncidCont is the continuum occupation number, attenuated incident ONLY */
	float OccNumbIncidCont[NCELL];

	/* OccNumbDiffCont is the continuum occupation number, for local diffuse continuum */
	float OccNumbDiffCont[NCELL];

	/* array of Boltzmann factors for the continuum energy grid and current temperature */
	/* >>>chng 00 apr 04 from float to double so that corr for stim emis H- not go negative */
	double ContBoltz[NCELL];

	/* ConLocInter is local diffuse continuum,
	 * for current zone, evaluated in RTDiffuse */
	float ConLocInter[NCELL];

	/* ConLocNoInter is local diffuse continuum,
	 * evaluated in RTDiffuse but does not interact
	 * contains heavy recombination continua, */
	float ConLocNoInter[NCELL];

	/* ContNoInter diffuse continuum, per unit area (not vol), 
	 * not counted in internal energy budget (handled other ways),
	 * outward and reflected */
	float ConOutNoInter[NCELL];

	/* this is summed grnd state rec continua that does not appear in plots,
	 * but does interact with gas */
	float ConOutRecInter[NCELL];

	/* this is set in RTDiffuse and carries interactive continua */
	float ConOutInter[NCELL];

	/* ConRefIncid is reflected portion of incident continuum */
	float ConRefIncid[NCELL];

	/* reflected diffuse emission continuum */
	float ConRefDiff[NCELL];

	/* this is the spectrum of continuum thrown into the outward beam,
	 * this is only used in metdif and makediffuse, remove after combining
	 * these routines */
	float ThrowOut[NCELL];

	float SummedCon[NCELL];
	float SummedDif[NCELL];
	float SummedOcc[NCELL];

	/* this will contail array of locally destroyed continuum photons,
	 * zeroed and evaluated in RTDiffuse, currently only two-photon */
	float ConOTS_local_photons[NCELL],
		/* the local photoionization rate corresponding to above photons */
		ConOTS_local_OTS_rate[NCELL];

	/* otsline and ots con - local ots fields for line and continua
	 * outlin outward line fields */
	float 
		/* the local ots line rates */
		otslin[NCELL], 
		/* the local ots continuum rates */
		otscon[NCELL], 

		/* these two are the new rates, that will be entered into the above once
		 * RTOTSUpdata has been called */
		 otslinNew[NCELL],
		 otsconNew[NCELL],

		outlin[NCELL], 
		otssav[2][NCELL];

	/* reflected line  */
	float reflin[NCELL];

	/* save incident continuum for later iterations */
	float FluxSave[NCELL];

	/* variables related to deriving the ratio of outward to incident continuum */
	/* SumHOutCon is H-ionizing only*/
	float SumOutCon, 
	  SumIncCon, 
	  /* sumOutCon is summed out outlin and ConOutInter less SavOutCon for all energy*/
	  SavOutCon[NCELL], 
	  SumHOutCon, 
	  SumHIncCon, 
	  /* SumOutMax - energy where max occurs, ipSumOutMax pointer to this*/
	  SumOutMax;

	long int ipSumOutMax;

	/* this is zero or one depending whether pumping by diffuse fields
	 * is turned on (1) or turned off (0) */
	float DiffPumpOn;

	/* string identifing the first line that occured at this energy */
	char chLineLabel[NCELL][5];

	/* string identifing the first continuum edge that occured at this energy */
	char chContLabel[NCELL][5];

	}	rfield;

