/*tfidle update some temperature dependent variables */
/*tauff compute optical depth where cloud is thin to free-free and plasma freq */
#include "cddefines.h"
#include "cooling.h"
#include "tff.h"
#include "opacity.h"
#include "plasnu.h"
#include "physconst.h"
#include "phycon.h"
#include "tgff.h"
#include "gffsub.h"
#include "hydrogenic.h"
#include "maxcor.h"
#include "gaunts.h"
#include "trace.h"
#include "rfield.h"
#include "h.h"
#include "tfidle.h"

/*tauff compute optical depth where cloud is thin to free-free and plasma freq */
static void tauff(void);

void tfidle(void)
{
	static float ttused = 0.f;
	static double edused = 0.;
	static int lgZLogSet = FALSE;
	int lgGauntF[2];
	long int n, 
	  i, 
	  if1;
	double fac, 
	  fanu;

#	ifdef DEBUG_FUN
	fputs( "<+>tfidle()\n", debug_fp );
#	endif

	/* check that eden not negative */
	if( phycon.eden <= 0. )
	{
		fprintf( ioQQQ, " I found a zero or negative electron density,%10.2e\n", 
		  phycon.eden );
		puts( "[Stop in tfidle]" );
		ShowMe();
		puts( "[Stop in tfidle]" );
		cdEXIT(1);
	}

	/* check that temperature not negative */
	if( phycon.te <= 0. )
	{
		fprintf( ioQQQ, " I found a zero or negative electron temperature,%10.2e\n", 
		  phycon.te );
		puts( "[Stop in tfidle]" );
		ShowMe();
		puts( "[Stop in tfidle]" );
		cdEXIT(1);
	}

	/* only time only, set up array of logs of charge squared */
	if( !lgZLogSet )
	{
		for( i=0; i<LIMELM; ++i )
		{
			/* this array is used to modify the log temperature array
			 * defined below, for hydrogenic species of charge i+1 */
			phycon.sqlogz[i] = (float)log10( POW2(i+1.) );
		}
		lgZLogSet = TRUE;
	}

	if( phycon.te != ttused )
	{
		ttused = phycon.te;
		/* current temperature in eV */
		phycon.te_eV = phycon.te/(float)EVDEGK;
		phycon.tesqrd = POW2(phycon.te);
		phycon.sqrte = (float)sqrt(phycon.te);
		cooling.halfte = (float)(0.5/phycon.te);
		cooling.tsq1 = 1.f/phycon.tesqrd;
		phycon.te32 = phycon.te*phycon.sqrte;
		phycon.te70 = (float)pow(phycon.te,0.70);
		phycon.te30 = (float)pow(phycon.te,0.30);
		phycon.te20 = (float)pow(phycon.te,0.20);
		phycon.te10 = (float)pow(phycon.te,0.10);
		phycon.te05 = (float)pow(phycon.te,0.05);
		phycon.te03 = (float)pow(phycon.te,0.03);
		phycon.te02 = (float)pow(phycon.te,0.02);
		phycon.te01 = (float)pow(phycon.te,0.01);
		phycon.te005 = (float)pow(phycon.te,0.005);
		phycon.te003 = (float)pow(phycon.te,0.003);
		phycon.te001 = (float)pow(phycon.te,0.001);
		phycon.teinv = 1.f/phycon.te;
		phycon.alogte = (float)log10(phycon.te);
		phycon.alogete = (float)log(phycon.te);
		phycon.telogn[0] = phycon.alogte;

		for( i=1; i < 7; i++ )
		{
			phycon.telogn[i] = phycon.telogn[i-1]*phycon.telogn[0];
		}
	}

	/* term with hi added June 4, 93, to account for warm pdr */
	phycon.edensqte = (float)((phycon.eden + h.hi/1e4)/phycon.sqrte);
	phycon.cdsqte = (float)(phycon.edensqte*8.629e-6);

	/* >>>chng 99 nov 23, removed this line, so back to old method of h coll */
	/* used for hydrogenic collisions */
	phycon.EdenHCorr = (float)(phycon.eden + h.hi*1.7e-4);


	/* phycon.ElecFrac is electron fraction, used for secondary ionization efficiency */
	phycon.ElecFrac = (float)(phycon.eden/phycon.TotalNuclei);

	phycon.alogne = (float)log10(phycon.eden);

	if( phycon.eden != edused )
	{
		edused = phycon.eden;
		phycon.SqrtEden = (float)sqrt(phycon.eden);
	}

	/* correction factors for induced recombination, 
	 * also used as Boltzmann factors
	 * check for zero is because ContBoltz is zeroed out in initizationation
	 * of code, its possible this is a constant density grid of models
	 * in which the code is called as a subroutine */
	if( tgff.tgffused != phycon.te || rfield.ContBoltz[0] <= 0. )
	{
		tgff.tgffused = phycon.te;
		fac = TE1RYD/phycon.te;
		i = 0;
		fanu = fac*rfield.anu[i];
		/* NB - the 84 in the following must be kept parallel with the 84 in sexp,
		 * since level2 uses ContBoltz to see whether the rates will be significant.
		 * If the numbers did not agree then this test would be flawed, resulting in
		 * div by zero */
		while( i < rfield.nupper && fanu < 84. )
		{
			rfield.ContBoltz[i] = exp(-fanu);
			++i;
			/* this is boltz factor for NEXT cell */
			fanu = fac*rfield.anu[i];
		}
		/* ipMaxBolt is number of cells defined, so defined up through ipMaxBolt-1 */
		maxcor.ipMaxBolt = i;

		/* zero out remainder */
		/* >>chng 01 apr 14, upper limit has been ipMaxBolt+1, off by one */
		for( i=maxcor.ipMaxBolt; i < rfield.nupper; i++ )
		{
			rfield.ContBoltz[i] = 0.;
		}
	}

	/* find frequency where thin to brems or plasma frequency */
	tauff();

	if( fabs(1.-tgff.tgffsued2/phycon.te) > 0.10 )
	{
		gffsub(0.,rfield.anulog,gaunts.gff,1,rfield.nflux,&if1);

		/* follwoing for ionized helium */
		gffsub(0.30103,rfield.anulog,gaunts.gffhe2,1,rfield.nflux,
		  &if1);

		tgff.tgffsued2 = phycon.te;
		lgGauntF[0] = TRUE;
	}
	else
	{
		/* this is flag that would have been set in gffsub, and
		 * printed in debug statement below.  We are not evaluating
		 * so set to -1 */
		if1 = -1;
		lgGauntF[0] = FALSE;
	}

	if( trace.lgTrace && trace.lgTrGant )
	{
		fprintf( ioQQQ, "     BOLTGN; gaunt facs?" );
		for(n=0; n < 2; n++)
			fprintf( ioQQQ, "%2c", TorF(lgGauntF[n]) );

		fprintf( ioQQQ, "%2f g 1 2=%10.2e%9.2ld flag%2f guv(1,n)%10.2e\n", 
		  gaunts.gff[0], gaunts.gff[hydro.ipHLevNIonCon[0][2]-1], if1, gaunts.gff[hydro.ipHLevNIonCon[0][2]], 
		  gaunts.gff[rfield.nflux-1] );
	}

#	ifdef DEBUG_FUN
	fputs( " <->tfidle()\n", debug_fp );
#	endif
	return;
}

/*tauff compute optical depth where cloud is thin to free-free and plasma freq */
static void tauff(void)
{
	double fac;

#	ifdef DEBUG_FUN
	fputs( "<+>tauff()\n", debug_fp );
#	endif

	/* routine sets variable ntff, pointer to lowest cont cell that is thin */

	/* find frequency where continuum thin to free-free */
	while( tffCom.ntff < rfield.nflux && 
		opac.TauAbsGeo[0][tffCom.ntff-1] >= 1. )
	{
		tffCom.ntff += 1;
	}

	/* TFF will be frequency where goes thin to brems
	 * >>chng 96 may 7, had been 2, change as per Kevin Volk bug report */
	if( tffCom.ntff > 2 && opac.TauAbsGeo[0][tffCom.ntff-1] > 0.001 )
	{
		/* for case where plasma frequency important, tau can be zero */
		fac = (1. - opac.TauAbsGeo[0][tffCom.ntff-2])/(opac.TauAbsGeo[0][tffCom.ntff-1] - 
		  opac.TauAbsGeo[0][tffCom.ntff-2]);
		fac = MAX2(fac,0.);
		tffCom.tff = (float)(rfield.anu[tffCom.ntff-2] + rfield.widflx[tffCom.ntff-2]*
		  fac);
	}
	else
	{
		tffCom.tff = 0.;
	}

	/* now evaluate the plasma frequency */
	plasnu.plsfrq = (float)(2.729e-12*sqrt(phycon.eden*1.2));

	/* also remember the largest plasma frequency we encounter */
	plasnu.plsfrqmax = MAX2(plasnu.plsfrqmax, plasnu.plsfrq);

	if( plasnu.plsfrq > rfield.anu[0] )
	{
		plasnu.lgPlasNu = TRUE;
	}

	/* >>chng 96 jul 15, did not include plasma freq before
	 * function returns larger of these two frequencies */
	tffCom.tff = (float)MAX2(plasnu.plsfrq,tffCom.tff);

	/* now increment ntff still further, until above plasma frequency */
	while( tffCom.ntff < rfield.nflux && 
		rfield.anu[tffCom.ntff-1] <= tffCom.tff )
	{
		tffCom.ntff += 1;
	}

	/* save max plasma frequency for case where density changes */
	if( tffCom.tff == plasnu.plsfrq )
	{
		plasnu.ipPlasmax = MAX2(plasnu.ipPlasma,tffCom.ntff);
	}


#	ifdef DEBUG_FUN
	fputs( " <->tauff()\n", debug_fp );
#	endif
	return;
}

